(herald "Electronic Purchase with Money Order Protocol Variant"
  (bound 12) (limit 5000)
  (comment "This version includes account numbers in exchanges"))

(comment "CPSA 2.3.1")
(comment "All input read from epmo_acctnum.scm")
(comment "Step count limited to 5000")
(comment "Strand count bounded at 12")

(defprotocol epmo_acctnum basic
  (defrole bank
    (vars (b c m name) (acctnum text) (hash name) (nc nm nb price text))
    (trace (recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    (non-orig (privk hash))
    (annotations b
      (1
        (forall ((pm name))
          (implies
            (and (authtransfer c acctnum b price pm nm)
              (reqtransfer pm b price pm nm))
            (dotransfer acctnum b price pm nm))))
      (2
        (and (says c (authtransfer c acctnum b price m nm))
          (says m (reqtransfer m b price m nm))))))
  (defrole customer
    (vars (b c m hash name) (acctnum nb nc nm goods price text))
    (trace (send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    (non-orig (privk b) (privk hash))
    (uniq-orig nc)
    (annotations c
      (1
        (says m
          (implies
            (exists ((acctnum2 text))
              (dotransfer acctnum2 b price m nm)) (doship m goods c))))
      (3
        (says b
          (forall ((pm name))
            (implies
              (and (authtransfer c acctnum b price m nm)
                (reqtransfer pm b price pm nm))
              (dotransfer acctnum b price pm nm)))))
      (4 (authtransfer c acctnum b price m nm))))
  (defrole merchant
    (vars (b c m hash name) (nb nc nm goods price text))
    (trace (recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    (non-orig (privk hash))
    (uniq-orig nm)
    (annotations m
      (1
        (implies
          (exists ((acctnum2 text)) (dotransfer acctnum2 b price m nm))
          (doship m goods c)))
      (2
        (and
          (says b
            (forall ((pm name))
              (exists ((acctnum2 text))
                (implies
                  (and (authtransfer c acctnum2 b price m nm)
                    (reqtransfer pm b price pm nm))
                  (dotransfer acctnum2 b price pm nm)))))
          (says c
            (exists ((acctnum2 text))
              (authtransfer c acctnum2 b price m nm)))))
      (3 (and (reqtransfer m b price m nm) (doship m goods c))))))

(defskeleton epmo_acctnum
  (vars (nm nc nb goods price text) (b m c hash name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nc nb)
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m)))))
  (label 0)
  (unrealized (0 2))
  (origs (nm (0 1)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb goods price acctnum text) (b m c hash name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (precedes ((0 1) (1 0)) ((1 1) (0 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nc nb)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b)) (0 2))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))))
  (label 1)
  (parent 0)
  (unrealized (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm nm)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0))
  (uniq-orig nm nc nb)
  (operation nonce-test (added-strand customer 3) nm (1 0)
    (enc nc nm m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b-0)))))
  (label 2)
  (parent 1)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb goods price acctnum goods-0 text) (m c hash b name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nm)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b))
  (uniq-orig nm nc nb)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nm
    (1 0) (enc nc nm m price (pubk c))
    (enc c nc nm acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))))
  (label 3)
  (parent 2)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm nm)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((0 1) (3 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (1 0)) ((3 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nm nc nb)
  (operation nonce-test (added-strand bank 2) nm (1 0)
    (enc nc nm m price (pubk c))
    (enc c nc nm acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b-0))))
    ((recv (enc c nc nm acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 4)
  (parent 2)
  (unrealized (0 0) (0 2) (1 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb price acctnum goods text) (m c hash b name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b))
  (uniq-orig nm nc nb)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))))
  (label 5)
  (parent 3)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb goods price acctnum goods-0 nm-0 price-0 text)
    (m c hash b name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nm)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (3 0)) ((2 2) (1 0))
    ((3 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b))
  (uniq-orig nm nc nb nm-0)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c)))))
  (label 6)
  (parent 3)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm nm)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nm nc nb)
  (operation nonce-test (displaced 4 2 customer 3) nm (3 0)
    (enc nc nm m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b-0))))
    ((recv (enc c nc nm acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 7)
  (parent 4)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb price acctnum goods text) (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (2 1)) ((1 1) (2 3)) ((2 0) (0 0)) ((2 2) (1 0))
    ((2 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nm nc nb)
  (operation nonce-test (displaced 2 3 customer 5) nb (0 2)
    (enc "hash" c nc nb nm price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))))
  (label 8)
  (parent 5)
  (realized)
  (shape)
  (maps
    ((0)
      ((b b) (m m) (c c) (nm nm) (nc nc) (nb nb) (hash hash)
        (goods goods) (price price))))
  (origs (nc (2 0)) (nm (0 1)) (nb (1 1))))

(defskeleton epmo_acctnum
  (vars (nm nc nb price acctnum goods nm-0 price-0 text)
    (m c hash b name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (3 0)) ((2 2) (1 0))
    ((3 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b))
  (uniq-orig nm nc nb nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm-0 m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c)))))
  (label 9)
  (parent 6)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nm)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nm nc nb)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nm
    (1 0) (enc "hash" c nc nb-0 nm price (pubk hash-0))
    (enc nc nm m price (pubk c)) (enc c nc nm acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 10)
  (parent 7)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 11)
  (parent 7)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 4 3 bank 2) nm (1 0)
    (enc "hash" c nc nb-0 nm price (pubk hash-0))
    (enc nc nm m price (pubk c))
    (enc c nc nm acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 12)
  (parent 7)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb price nm-0 price-0 acctnum goods text)
    (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (0 0)) ((3 0) (2 0))
    ((3 2) (1 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nm nc nb nm-0)
  (operation nonce-test (displaced 2 4 customer 5) nb (0 2)
    (enc "hash" c nc nb nm price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))))
  (label 13)
  (parent 9)
  (seen 8)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nm nc nb)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 14)
  (parent 10)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb goods price acctnum goods-0 nb-0 nm-0 price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nm)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (4 0)) ((2 2) (3 0))
    ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nm nc nb nm-0)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c)))))
  (label 15)
  (parent 10)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 5 0 merchant 2) price (4 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 16)
  (parent 11)
  (unrealized (0 0) (0 2) (1 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 17)
  (parent 12)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 18)
  (parent 12)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 2 4 customer 5) nb-0 (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 19)
  (parent 12)
  (unrealized (0 0) (0 2) (1 0) (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb price nb-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nm nc nb)
  (operation nonce-test (displaced 2 4 customer 5) nb (0 2)
    (enc "hash" c nc nb nm price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))))
  (label 20)
  (parent 14)
  (seen 8)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb price acctnum goods nb-0 nm-0 price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (4 0)) ((2 2) (3 0))
    ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nm nc nb nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm-0 m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c)))))
  (label 21)
  (parent 15)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 22)
  (parent 16)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 23)
  (parent 16)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 24)
  (parent 16)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 2 4 customer 5) nb-0 (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 25)
  (parent 16)
  (unrealized (0 0) (0 2) (1 0) (3 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 26)
  (parent 16)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 27)
  (parent 17)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (4 0)) ((2 2) (3 0))
    ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 28)
  (parent 17)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 5 2 customer 3) nb-0 (4 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 29)
  (parent 18)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation encryption-test (displaced 4 2 bank 2)
    (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0)) (privk b-0))
    (3 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 30)
  (parent 19)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0)) (privk b-0))
    (3 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 31)
  (parent 19)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nc nb price nb-0 nm-0 price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (0 0))
    ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nm nc nb nm-0)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb nm price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))))
  (label 32)
  (parent 21)
  (seen 13)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 33)
  (parent 22)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (4 0)) ((2 2) (3 0))
    ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 34)
  (parent 22)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 5 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 35)
  (parent 23)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 36)
  (parent 23)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 5 3 bank 2) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c))))))
  (label 37)
  (parent 24)
  (unrealized (0 0) (0 2) (1 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 5 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 38)
  (parent 24)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 5 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 39)
  (parent 24)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 40)
  (parent 24)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nb price nb-0)
  (operation encryption-test (displaced 4 2 bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0)) (privk b-0))
    (3 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 41)
  (parent 25)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nb price nb-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0)) (privk b-0))
    (3 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 42)
  (parent 25)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 43)
  (parent 26)
  (seen 34)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 44)
  (parent 26)
  (seen 36)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 45)
  (parent 26)
  (seen 40 67)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 5 customer 5) nb-0 (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 46)
  (parent 26)
  (unrealized (0 0) (0 2) (1 0) (4 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 acctnum goods text) (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 2 4 customer 5) nb (0 2)
    (enc "hash" c nc nb nb-0 price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))))
  (label 47)
  (parent 27)
  (seen 8)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (4 0)) ((2 2) (3 0))
    ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 48)
  (parent 28)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 49)
  (parent 29)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 50)
  (parent 29)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 2 5 customer 5) nb-0 (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 51)
  (parent 29)
  (seen 30 31)
  (unrealized (0 0) (0 2) (1 0) (4 3))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 2 5 customer 5) nb-0 (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 52)
  (parent 29)
  (unrealized (0 0) (0 2) (1 0) (4 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0))))
  (label 53)
  (parent 30)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 54)
  (parent 30)
  (seen 88)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (4 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 55)
  (parent 30)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 5 3 customer 3) nb-0 (4 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 56)
  (parent 31)
  (seen 30)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 acctnum goods text) (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 2 4 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 57)
  (parent 33)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (4 0)) ((2 2) (3 0))
    ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 58)
  (parent 34)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 59)
  (parent 35)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 2 5 customer 5) nb-0 (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 60)
  (parent 35)
  (unrealized (0 0) (0 2) (1 0) (4 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 2 5 customer 5) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price))))
  (label 61)
  (parent 35)
  (unrealized (0 0) (0 2) (1 0) (4 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 62)
  (parent 36)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c))))))
  (label 63)
  (parent 37)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (3 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 64)
  (parent 37)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 2 5 customer 5) nb-0 (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 65)
  (parent 37)
  (unrealized (0 0) (0 2) (1 0) (4 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 2 5 customer 5) goods-0 (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 goods-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0))))
  (label 66)
  (parent 37)
  (unrealized (0 0) (0 2) (1 0) (4 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 67)
  (parent 37)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 68)
  (parent 38)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 69)
  (parent 38)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price acctnum)
  (operation nonce-test
    (contracted (b-0 b) (acctnum-0 acctnum) (hash-1 hash-0)
      (nb-0 goods-0)) price (1 0)
    (enc "hash" c acctnum goods-0 price price (pubk hash-0))
    (enc acctnum price m price (pubk c))
    (enc c acctnum price acctnum price (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 price price (pubk hash-0))
            (privk b)) (enc acctnum goods-0 (pubk c))))))
  (label 70)
  (parent 39)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 5 3 bank 2) price (1 0)
    (enc "hash" c acctnum-0 goods-0 price price (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 price price (pubk hash-0))
    (enc acctnum-0 price m price (pubk c))
    (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 71)
  (parent 39)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 6 3 bank 2) nc (4 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 72)
  (parent 40)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 73)
  (parent 40)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c)))))
  (label 74)
  (parent 40)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0))))
  (label 75)
  (parent 41)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 76)
  (parent 41)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 5 3 customer 3) price (4 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 77)
  (parent 42)
  (seen 41)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 78)
  (parent 43)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 79)
  (parent 44)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 80)
  (parent 45)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 81)
  (parent 45)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (displaced 5 2 bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 82)
  (parent 46)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 83)
  (parent 46)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (0 0))
    ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb nb-0 price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))))
  (label 84)
  (parent 48)
  (seen 13)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 85)
  (parent 49)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 86)
  (parent 49)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 0 merchant 2) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 87)
  (parent 50)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation encryption-test (displaced 5 2 bank 2)
    (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 88)
  (parent 52)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation encryption-test (displaced 5 3 bank 2)
    (enc (enc "hash" c nc nb-1 nb-1 price (pubk hash-1)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 89)
  (parent 52)
  (seen 53 54 55)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 90)
  (parent 52)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0))))
  (label 91)
  (parent 53)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (4 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 92)
  (parent 53)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (4 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 93)
  (parent 55)
  (seen 91 92)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 0) (4 0)) ((3 2) (2 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 94)
  (parent 55)
  (seen 160)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (4 0)
    (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 95)
  (parent 56)
  (unrealized (0 0) (0 2) (1 0) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text) (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (2 1)) ((1 1) (2 3)) ((2 0) (0 0)) ((2 2) (1 0))
    ((2 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nc nb price)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 96)
  (parent 57)
  (seen 8)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (0 0))
    ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 97)
  (parent 58)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nb-0 text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 98)
  (parent 59)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 99)
  (parent 59)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation encryption-test (displaced 5 3 bank 2)
    (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 100)
  (parent 60)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 101)
  (parent 60)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation encryption-test (displaced 5 2 bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-1)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 102)
  (parent 61)
  (seen 168 169 170)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation encryption-test (displaced 5 3 bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-1)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price))))
  (label 103)
  (parent 61)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-1)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 104)
  (parent 61)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 105)
  (parent 62)
  (seen 99)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) nb-0 (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 106)
  (parent 62)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 6 customer 5) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price))))
  (label 107)
  (parent 62)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 108)
  (parent 63)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 109)
  (parent 63)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (5 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 2 customer 3) nb-0 (5 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 110)
  (parent 64)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (3 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (5 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 111)
  (parent 64)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation encryption-test (displaced 5 2 bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 112)
  (parent 65)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 113)
  (parent 65)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nb price goods-0)
  (operation encryption-test (displaced 5 2 bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
      (privk b-0)) (4 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0))))
  (label 114)
  (parent 66)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0)
  (operation encryption-test (displaced 5 3 bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
      (privk b-0)) (4 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0))))
  (label 115)
  (parent 66)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
      (privk b-0)) (4 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 116)
  (parent 66)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 117)
  (parent 67)
  (seen 109)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 118)
  (parent 67)
  (seen 111)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) nb-0 (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 119)
  (parent 67)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) goods-0 (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 goods-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0))))
  (label 120)
  (parent 67)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c))))))
  (label 121)
  (parent 68)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (4 0)) ((2 2) (3 0))
    ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 122)
  (parent 68)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 5 2 customer 3) price (4 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 123)
  (parent 69)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((3 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 124)
  (parent 70)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (4 0)) ((2 2) (3 0))
    ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 price price (pubk hash-0))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c)))))
  (label 125)
  (parent 70)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc acctnum nb-0 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c))))))
  (label 126)
  (parent 71)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 2 5 customer 5) nb-0 (1 0)
    (enc "hash" c acctnum-0 goods-1 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 127)
  (parent 71)
  (unrealized (0 0) (0 2) (1 0) (4 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 128)
  (parent 72)
  (seen 109)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 129)
  (parent 72)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) nb-0 (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 130)
  (parent 72)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) goods-0 (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 goods-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0))))
  (label 131)
  (parent 72)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 132)
  (parent 73)
  (seen 122)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 133)
  (parent 73)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test
    (contracted (b-0 b) (acctnum-0 acctnum) (hash-1 hash-0)
      (nb-0 goods-0)) price (1 0)
    (enc "hash" c acctnum goods-0 price price (pubk hash-0))
    (enc acctnum price m price (pubk c))
    (enc c acctnum price acctnum price (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 price price (pubk hash-0))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c)))))
  (label 134)
  (parent 74)
  (seen 125)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 6 3 bank 2) price-0 (1 0)
    (enc "hash" c acctnum-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc acctnum-0 price-0 m price-0 (pubk c))
    (enc c acctnum-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 135)
  (parent 74)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0))))
  (label 136)
  (parent 75)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (4 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 137)
  (parent 75)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 5 3 customer 3) price (4 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 138)
  (parent 76)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (4 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 139)
  (parent 77)
  (unrealized (0 0) (0 2) (1 0) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 140)
  (parent 78)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 141)
  (parent 79)
  (seen 99)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) nb-0 (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 142)
  (parent 79)
  (seen 170)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 6 customer 5) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price))))
  (label 143)
  (parent 79)
  (seen 174)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 144)
  (parent 80)
  (seen 122)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 145)
  (parent 80)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test
    (contracted (b-0 b) (acctnum-0 acctnum) (hash-1 hash-0)
      (nb-0 goods-0)) price (1 0)
    (enc "hash" c acctnum goods-0 price price (pubk hash-0))
    (enc acctnum price m price (pubk c))
    (enc c acctnum price acctnum price (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 price price (pubk hash-0))
            (privk b)) (enc acctnum goods-0 (pubk c))))))
  (label 146)
  (parent 81)
  (seen 125)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 6 3 bank 2) price-0 (1 0)
    (enc "hash" c acctnum-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc acctnum-0 price-0 m price-0 (pubk c))
    (enc c acctnum-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 147)
  (parent 81)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0))))
  (label 148)
  (parent 82)
  (seen 137)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 149)
  (parent 82)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 150)
  (parent 83)
  (seen 82)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nb-1 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb nb-0 price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))))
  (label 151)
  (parent 85)
  (seen 20)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 152)
  (parent 86)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 153)
  (parent 87)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 154)
  (parent 87)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 2 5 customer 5) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 155)
  (parent 87)
  (seen 229)
  (unrealized (0 0) (0 2) (1 0) (4 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 156)
  (parent 87)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nb-1 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0))))
  (label 157)
  (parent 88)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 158)
  (parent 88)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 159)
  (parent 88)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 160)
  (parent 88)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 3)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc price nb-0 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 161)
  (parent 89)
  (seen 261)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 6 4 customer 3) nb-0 (5 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 162)
  (parent 90)
  (seen 88 164)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (4 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 163)
  (parent 92)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 6 3 customer 3) nb-0 (5 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 164)
  (parent 95)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 acctnum goods text) (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (0 0)) ((3 0) (2 0))
    ((3 2) (1 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 165)
  (parent 97)
  (seen 96)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 acctnum goods text) (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 166)
  (parent 98)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 167)
  (parent 99)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0))))
  (label 168)
  (parent 100)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((0 1) (4 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 169)
  (parent 100)
  (seen 103)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (4 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 170)
  (parent 100)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 5 3 customer 3) nb-0 (4 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 171)
  (parent 101)
  (seen 100)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price))))
  (label 172)
  (parent 103)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 3)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 173)
  (parent 103)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 174)
  (parent 103)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 175)
  (parent 104)
  (seen 103)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 176)
  (parent 105)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation encryption-test (displaced 6 2 bank 2)
    (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 177)
  (parent 106)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 178)
  (parent 106)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation encryption-test (displaced 6 2 bank 2)
    (enc (enc "hash" c nc price-0 price-0 price-0 (pubk hash-1))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 179)
  (parent 107)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation encryption-test (displaced 6 3 bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-1)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price))))
  (label 180)
  (parent 107)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-1)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 181)
  (parent 107)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price price (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))))
  (label 182)
  (parent 108)
  (seen 202)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 183)
  (parent 109)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (5 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 184)
  (parent 110)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 2 6 customer 5) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-1 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 185)
  (parent 110)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (5 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 186)
  (parent 111)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0))))
  (label 187)
  (parent 112)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (4 3)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 188)
  (parent 112)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 189)
  (parent 113)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0))))
  (label 190)
  (parent 114)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (4 3)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 191)
  (parent 114)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0))))
  (label 192)
  (parent 115)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (4 3)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 193)
  (parent 115)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 194)
  (parent 116)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 195)
  (parent 117)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 196)
  (parent 118)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (displaced 6 2 bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 197)
  (parent 119)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 198)
  (parent 119)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nb price goods-0 nm)
  (operation encryption-test (displaced 6 2 bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0))))
  (label 199)
  (parent 120)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation encryption-test (displaced 6 3 bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0))))
  (label 200)
  (parent 120)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 201)
  (parent 120)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text) (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 2 4 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 202)
  (parent 121)
  (seen 96)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (4 0)) ((2 2) (3 0))
    ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 203)
  (parent 122)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 204)
  (parent 123)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 2 5 customer 5) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-1 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price))))
  (label 205)
  (parent 123)
  (seen 282)
  (unrealized (0 0) (0 2) (1 0) (4 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods text) (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nb price acctnum)
  (operation nonce-test (displaced 2 4 customer 5) nb (0 2)
    (enc "hash" c acctnum-0 nb price price (pubk hash))
    (enc acctnum-0 nb (pubk c)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))))
  (label 206)
  (parent 124)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (4 0)) ((2 2) (3 0))
    ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price-0 (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c)))))
  (label 207)
  (parent 125)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 208)
  (parent 126)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 209)
  (parent 126)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0)
  (operation encryption-test (displaced 5 2 bank 2)
    (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
      (privk b-0)) (4 3))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 210)
  (parent 127)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
      (privk b-0)) (4 3))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 211)
  (parent 127)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 212)
  (parent 128)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 213)
  (parent 129)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 5 merchant 2) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 214)
  (parent 129)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 0) (7 0)) ((2 2) (3 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 215)
  (parent 129)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (displaced 6 2 bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 216)
  (parent 130)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 217)
  (parent 130)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nb price goods-0 nm)
  (operation encryption-test (displaced 6 2 bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0))))
  (label 218)
  (parent 131)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation encryption-test (displaced 6 3 bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0))))
  (label 219)
  (parent 131)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 220)
  (parent 131)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 221)
  (parent 132)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 6 2 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 222)
  (parent 133)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((3 1) (1 0)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c)))))
  (label 223)
  (parent 134)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc acctnum nb-0 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 224)
  (parent 135)
  (seen 209)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) nb-0 (1 0)
    (enc "hash" c acctnum-0 goods-1 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 225)
  (parent 135)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (4 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 226)
  (parent 137)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 227)
  (parent 138)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 228)
  (parent 138)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 5 4 bank 2) price (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 229)
  (parent 138)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 3 customer 3) price (5 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 230)
  (parent 139)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 acctnum goods text) (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (1 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 231)
  (parent 140)
  (seen 96)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 232)
  (parent 141)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 233)
  (parent 142)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation encryption-test (displaced 6 2 bank 2)
    (enc (enc "hash" c nc price-0 price-0 price-0 (pubk hash-1))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 234)
  (parent 143)
  (seen 276 277)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-1)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 235)
  (parent 143)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c))))))
  (label 236)
  (parent 144)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 6 2 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 237)
  (parent 145)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 238)
  (parent 146)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc acctnum nb-0 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c))))))
  (label 239)
  (parent 147)
  (seen 209)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) nb-0 (1 0)
    (enc "hash" c acctnum-0 goods-1 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 240)
  (parent 147)
  (seen 338)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0))))
  (label 241)
  (parent 148)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 242)
  (parent 149)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 243)
  (parent 150)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nb-1 nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb nb-0 price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))))
  (label 244)
  (parent 152)
  (seen 32)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 245)
  (parent 153)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 246)
  (parent 153)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (5 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 4 bank 2) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c))))))
  (label 247)
  (parent 154)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 248)
  (parent 154)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 249)
  (parent 154)
  (seen 126 127)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 250)
  (parent 154)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 251)
  (parent 155)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 252)
  (parent 156)
  (seen 246)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 253)
  (parent 156)
  (seen 250 388)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 254)
  (parent 156)
  (seen 381)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 nb-1 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0))))
  (label 255)
  (parent 157)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nb-1 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 256)
  (parent 157)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 6 3 bank 2) nb-1 (5 0)
    (enc nc nb-1 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 257)
  (parent 158)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 258)
  (parent 158)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 6 4 customer 3) nb-0 (5 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 259)
  (parent 158)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (displaced 6 4 customer 3) nb-1 (5 0)
    (enc nc nb-1 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 260)
  (parent 158)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 0 merchant 2) nb-0 (5 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 261)
  (parent 159)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nb-1 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 262)
  (parent 160)
  (seen 255 256)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 263)
  (parent 160)
  (seen 403 408)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 nm price-0 nb-2
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 264)
  (parent 160)
  (seen 414)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 2 bank 2) nb-0 (5 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 265)
  (parent 161)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 6 4 customer 3) nb-1 (5 0)
    (enc price nb-1 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c))))))
  (label 266)
  (parent 161)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 acctnum-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (5 0)
    (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-2 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-2 (pubk c))))))
  (label 267)
  (parent 162)
  (seen 164)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 268)
  (parent 164)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (4 0)
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 269)
  (parent 164)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 6 5 bank 2) nb-1 (4 0)
    (enc "hash" c nc nb-0 nb-1 price (pubk hash-1))
    (enc nc nb-1 m price (pubk c))
    (enc c nc nb-1 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 270)
  (parent 164)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 6 2 bank 2) nb-0 (4 0)
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 271)
  (parent 164)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods text) (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 272)
  (parent 166)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb nm acctnum)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb price-0 price-0 (pubk hash))
    (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 273)
  (parent 167)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0))))
  (label 274)
  (parent 168)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (4 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 275)
  (parent 168)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (4 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 276)
  (parent 170)
  (seen 275)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 0) (4 0)) ((3 2) (2 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 277)
  (parent 170)
  (seen 174)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (4 0)
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 278)
  (parent 171)
  (seen 285)
  (unrealized (0 0) (0 2) (1 0) (4 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price))))
  (label 279)
  (parent 172)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 3))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 280)
  (parent 172)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 6 2 bank 2) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 281)
  (parent 173)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 282)
  (parent 173)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 283)
  (parent 174)
  (seen 280)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 3)) ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 284)
  (parent 174)
  (seen 441 443)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 2 bank 2) price (5 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 285)
  (parent 175)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 286)
  (parent 175)
  (seen 285)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb nm acctnum)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb price-0 price-0 (pubk hash))
    (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 287)
  (parent 176)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0))))
  (label 288)
  (parent 177)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 289)
  (parent 177)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) nb-0 (6 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 290)
  (parent 178)
  (seen 177)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0))))
  (label 291)
  (parent 179)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 292)
  (parent 179)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price))))
  (label 293)
  (parent 180)
  (seen 280)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (5 3)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 294)
  (parent 180)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 295)
  (parent 181)
  (seen 457)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price price (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))))
  (label 296)
  (parent 183)
  (seen 329)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (5 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 297)
  (parent 184)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (3 0))
    ((2 2) (5 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 298)
  (parent 184)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (5 3)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation encryption-test (displaced 6 4 bank 2)
    (enc (enc "hash" c nb-0 price price price (pubk hash-2))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 299)
  (parent 185)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 price price price (pubk hash-2))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 300)
  (parent 185)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 301)
  (parent 186)
  (seen 298)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-1 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 302)
  (parent 186)
  (unrealized (0 0) (0 2) (1 0) (6 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0))))
  (label 303)
  (parent 187)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 304)
  (parent 187)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 305)
  (parent 188)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 306)
  (parent 189)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 2 bank 2) price (5 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 307)
  (parent 189)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 308)
  (parent 189)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods))))
  (label 309)
  (parent 190)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 310)
  (parent 190)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 311)
  (parent 191)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods))))
  (label 312)
  (parent 192)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 313)
  (parent 192)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 314)
  (parent 193)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 315)
  (parent 194)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 6 2 bank 2) price (5 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 316)
  (parent 194)
  (seen 115)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 317)
  (parent 194)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price price (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))))
  (label 318)
  (parent 195)
  (seen 373)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 319)
  (parent 196)
  (seen 298)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-1 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 320)
  (parent 196)
  (seen 467)
  (unrealized (0 0) (0 2) (1 0) (6 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0))))
  (label 321)
  (parent 197)
  (seen 304)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 322)
  (parent 197)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 323)
  (parent 198)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0))))
  (label 324)
  (parent 199)
  (seen 310)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 325)
  (parent 199)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0))))
  (label 326)
  (parent 200)
  (seen 313)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 3)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 327)
  (parent 200)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 328)
  (parent 201)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (0 0))
    ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 329)
  (parent 203)
  (seen 165)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 330)
  (parent 204)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 331)
  (parent 204)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-1)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 332)
  (parent 205)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods text) (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (2 1)) ((1 1) (2 3)) ((2 0) (0 0)) ((2 2) (1 0))
    ((2 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nb price acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))))
  (label 333)
  (parent 206)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (0 0))
    ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nb price nm acctnum)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c acctnum-0 nb price price (pubk hash))
    (enc acctnum-0 nb (pubk c)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))))
  (label 334)
  (parent 207)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 acctnum goods text) (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb nb-0 acctnum)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
    (enc acctnum-0 nb (pubk c)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))))
  (label 335)
  (parent 208)
  (seen 206)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 336)
  (parent 209)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0))))
  (label 337)
  (parent 210)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 goods-0 (pubk c))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 338)
  (parent 210)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 6 4 customer 3) nb-0 (5 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 339)
  (parent 211)
  (seen 210)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price price (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))))
  (label 340)
  (parent 212)
  (seen 354)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 341)
  (parent 213)
  (seen 298)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-1 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 342)
  (parent 213)
  (unrealized (0 0) (0 2) (1 0) (6 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 343)
  (parent 214)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 8 2 customer 3) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 344)
  (parent 215)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nm-0 price-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc price-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 0) (7 0)) ((2 2) (3 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 5 merchant 2) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 345)
  (parent 215)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0))))
  (label 346)
  (parent 216)
  (seen 304)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 347)
  (parent 216)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 348)
  (parent 217)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0))))
  (label 349)
  (parent 218)
  (seen 310)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 350)
  (parent 218)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0))))
  (label 351)
  (parent 219)
  (seen 313)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 3)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 352)
  (parent 219)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 353)
  (parent 220)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 354)
  (parent 221)
  (seen 231)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 355)
  (parent 222)
  (seen 331)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 6 customer 5) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-1 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price))))
  (label 356)
  (parent 222)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nb price nm acctnum)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c acctnum-0 nb price price (pubk hash))
    (enc acctnum-0 nb (pubk c)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))))
  (label 357)
  (parent 223)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 358)
  (parent 224)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation encryption-test (displaced 6 2 bank 2)
    (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0))))
  (label 359)
  (parent 225)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 360)
  (parent 225)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 361)
  (parent 227)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 362)
  (parent 227)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 3 customer 3) price (5 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 363)
  (parent 228)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 364)
  (parent 229)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 365)
  (parent 229)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (4 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 366)
  (parent 230)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (4 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 367)
  (parent 230)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 6 5 bank 2) price (4 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 368)
  (parent 230)
  (seen 551)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 2 bank 2) price (4 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 369)
  (parent 230)
  (seen 553)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb nm acctnum)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb price-0 price-0 (pubk hash))
    (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 370)
  (parent 232)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 6 4 customer 3) nb-0 (5 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 371)
  (parent 233)
  (seen 170)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 372)
  (parent 235)
  (seen 174)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (2 0)) ((4 2) (3 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 373)
  (parent 236)
  (seen 202)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 374)
  (parent 237)
  (seen 331)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 2) (3 0))
    ((5 2) (4 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 6 customer 5) price (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-1 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price))))
  (label 375)
  (parent 237)
  (seen 443)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (2 0)) ((4 2) (3 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nb price nm acctnum)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c acctnum-0 nb price price (pubk hash))
    (enc acctnum-0 nb (pubk c)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))))
  (label 376)
  (parent 238)
  (seen 206)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 377)
  (parent 239)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 378)
  (parent 240)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 379)
  (parent 242)
  (seen 362)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 380)
  (parent 242)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 6 5 bank 2) price-0 (1 0)
    (enc "hash" c nb-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-1))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 381)
  (parent 242)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 382)
  (parent 243)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 383)
  (parent 245)
  (seen 57)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 384)
  (parent 246)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (5 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c))))))
  (label 385)
  (parent 247)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 2 6 customer 5) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 386)
  (parent 247)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 2 6 customer 5) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0))))
  (label 387)
  (parent 247)
  (seen 483 487)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 388)
  (parent 247)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 389)
  (parent 248)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 2 5 customer 5) price (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-1 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price))))
  (label 390)
  (parent 248)
  (unrealized (0 0) (0 2) (1 0) (4 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (4 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 2 6 customer 5) nb-0 (1 0)
    (enc "hash" c acctnum-0 goods-1 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 391)
  (parent 249)
  (seen 210 211)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 4 bank 2) nc (5 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 392)
  (parent 250)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 393)
  (parent 250)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c)))))
  (label 394)
  (parent 250)
  (seen 224 225)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 395)
  (parent 251)
  (seen 229 368)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 396)
  (parent 252)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 397)
  (parent 253)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 2) (6 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 398)
  (parent 253)
  (seen 239 240)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 399)
  (parent 254)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 nb-1 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 400)
  (parent 256)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 401)
  (parent 257)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 402)
  (parent 257)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 403)
  (parent 257)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test
    (contracted (b-0 b) (acctnum-0 acctnum) (hash-2 hash-1)
      (nb-0 goods-0)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 404)
  (parent 258)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 405)
  (parent 258)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 406)
  (parent 259)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 407)
  (parent 259)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 408)
  (parent 259)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum)
  (operation nonce-test
    (contracted (b-0 b) (acctnum-0 acctnum) (hash-2 hash-1)
      (nb-0 goods-0)) nc (1 0)
    (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc goods-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc acctnum m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 409)
  (parent 260)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price goods-0 (pubk c)) (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 410)
  (parent 260)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc acctnum-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 411)
  (parent 260)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nb-1 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0))))
  (label 412)
  (parent 261)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 413)
  (parent 261)
  (seen 608)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 414)
  (parent 261)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 415)
  (parent 263)
  (seen 605)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (displaced 7 4 customer 3) nb-1 (6 0)
    (enc nc nb-1 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 416)
  (parent 263)
  (seen 615)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 417)
  (parent 265)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (4 3)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 418)
  (parent 265)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 419)
  (parent 265)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c))))))
  (label 420)
  (parent 266)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 421)
  (parent 266)
  (seen 614)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 422)
  (parent 266)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 423)
  (parent 267)
  (seen 446)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (displaced 7 4 customer 3) nb-2 (6 0)
    (enc nc nb-2 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 424)
  (parent 267)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 425)
  (parent 268)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 426)
  (parent 268)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 427)
  (parent 268)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 428)
  (parent 268)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 0 merchant 2) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 429)
  (parent 269)
  (seen 446 635 638)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "5 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 430)
  (parent 270)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 2 bank 2) price (4 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 431)
  (parent 270)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (4 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 432)
  (parent 270)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 433)
  (parent 271)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (4 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 434)
  (parent 271)
  (seen 638)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods text) (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (2 1)) ((1 1) (2 3)) ((2 0) (0 0)) ((2 2) (1 0))
    ((2 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nc nb acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 435)
  (parent 272)
  (seen 96)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price acctnum goods text) (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (0 0))
    ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb nm acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 436)
  (parent 273)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nm price text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (4 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 437)
  (parent 275)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nm price text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (4 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 438)
  (parent 276)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 3))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 439)
  (parent 280)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 440)
  (parent 281)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 441)
  (parent 281)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 442)
  (parent 282)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 443)
  (parent 282)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 444)
  (parent 283)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 445)
  (parent 285)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 446)
  (parent 285)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 3 bank 2) price (5 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 447)
  (parent 285)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 448)
  (parent 285)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price acctnum goods text) (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb nm acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 449)
  (parent 287)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0))))
  (label 450)
  (parent 288)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 451)
  (parent 288)
  (seen 437)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) nb-0 (6 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 452)
  (parent 289)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 7 3 bank 2) nb-0 (6 0)
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 453)
  (parent 290)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 454)
  (parent 290)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0))))
  (label 455)
  (parent 291)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 456)
  (parent 291)
  (seen 437)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) nb-0 (6 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 457)
  (parent 292)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price))))
  (label 458)
  (parent 293)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 7 2 bank 2) price-0 (6 0)
    (enc nc price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 459)
  (parent 294)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 460)
  (parent 294)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 2 bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 461)
  (parent 295)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 3 bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 462)
  (parent 295)
  (seen 693)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 463)
  (parent 295)
  (seen 461)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb nb-0 acctnum)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price price (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum acctnum (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum acctnum (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum acctnum (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m acctnum (pubk c)))
      (send (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 464)
  (parent 297)
  (seen 506)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (3 0))
    ((2 2) (5 0)) ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 465)
  (parent 298)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (5 3)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 466)
  (parent 299)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (5 3)) ((5 0) (0 0)) ((5 0) (6 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 467)
  (parent 299)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 468)
  (parent 300)
  (seen 299)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 469)
  (parent 301)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 3)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (displaced 7 4 bank 2)
    (enc (enc "hash" c nb-0 price price price (pubk hash-2))
      (privk b-0)) (6 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 470)
  (parent 302)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 price price price (pubk hash-2))
      (privk b-0)) (6 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 471)
  (parent 302)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 472)
  (parent 304)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 473)
  (parent 305)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 474)
  (parent 306)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 475)
  (parent 306)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 476)
  (parent 307)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 3 bank 2) price (5 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 477)
  (parent 307)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 478)
  (parent 307)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 479)
  (parent 308)
  (seen 715)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 480)
  (parent 310)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 481)
  (parent 311)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (4 3)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 482)
  (parent 311)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 6 5 bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 483)
  (parent 311)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 484)
  (parent 313)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 485)
  (parent 314)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (4 3)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 486)
  (parent 314)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 6 5 bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 487)
  (parent 314)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))))
  (label 488)
  (parent 315)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 489)
  (parent 315)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 490)
  (parent 316)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 491)
  (parent 317)
  (seen 732)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 492)
  (parent 319)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 price price price (pubk hash-2))
      (privk b-0)) (6 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 493)
  (parent 320)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0))))
  (label 494)
  (parent 321)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 495)
  (parent 322)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 496)
  (parent 323)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 bank 2) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 497)
  (parent 323)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 498)
  (parent 323)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods))))
  (label 499)
  (parent 324)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 500)
  (parent 325)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods))))
  (label 501)
  (parent 326)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 502)
  (parent 327)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 503)
  (parent 328)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 7 2 bank 2) price (6 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 504)
  (parent 328)
  (seen 200)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 505)
  (parent 328)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods text) (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 506)
  (parent 330)
  (seen 272)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 507)
  (parent 331)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 508)
  (parent 332)
  (seen 282)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods price-0 text) (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price-0) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (2 1)) ((1 1) (2 3)) ((2 0) (0 0)) ((2 2) (1 0))
    ((2 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nb price acctnum)
  (operation generalization separated price-0)
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price-0 (pubk c)))
      (send (enc c acctnum price acctnum price-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price-0 (pubk hash))
            (privk b)) nb))))
  (label 509)
  (parent 333)
  (seen 8)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 acctnum goods text) (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (0 0)) ((3 0) (2 0))
    ((3 2) (1 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nb price nm acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))))
  (label 510)
  (parent 334)
  (seen 333)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb nb-0 nm acctnum)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
    (enc acctnum-0 nb (pubk c)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))))
  (label 511)
  (parent 336)
  (seen 334)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0))))
  (label 512)
  (parent 337)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 513)
  (parent 337)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc acctnum nm m price (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 514)
  (parent 338)
  (seen 513)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 6 3 bank 2) nb-0 (5 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 515)
  (parent 339)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (5 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 516)
  (parent 339)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 517)
  (parent 341)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (6 3)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (displaced 7 5 bank 2)
    (enc (enc "hash" c nb-0 price price price (pubk hash-2))
      (privk b-0)) (6 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 518)
  (parent 342)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 price price price (pubk hash-2))
      (privk b-0)) (6 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 519)
  (parent 342)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 520)
  (parent 343)
  (seen 298)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-1 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 521)
  (parent 343)
  (unrealized (0 0) (0 2) (1 0) (6 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 522)
  (parent 344)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 2 8 customer 5) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-1 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 523)
  (parent 344)
  (unrealized (0 0) (0 2) (1 0) (7 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nm-0 price-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc price-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 2 customer 3) price-0 (6 0)
    (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 524)
  (parent 345)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0))))
  (label 525)
  (parent 346)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 526)
  (parent 347)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 527)
  (parent 348)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 bank 2) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 528)
  (parent 348)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 529)
  (parent 348)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods))))
  (label 530)
  (parent 349)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 531)
  (parent 350)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods))))
  (label 532)
  (parent 351)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 533)
  (parent 352)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 534)
  (parent 353)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 7 2 bank 2) price (6 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 535)
  (parent 353)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 536)
  (parent 353)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 537)
  (parent 355)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (5 3)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation encryption-test (displaced 6 4 bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-1)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price))))
  (label 538)
  (parent 356)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-1)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 539)
  (parent 356)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 acctnum goods text) (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (1 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nb price nm acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))))
  (label 540)
  (parent 357)
  (seen 333)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb nb-0 nm acctnum)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
    (enc acctnum-0 nb (pubk c)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))))
  (label 541)
  (parent 358)
  (seen 357)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc acctnum nm m price (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0))))
  (label 542)
  (parent 359)
  (seen 513)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) nb-0 (6 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 543)
  (parent 360)
  (seen 359)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 544)
  (parent 362)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 545)
  (parent 363)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 546)
  (parent 364)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 547)
  (parent 364)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 6 3 customer 3) nb-1 (5 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 548)
  (parent 365)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 549)
  (parent 366)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 550)
  (parent 366)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 5 4 bank 2) price (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 551)
  (parent 366)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 3 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 552)
  (parent 367)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 6 2 bank 2) nb-1 (4 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 553)
  (parent 368)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (4 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 554)
  (parent 368)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (4 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 555)
  (parent 369)
  (seen 811)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (4 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 556)
  (parent 369)
  (seen 810)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price acctnum goods text) (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (2 0)) ((4 2) (3 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nc nb nm acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 557)
  (parent 370)
  (seen 272)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (5 0)
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 558)
  (parent 371)
  (seen 559)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 2 bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 559)
  (parent 372)
  (seen 675)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 560)
  (parent 372)
  (seen 559)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 561)
  (parent 374)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 2) (3 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-1)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 562)
  (parent 375)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb nb-0 nm acctnum)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
    (enc acctnum-0 nb (pubk c)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))))
  (label 563)
  (parent 377)
  (seen 376)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) nb-0 (6 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 564)
  (parent 378)
  (seen 338)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 565)
  (parent 379)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 566)
  (parent 380)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 567)
  (parent 381)
  (seen 547)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 568)
  (parent 381)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 569)
  (parent 382)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 570)
  (parent 382)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 7 6 bank 2) price-0 (5 0)
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-1))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 571)
  (parent 382)
  (seen 828)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 bank 2) price (5 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 572)
  (parent 382)
  (seen 830)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 573)
  (parent 384)
  (seen 97)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (5 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 574)
  (parent 385)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 575)
  (parent 385)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (5 3)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation encryption-test (displaced 6 3 bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 576)
  (parent 386)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 577)
  (parent 386)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
      (privk b-0)) (5 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 578)
  (parent 387)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 579)
  (parent 388)
  (seen 575)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 580)
  (parent 388)
  (unrealized (0 0) (0 2) (1 0) (6 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0))))
  (label 581)
  (parent 388)
  (seen 747 750)
  (unrealized (0 0) (0 2) (1 0) (6 3))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 582)
  (parent 389)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 583)
  (parent 389)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation encryption-test (displaced 5 2 bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-0)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price))))
  (label 584)
  (parent 390)
  (seen 404 405)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-0)) (privk b-0))
    (4 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 585)
  (parent 390)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 586)
  (parent 392)
  (seen 575)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 587)
  (parent 392)
  (unrealized (0 0) (0 2) (1 0) (6 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0))))
  (label 588)
  (parent 392)
  (seen 784 787)
  (unrealized (0 0) (0 2) (1 0) (6 3))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 589)
  (parent 393)
  (seen 583)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 6 customer 5) price (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-1 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price))))
  (label 590)
  (parent 393)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (4 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) nb-0 (1 0)
    (enc "hash" c acctnum-0 goods-1 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 591)
  (parent 394)
  (seen 359 360)
  (unrealized (0 0) (0 2) (1 0) (6 3))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 592)
  (parent 395)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 593)
  (parent 396)
  (seen 140)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 594)
  (parent 397)
  (seen 583)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 6 customer 5) price (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-1 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price))))
  (label 595)
  (parent 397)
  (seen 405)
  (unrealized (0 0) (0 2) (1 0) (5 3))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (5 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) nb-0 (1 0)
    (enc "hash" c acctnum-0 goods-1 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 596)
  (parent 398)
  (seen 338 378)
  (unrealized (0 0) (0 2) (1 0) (6 3))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 597)
  (parent 399)
  (seen 381 571)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 598)
  (parent 401)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 599)
  (parent 401)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 0 merchant 2) nb-0 (6 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 600)
  (parent 402)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 601)
  (parent 403)
  (seen 598 599)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 602)
  (parent 403)
  (seen 859)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 603)
  (parent 404)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 604)
  (parent 404)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 605)
  (parent 405)
  (seen 604)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 606)
  (parent 406)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 607)
  (parent 406)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 0 merchant 2) nb-0 (5 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 608)
  (parent 407)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 609)
  (parent 408)
  (seen 606 607)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0))
    ((3 4) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 610)
  (parent 408)
  (seen 864)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 611)
  (parent 409)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 612)
  (parent 409)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 3 bank 2) acctnum-0 (6 0)
    (enc price acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 613)
  (parent 410)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 7 0 merchant 2) acctnum-0 (6 0)
    (enc price acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 614)
  (parent 410)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc goods-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc acctnum m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 615)
  (parent 411)
  (seen 611 612)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0))
    ((3 4) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c))))))
  (label 616)
  (parent 411)
  (seen 869)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 nb-1 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0))))
  (label 617)
  (parent 412)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nb-1 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 618)
  (parent 412)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nb-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-1)
    (nm nb-0) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 6 3 bank 2) price (5 0)
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-1 nb-1 (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-1 goods-0 (pubk c))))))
  (label 619)
  (parent 413)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nb-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-1 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c))))))
  (label 620)
  (parent 413)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (5 0)
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 621)
  (parent 413)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nb-1 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 622)
  (parent 414)
  (seen 618)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 nb-1 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 623)
  (parent 414)
  (seen 621 864 872)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price goods-0 (pubk c)) (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 624)
  (parent 416)
  (seen 867 869)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 625)
  (parent 417)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 626)
  (parent 417)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 4 customer 3) nb-0 (6 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 627)
  (parent 418)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 628)
  (parent 419)
  (seen 626)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (4 3)) ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 629)
  (parent 419)
  (seen 883)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c))))))
  (label 630)
  (parent 420)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 631)
  (parent 420)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (5 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 4 bank 2) acctnum-0 (5 0)
    (enc price acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 632)
  (parent 421)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 633)
  (parent 422)
  (seen 631)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0))
    ((3 4) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 634)
  (parent 422)
  (seen 869 887)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test
    (contracted (hash-2 hash-1) (nb-1 nb-0) (acctnum-1 acctnum-0)) price
    (5 0) (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 635)
  (parent 423)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 3 bank 2) price (5 0)
    (enc "hash" c nc nb-1 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 636)
  (parent 423)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (displaced 7 6 bank 2) price (5 0)
    (enc "hash" c nc nb-1 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 637)
  (parent 423)
  (seen 636)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 2 bank 2) price (5 0)
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 638)
  (parent 423)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test
    (contracted (hash-2 hash-1) (nb-1 nb-0) (acctnum-1 acctnum-0))
    acctnum-0 (5 0) (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 m price (pubk c))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 639)
  (parent 424)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (added-strand bank 2) acctnum-0 (5 0)
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 m price (pubk c))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))))
  (label 640)
  (parent 424)
  (seen 644)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 2 bank 2) price (5 0)
    (enc "hash" c nc nb-1 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 641)
  (parent 424)
  (seen 677)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 3 bank 2) price (5 0)
    (enc "hash" c nc nb-1 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 642)
  (parent 424)
  (seen 445 446 447 676)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "5 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nc nb-1 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-2 price price (pubk hash-3))
            (privk b-0)) (enc nc nb-2 (pubk c))))))
  (label 643)
  (parent 424)
  (seen 285)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 3 bank 2) acctnum-1 (5 0)
    (enc "hash" c nc nb-1 acctnum-1 price (pubk hash-2))
    (enc nc acctnum-1 m price (pubk c))
    (enc c nc acctnum-1 acctnum-1 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 644)
  (parent 424)
  (seen 900)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (displaced 7 2 bank 2) acctnum-0 (5 0)
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 m price (pubk c))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 645)
  (parent 424)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (displaced 7 4 customer 3) acctnum-0 (5 0)
    (enc "hash" c nc nb-1 acctnum-0 price (pubk hash-2))
    (enc nc acctnum-0 m price (pubk c))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))))
  (label 646)
  (parent 424)
  (seen 644)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 647)
  (parent 425)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 648)
  (parent 425)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 6 2 customer 3) nb-0 (5 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 649)
  (parent 426)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 0 merchant 2) nb-0 (5 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 650)
  (parent 427)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 651)
  (parent 428)
  (seen 647 648)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 652)
  (parent 428)
  (seen 912)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 nb-2
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 653)
  (parent 428)
  (seen 915)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (4 0)
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 654)
  (parent 429)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm
      price-0 text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (4 0)
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 655)
  (parent 429)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 656)
  (parent 430)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 657)
  (parent 430)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 658)
  (parent 430)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb price)
  (operation nonce-test
    (contracted (acctnum-1 acctnum-0) (hash-1 hash-0)) price (4 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 659)
  (parent 431)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (4 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 660)
  (parent 431)
  (seen 638)
  (unrealized (0 0) (0 2) (1 0) (4 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 3 customer 3) nb-0 (6 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 661)
  (parent 432)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 662)
  (parent 433)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 663)
  (parent 433)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 664)
  (parent 433)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 665)
  (parent 433)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price acctnum goods text) (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (0 0)) ((3 0) (2 0))
    ((3 2) (1 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nc nb nm acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 666)
  (parent 436)
  (seen 435)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods text) (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 667)
  (parent 440)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 668)
  (parent 440)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 669)
  (parent 441)
  (seen 668)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 670)
  (parent 442)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 671)
  (parent 442)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 672)
  (parent 443)
  (seen 671)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 673)
  (parent 445)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 674)
  (parent 445)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 675)
  (parent 445)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 676)
  (parent 446)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 677)
  (parent 447)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 678)
  (parent 448)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 679)
  (parent 448)
  (seen 676)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price acctnum goods text) (c hash b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (3 1)) ((1 1) (3 3)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (1 0)) ((3 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk b) (privk m))
  (uniq-orig nc nb nm acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 680)
  (parent 449)
  (seen 435)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 681)
  (parent 452)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 682)
  (parent 452)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 683)
  (parent 453)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 7 2 bank 2) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 684)
  (parent 453)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 685)
  (parent 453)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 686)
  (parent 454)
  (seen 966)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 687)
  (parent 457)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 688)
  (parent 457)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 689)
  (parent 459)
  (seen 668)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 690)
  (parent 460)
  (seen 671)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 691)
  (parent 461)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 692)
  (parent 461)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 3 bank 2) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 693)
  (parent 461)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 694)
  (parent 461)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 695)
  (parent 462)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 696)
  (parent 462)
  (seen 693)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (0 0)) ((6 0) (5 0)) ((6 2) (2 0))
    ((6 2) (4 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb nb-0 nm acctnum)
  (operation nonce-test (displaced 2 7 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price-0 price-0 (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum acctnum (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum acctnum (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum acctnum (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m acctnum (pubk c)))
      (send (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 697)
  (parent 465)
  (seen 755)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (5 3)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 698)
  (parent 466)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (5 3)) ((5 0) (6 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 699)
  (parent 466)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (5 3)) ((5 0) (0 0)) ((5 0) (6 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 700)
  (parent 467)
  (seen 699)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 2 bank 2) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 701)
  (parent 468)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 702)
  (parent 468)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb nb-0 nm acctnum)
  (operation nonce-test (displaced 2 7 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price-0 price-0 (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum acctnum (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum acctnum (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum acctnum (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m acctnum (pubk c)))
      (send (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 703)
  (parent 469)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 3)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 704)
  (parent 470)
  (seen 699)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 6 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 705)
  (parent 471)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 706)
  (parent 473)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 707)
  (parent 473)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 708)
  (parent 474)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 709)
  (parent 474)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 710)
  (parent 475)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 711)
  (parent 476)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 712)
  (parent 476)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 713)
  (parent 477)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 714)
  (parent 477)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 715)
  (parent 478)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 716)
  (parent 479)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 717)
  (parent 479)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 6 bank 2) price (5 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 718)
  (parent 479)
  (seen 1010 1013)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 719)
  (parent 481)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 720)
  (parent 481)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 721)
  (parent 482)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 722)
  (parent 483)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (4 3)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 723)
  (parent 483)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 724)
  (parent 485)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 725)
  (parent 485)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 726)
  (parent 486)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 727)
  (parent 487)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (4 3)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 728)
  (parent 487)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))))
  (label 729)
  (parent 488)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 730)
  (parent 488)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 731)
  (parent 489)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 732)
  (parent 490)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 733)
  (parent 491)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 734)
  (parent 491)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price (5 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 735)
  (parent 491)
  (seen 1032 1036)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb nb-0 nm acctnum)
  (operation nonce-test (displaced 2 7 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price-0 price-0 (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum acctnum (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum acctnum (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum acctnum (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m acctnum (pubk c)))
      (send (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 736)
  (parent 492)
  (seen 819)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 6 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 737)
  (parent 493)
  (seen 467)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 738)
  (parent 495)
  (seen 707)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 739)
  (parent 496)
  (seen 709)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 740)
  (parent 496)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 741)
  (parent 497)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 3 bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 742)
  (parent 497)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 743)
  (parent 497)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 744)
  (parent 498)
  (seen 1048)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 745)
  (parent 500)
  (seen 720)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 746)
  (parent 500)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-1))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 747)
  (parent 500)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 748)
  (parent 502)
  (seen 725)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 3)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 749)
  (parent 502)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 750)
  (parent 502)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))))
  (label 751)
  (parent 503)
  (seen 730)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 752)
  (parent 503)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 753)
  (parent 504)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 754)
  (parent 505)
  (seen 1062)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb nm acctnum)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb price-0 price-0 (pubk hash))
    (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 755)
  (parent 507)
  (seen 436)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 2 bank 2) price (5 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 756)
  (parent 508)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 757)
  (parent 508)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 758)
  (parent 513)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 759)
  (parent 514)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 760)
  (parent 515)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 6 2 bank 2) nb-0 (5 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 761)
  (parent 515)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 7 4 customer 3) nb-0 (6 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 762)
  (parent 516)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb nb-0 nm acctnum)
  (operation nonce-test (displaced 2 7 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price-0 price-0 (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum acctnum (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum acctnum (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum acctnum (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m acctnum (pubk c)))
      (send (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 763)
  (parent 517)
  (seen 794)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (6 3)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 764)
  (parent 518)
  (seen 699)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 6 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 765)
  (parent 519)
  (seen 518)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 766)
  (parent 520)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (6 3)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (displaced 7 5 bank 2)
    (enc (enc "hash" c nb-0 price price price (pubk hash-2))
      (privk b-0)) (6 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 767)
  (parent 521)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 price price price (pubk hash-2))
      (privk b-0)) (6 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 768)
  (parent 521)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 769)
  (parent 522)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 770)
  (parent 522)
  (seen 465)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (7 3)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation encryption-test (displaced 8 5 bank 2)
    (enc (enc "hash" c nb-0 price price price (pubk hash-2))
      (privk b-0)) (7 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 771)
  (parent 523)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 price price price (pubk hash-2))
      (privk b-0)) (7 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 772)
  (parent 523)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc price-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 773)
  (parent 524)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 2 8 customer 5) price (1 0)
    (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 goods-1 price price (pubk hash-1))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 774)
  (parent 524)
  (unrealized (0 0) (0 2) (1 0) (7 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 775)
  (parent 526)
  (seen 707)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 776)
  (parent 527)
  (seen 709)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 777)
  (parent 527)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 778)
  (parent 528)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 3 bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 779)
  (parent 528)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 780)
  (parent 528)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 781)
  (parent 529)
  (seen 1098)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 782)
  (parent 531)
  (seen 720)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 783)
  (parent 531)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-1))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 784)
  (parent 531)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 785)
  (parent 533)
  (seen 725)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 3)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 786)
  (parent 533)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 787)
  (parent 533)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))))
  (label 788)
  (parent 534)
  (seen 730)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 789)
  (parent 534)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 790)
  (parent 535)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 7 3 bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 791)
  (parent 535)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 792)
  (parent 535)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 793)
  (parent 536)
  (seen 1116)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb nm acctnum)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb price-0 price-0 (pubk hash))
    (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 794)
  (parent 537)
  (seen 557)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (5 3)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price))))
  (label 795)
  (parent 538)
  (seen 671)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 796)
  (parent 539)
  (seen 538)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0))))
  (label 797)
  (parent 542)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 7 3 bank 2) nb-0 (6 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 798)
  (parent 543)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 799)
  (parent 543)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 800)
  (parent 545)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 801)
  (parent 545)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 802)
  (parent 547)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 803)
  (parent 548)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 804)
  (parent 549)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 805)
  (parent 549)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 2 customer 3) price (5 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 806)
  (parent 550)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 807)
  (parent 551)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 808)
  (parent 551)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 809)
  (parent 552)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 2 bank 2) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 810)
  (parent 552)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (4 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 811)
  (parent 553)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (4 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 812)
  (parent 553)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 3 customer 3) nb-1 (6 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 813)
  (parent 554)
  (seen 1138)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 814)
  (parent 555)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 815)
  (parent 555)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 816)
  (parent 559)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 4 bank 2) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 817)
  (parent 559)
  (seen 954)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 818)
  (parent 559)
  (seen 957)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 2) (3 0))
    ((5 2) (4 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb nm acctnum)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb price-0 price-0 (pubk hash))
    (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 819)
  (parent 561)
  (seen 506)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 2) (3 0))
    ((5 2) (4 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 820)
  (parent 562)
  (seen 443)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 7 4 bank 2) nb-0 (6 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 821)
  (parent 564)
  (seen 1071)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 822)
  (parent 564)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 823)
  (parent 566)
  (seen 801)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 824)
  (parent 567)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 7 4 customer 3) nb-1 (6 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 825)
  (parent 568)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 826)
  (parent 569)
  (seen 805)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 827)
  (parent 569)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 6 5 bank 2) price-0 (1 0)
    (enc "hash" c nb-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-1))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 828)
  (parent 569)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 829)
  (parent 570)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 7 2 bank 2) nb-1 (5 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 830)
  (parent 571)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (5 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 831)
  (parent 571)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 832)
  (parent 572)
  (seen 1157)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 833)
  (parent 572)
  (seen 1156)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price price (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))))
  (label 834)
  (parent 574)
  (seen 182)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 835)
  (parent 575)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (5 3)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0))))
  (label 836)
  (parent 576)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 837)
  (parent 577)
  (seen 718)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 838)
  (parent 578)
  (seen 735 1029)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 839)
  (parent 579)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 3)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (displaced 7 3 bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1)) (privk b-0))
    (6 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 840)
  (parent 580)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1)) (privk b-0))
    (6 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 841)
  (parent 580)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
      (privk b-0)) (6 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 842)
  (parent 581)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (4 1)) ((1 1) (4 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 2 5 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 843)
  (parent 582)
  (seen 202)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (3 0))
    ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 844)
  (parent 583)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 4 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 845)
  (parent 585)
  (seen 584 917)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 846)
  (parent 586)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 3)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (displaced 7 3 bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1)) (privk b-0))
    (6 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0))))
  (label 847)
  (parent 587)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1)) (privk b-0))
    (6 3))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 848)
  (parent 587)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
      (privk b-0)) (6 3))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 849)
  (parent 588)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 850)
  (parent 589)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation encryption-test (displaced 6 2 bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-0)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price))))
  (label 851)
  (parent 590)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-0)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 852)
  (parent 590)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 853)
  (parent 592)
  (seen 813)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum acctnum) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((3 1) (1 0)) ((4 1) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 854)
  (parent 594)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c nc price price price (pubk hash-0)) (privk b-0))
    (5 3))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 855)
  (parent 595)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 856)
  (parent 597)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 857)
  (parent 599)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 858)
  (parent 600)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 859)
  (parent 600)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 860)
  (parent 604)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 861)
  (parent 605)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 862)
  (parent 607)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 863)
  (parent 608)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 864)
  (parent 608)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 865)
  (parent 612)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 866)
  (parent 613)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 867)
  (parent 613)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price goods-0 (pubk c))
    (enc price acctnum m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-1))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 868)
  (parent 614)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 869)
  (parent 614)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 nb-1 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 870)
  (parent 618)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 nb-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
    (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
    (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-2))
    (enc nb-1 nb-0 (pubk c)) (enc nb-1 nb-1 (pubk c))
    (enc nb-1 goods-0 (pubk c)) (enc nb-1 nb-0 m nb-1 (pubk c))
    (enc c nb-1 goods-0 (pubk m))
    (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 goods-0 (pubk c))))))
  (label 871)
  (parent 619)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nb-1 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-1)
    (nm nb-0) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-1 (1 0)
    (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
    (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
    (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-2))
    (enc nb-1 nb-0 (pubk c)) (enc nb-1 nb-1 (pubk c))
    (enc nb-1 goods-0 (pubk c)) (enc nb-1 nb-0 m nb-1 (pubk c))
    (enc c nb-1 goods-0 (pubk m))
    (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-1 nb-1 (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-1 goods-0 (pubk c)))))
    ((recv (enc c nb-1 goods-0 (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 872)
  (parent 619)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test
    (contracted (b-0 b) (acctnum-0 acctnum) (hash-2 hash-1)
      (nb-1 goods-0)) acctnum (1 0)
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m acctnum (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c))))))
  (label 873)
  (parent 620)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (added-strand bank 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 goods-0 (pubk c))
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-1)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c))))))
  (label 874)
  (parent 620)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 goods-0 (pubk c))
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-1)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 875)
  (parent 620)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nb-1 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-1)
    (nm nb-0) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (displaced 7 3 bank 2) price-0 (5 0)
    (enc price-0 nb-0 m price-0 (pubk c))
    (enc price-0 nm m price (pubk c)) (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-1 nb-1 (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-1 goods-0 (pubk c)))))
    ((recv (enc c nb-1 goods-0 (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 876)
  (parent 621)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 4 customer 3) price (5 0)
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 877)
  (parent 621)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nb-1 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 7 4 customer 3) price-0 (5 0)
    (enc price-0 nb-0 m price-0 (pubk c))
    (enc price-0 nm m price (pubk c)) (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-1 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 878)
  (parent 621)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 nb-1 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 879)
  (parent 622)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nb-1 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (5 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 7 4 customer 3) price-0 (6 0)
    (enc price-0 nb-0 m price-0 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-1 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c))))))
  (label 880)
  (parent 623)
  (seen 1208 1209)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 881)
  (parent 626)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 882)
  (parent 627)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 883)
  (parent 627)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 884)
  (parent 628)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 885)
  (parent 631)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (5 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 886)
  (parent 632)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 887)
  (parent 632)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 888)
  (parent 633)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 889)
  (parent 635)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 890)
  (parent 635)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 891)
  (parent 635)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 892)
  (parent 636)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (5 0)
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nc nb-1 (pubk c)) (enc nc nb-1 m nb-1 (pubk c))
    (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 893)
  (parent 637)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1 nb-2
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (5 0)
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nc nb-1 (pubk c)) (enc nc nb-1 m nb-1 (pubk c))
    (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-2 (pubk c))))))
  (label 894)
  (parent 637)
  (seen 636)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (4 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 895)
  (parent 638)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 896)
  (parent 639)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 897)
  (parent 639)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 898)
  (parent 639)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 899)
  (parent 639)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 3 bank 2) price (5 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-1 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 900)
  (parent 641)
  (seen 677)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (displaced 8 6 bank 2) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-2 (pubk c))))))
  (label 901)
  (parent 643)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (5 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 902)
  (parent 644)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 903)
  (parent 644)
  (seen 676)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) acctnum-0
    (4 0) (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc acctnum-0 m price (pubk c))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 904)
  (parent 645)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (4 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 905)
  (parent 645)
  (seen 447)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) acctnum-0
    (5 0) (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
    (enc nc acctnum-0 (pubk c)) (enc nc acctnum-0 m price (pubk c))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))))
  (label 906)
  (parent 646)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 2 bank 2) price (5 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 907)
  (parent 646)
  (seen 447)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (added-strand bank 2) acctnum-0 (5 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
    (enc nc acctnum-0 (pubk c)) (enc nc acctnum-0 m price (pubk c))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 908)
  (parent 646)
  (seen 644 1231)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 909)
  (parent 648)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 910)
  (parent 649)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 911)
  (parent 649)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 912)
  (parent 649)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 913)
  (parent 650)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 914)
  (parent 650)
  (seen 1262)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 915)
  (parent 650)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 5 bank 2) nc (6 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c))))))
  (label 916)
  (parent 654)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 3 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 917)
  (parent 654)
  (seen 1067)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc acctnum-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 7 3 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c acctnum-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 918)
  (parent 654)
  (seen 760 761)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm
      price-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (6 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 919)
  (parent 654)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nc (4 0)
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 920)
  (parent 655)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (4 0)
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 921)
  (parent 655)
  (unrealized (0 0) (0 2) (1 0) (4 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm
      price-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (4 0)
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 922)
  (parent 655)
  (seen 919 1273)
  (unrealized (0 0) (0 2) (1 0) (4 0) (7 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm
      price-0 text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 2 bank 2) nc (4 0)
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 923)
  (parent 655)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 924)
  (parent 656)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 925)
  (parent 656)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 6 2 customer 3) nb-0 (5 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 926)
  (parent 657)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 927)
  (parent 658)
  (seen 924 925)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 928)
  (parent 658)
  (seen 1292)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))))
  (label 929)
  (parent 659)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 930)
  (parent 659)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 931)
  (parent 659)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 932)
  (parent 661)
  (unrealized (0 0) (0 2) (1 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 2 bank 2) price (4 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 933)
  (parent 661)
  (seen 895)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (4 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 934)
  (parent 661)
  (seen 636)
  (unrealized (0 0) (0 2) (1 0) (4 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 935)
  (parent 662)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 936)
  (parent 662)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 3 customer 3) nb-0 (6 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 937)
  (parent 663)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 0 merchant 2) nb-0 (6 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 938)
  (parent 664)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 939)
  (parent 665)
  (seen 935 936)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 940)
  (parent 665)
  (seen 1305)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 nb-2
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 941)
  (parent 665)
  (seen 1308)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 942)
  (parent 668)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 943)
  (parent 669)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 944)
  (parent 671)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 945)
  (parent 672)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 946)
  (parent 673)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 947)
  (parent 673)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 3 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 948)
  (parent 674)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 949)
  (parent 675)
  (seen 947)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 950)
  (parent 675)
  (seen 1311)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 951)
  (parent 676)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 952)
  (parent 677)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 953)
  (parent 677)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 954)
  (parent 677)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 955)
  (parent 678)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 956)
  (parent 678)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 957)
  (parent 678)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 958)
  (parent 681)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 959)
  (parent 681)
  (seen 439)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nm price goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (7 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 8 6 bank 2) nb-1 (7 0)
    (enc nc nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 960)
  (parent 682)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 961)
  (parent 682)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 962)
  (parent 683)
  (seen 1322)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 963)
  (parent 683)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 964)
  (parent 684)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 965)
  (parent 684)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 966)
  (parent 685)
  (seen 1332)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 967)
  (parent 686)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 968)
  (parent 686)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) nb-0 (6 0)
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 969)
  (parent 686)
  (seen 1332)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 970)
  (parent 686)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 971)
  (parent 687)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 972)
  (parent 687)
  (seen 439)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nm price goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (7 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 8 6 bank 2) nb-1 (7 0)
    (enc nc nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 973)
  (parent 688)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 974)
  (parent 688)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 975)
  (parent 689)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 976)
  (parent 690)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 977)
  (parent 691)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 978)
  (parent 691)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 979)
  (parent 692)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 980)
  (parent 693)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 981)
  (parent 694)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 982)
  (parent 694)
  (seen 979)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 983)
  (parent 695)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 984)
  (parent 695)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (5 3)) ((5 0) (6 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 985)
  (parent 699)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (5 3)) ((5 0) (0 0)) ((5 0) (6 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 986)
  (parent 700)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 987)
  (parent 701)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 3 bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 988)
  (parent 701)
  (seen 1271)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 4 bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 989)
  (parent 701)
  (seen 1357)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 990)
  (parent 701)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 991)
  (parent 702)
  (seen 1360)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb nb-0 nm acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum acctnum (pubk hash-0))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m acctnum (pubk c)))
      (send (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 992)
  (parent 703)
  (seen 449)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 3)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 993)
  (parent 704)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 994)
  (parent 705)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 995)
  (parent 705)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 4 bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 996)
  (parent 705)
  (seen 1368)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 997)
  (parent 705)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 998)
  (parent 707)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 999)
  (parent 709)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1000)
  (parent 710)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1001)
  (parent 711)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1002)
  (parent 711)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1003)
  (parent 712)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1004)
  (parent 713)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1005)
  (parent 713)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1006)
  (parent 714)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1007)
  (parent 715)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1008)
  (parent 715)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 7 3 bank 2) price (5 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-2))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1009)
  (parent 715)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 6 bank 2) price (5 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1010)
  (parent 715)
  (seen 1383)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1011)
  (parent 716)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1012)
  (parent 716)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 6 bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1013)
  (parent 716)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1014)
  (parent 717)
  (seen 1384)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (5 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1015)
  (parent 718)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 1016)
  (parent 720)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 1017)
  (parent 721)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1018)
  (parent 722)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 1019)
  (parent 722)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 7 4 customer 3) nb-0 (6 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1020)
  (parent 723)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 1021)
  (parent 725)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 1022)
  (parent 726)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1023)
  (parent 727)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 1024)
  (parent 727)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 7 4 customer 3) nb-0 (6 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1025)
  (parent 728)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 1026)
  (parent 730)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1027)
  (parent 731)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1028)
  (parent 731)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1029)
  (parent 731)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1030)
  (parent 732)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1031)
  (parent 732)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price (5 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1032)
  (parent 732)
  (seen 1411)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 7 3 bank 2) price (5 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1033)
  (parent 732)
  (seen 1413)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1034)
  (parent 733)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1035)
  (parent 733)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1036)
  (parent 733)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1037)
  (parent 734)
  (seen 1412)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (5 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1038)
  (parent 735)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1039)
  (parent 737)
  (seen 1355)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1040)
  (parent 737)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1041)
  (parent 738)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1042)
  (parent 739)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1043)
  (parent 740)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1044)
  (parent 741)
  (seen 1002)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1045)
  (parent 741)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1046)
  (parent 742)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1047)
  (parent 742)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1048)
  (parent 743)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1049)
  (parent 744)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1050)
  (parent 744)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-2))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1051)
  (parent 744)
  (seen 1437 1440)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1052)
  (parent 745)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1053)
  (parent 746)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1054)
  (parent 747)
  (seen 1019)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1055)
  (parent 747)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1056)
  (parent 748)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1057)
  (parent 749)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1058)
  (parent 750)
  (seen 1024)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 3)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1059)
  (parent 750)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))))
  (label 1060)
  (parent 751)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1061)
  (parent 752)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1062)
  (parent 753)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1063)
  (parent 754)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1064)
  (parent 754)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1065)
  (parent 754)
  (seen 1454 1458)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1066)
  (parent 756)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1067)
  (parent 756)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 3 bank 2) price (5 0)
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1068)
  (parent 756)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1069)
  (parent 757)
  (seen 948)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 1070)
  (parent 760)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 goods-0 (pubk c))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 1071)
  (parent 760)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 1072)
  (parent 761)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 1073)
  (parent 762)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 7 6 bank 2) nb-1 (5 0)
    (enc "hash" c acctnum-0 nb-0 nb-1 nb-1 (pubk hash-2))
    (enc acctnum-0 nb-1 m nb-1 (pubk c))
    (enc c acctnum-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 1074)
  (parent 762)
  (seen 1276)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 7 2 bank 2) nb-0 (5 0)
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 1075)
  (parent 762)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (6 3)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 1076)
  (parent 764)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1077)
  (parent 765)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1078)
  (parent 765)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb nb-0 nm acctnum)
  (operation nonce-test (displaced 2 7 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price-0 price-0 (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum acctnum (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum acctnum (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum acctnum (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m acctnum (pubk c)))
      (send (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 1079)
  (parent 766)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (6 3)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 1080)
  (parent 767)
  (seen 699)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 6 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1081)
  (parent 768)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (7 1)) ((1 1) (7 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb nb-0 nm nm-0 acctnum)
  (operation nonce-test (displaced 2 8 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price-1 price-1 (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum acctnum (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum acctnum (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum acctnum (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m acctnum (pubk c)))
      (send (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 1082)
  (parent 769)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc price-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (0 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 5 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 1083)
  (parent 770)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (7 3)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 1084)
  (parent 771)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (8 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 9 7 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1085)
  (parent 772)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc price-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 1086)
  (parent 773)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc price-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (0 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 5 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 1087)
  (parent 773)
  (seen 1083)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc price-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 1088)
  (parent 773)
  (seen 1083)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 3)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation encryption-test (displaced 8 5 bank 2)
    (enc (enc "hash" c price-0 price price price (pubk hash-2))
      (privk b-0)) (7 3))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price))))
  (label 1089)
  (parent 774)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation encryption-test (added-strand bank 2)
    (enc (enc "hash" c price-0 price price price (pubk hash-2))
      (privk b-0)) (7 3))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 1090)
  (parent 774)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1091)
  (parent 775)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1092)
  (parent 776)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1093)
  (parent 777)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1094)
  (parent 778)
  (seen 1002)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1095)
  (parent 778)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1096)
  (parent 779)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1097)
  (parent 779)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1098)
  (parent 780)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1099)
  (parent 781)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1100)
  (parent 781)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-2))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1101)
  (parent 781)
  (seen 1511 1514)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1102)
  (parent 782)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1103)
  (parent 783)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1104)
  (parent 784)
  (seen 1019)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 3)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1105)
  (parent 784)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1106)
  (parent 785)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1107)
  (parent 786)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1108)
  (parent 787)
  (seen 1024)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 3)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1109)
  (parent 787)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))))
  (label 1110)
  (parent 788)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1111)
  (parent 789)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))))
  (label 1112)
  (parent 790)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1113)
  (parent 790)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 1114)
  (parent 791)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1115)
  (parent 791)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1116)
  (parent 792)
  (seen 1531)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1117)
  (parent 793)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1118)
  (parent 793)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1119)
  (parent 793)
  (seen 1534 1537)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (5 3)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price))))
  (label 1120)
  (parent 795)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 2 bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1121)
  (parent 796)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1122)
  (parent 796)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 1123)
  (parent 798)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 7 2 bank 2) nb-0 (6 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 1124)
  (parent 798)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 1125)
  (parent 799)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1126)
  (parent 801)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 1127)
  (parent 803)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 1128)
  (parent 803)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1129)
  (parent 805)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1130)
  (parent 806)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1131)
  (parent 807)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 1132)
  (parent 807)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 6 2 customer 3) nb-1 (5 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1133)
  (parent 808)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1134)
  (parent 809)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1135)
  (parent 810)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1136)
  (parent 811)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1137)
  (parent 811)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 3 customer 3) nb-1 (6 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1138)
  (parent 812)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (4 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1139)
  (parent 813)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1140)
  (parent 814)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1141)
  (parent 814)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 3 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1142)
  (parent 815)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1143)
  (parent 816)
  (seen 1315)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1144)
  (parent 818)
  (seen 1143)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 2) (3 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 3 bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1145)
  (parent 820)
  (seen 1462)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 2) (3 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1146)
  (parent 820)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 7 2 bank 2) nb-0 (6 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 1147)
  (parent 821)
  (seen 1472)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 1148)
  (parent 822)
  (seen 1474)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1149)
  (parent 823)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 1150)
  (parent 825)
  (seen 1128)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1151)
  (parent 826)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 3 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1152)
  (parent 827)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1153)
  (parent 828)
  (seen 1132)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1154)
  (parent 828)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1155)
  (parent 829)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) price (5 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1156)
  (parent 829)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (5 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1157)
  (parent 830)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (5 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1158)
  (parent 830)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 4 customer 3) nb-1 (7 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1159)
  (parent 831)
  (seen 1577)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1160)
  (parent 832)
  (seen 1141)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1161)
  (parent 832)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (0 0)) ((6 0) (5 0)) ((6 2) (2 0))
    ((6 2) (3 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price price (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))))
  (label 1162)
  (parent 835)
  (seen 296)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (5 3)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0))))
  (label 1163)
  (parent 836)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (5 3)) ((4 1) (1 0)) ((5 0) (6 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1164)
  (parent 836)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1165)
  (parent 837)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 3 bank 2) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1166)
  (parent 837)
  (seen 1010)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1167)
  (parent 837)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 7 3 bank 2) price (6 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 1168)
  (parent 838)
  (seen 487 1032)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1169)
  (parent 838)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price price (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))))
  (label 1170)
  (parent 839)
  (seen 318)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 3)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0))))
  (label 1171)
  (parent 840)
  (seen 1164)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 6 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1172)
  (parent 841)
  (seen 1051)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 6 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 1173)
  (parent 842)
  (seen 1065 1451)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 1174)
  (parent 844)
  (seen 329)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1175)
  (parent 845)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 2 7 customer 5) nb (0 2)
    (enc "hash" c nb-0 nb price price (pubk hash))
    (enc nb-0 nb (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))))
  (label 1176)
  (parent 846)
  (seen 340)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 3)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0))))
  (label 1177)
  (parent 847)
  (seen 1164)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 6 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1178)
  (parent 848)
  (seen 1101)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 6 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 1179)
  (parent 849)
  (seen 1119 1525)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 1180)
  (parent 850)
  (seen 354)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price))))
  (label 1181)
  (parent 851)
  (seen 604)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1182)
  (parent 852)
  (seen 851 1278)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1183)
  (parent 853)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1184)
  (parent 853)
  (seen 552)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 6 bank 2) price (5 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1185)
  (parent 853)
  (seen 551 553 554)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 3 bank 2) price (5 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1186)
  (parent 853)
  (seen 1138)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 2 6 customer 5) nb (0 2)
    (enc "hash" c nc nb price price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 1187)
  (parent 854)
  (seen 373)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 5 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1188)
  (parent 855)
  (seen 405)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1189)
  (parent 856)
  (seen 1159)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c))))))
  (label 1190)
  (parent 858)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1191)
  (parent 858)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1192)
  (parent 859)
  (seen 1191)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-1))
            (privk b)) (enc price goods (pubk c))))))
  (label 1193)
  (parent 863)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1194)
  (parent 863)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1195)
  (parent 864)
  (seen 1194)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 1196)
  (parent 866)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (6 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1197)
  (parent 866)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1198)
  (parent 867)
  (seen 1197)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-1))
            (privk b)) (enc price goods (pubk c))))))
  (label 1199)
  (parent 868)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-1))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1200)
  (parent 868)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price goods-0 (pubk c))
    (enc price acctnum m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-1))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1201)
  (parent 869)
  (seen 1200)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 nb-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-1 (0 0)
    (enc c nb-1 goods (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((send (enc c nb-1 goods (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 goods (pubk c))))))
  (label 1202)
  (parent 871)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 nb-1 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-1 (0 0)
    (enc c nb-1 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 goods-0 (pubk c)))))
    ((recv (enc c nb-1 goods-0 (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 1203)
  (parent 871)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 nb-1 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
    (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
    (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-2))
    (enc nb-1 nb-0 (pubk c)) (enc nb-1 nb-1 (pubk c))
    (enc nb-1 goods-0 (pubk c)) (enc nb-1 nb-0 m nb-1 (pubk c))
    (enc nb-1 nm m price (pubk c)) (enc c nb-1 goods-0 (pubk m))
    (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 goods-0 (pubk c)))))
    ((recv (enc c nb-1 goods-0 (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 1204)
  (parent 872)
  (seen 1203)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 1205)
  (parent 873)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1206)
  (parent 873)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 6 3 customer 3) nb-0 (5 0)
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-1)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c))))))
  (label 1207)
  (parent 874)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m acctnum (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1208)
  (parent 875)
  (seen 1206)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0))
    ((3 4) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand bank 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 goods-0 (pubk c))
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c))
    (enc acctnum-0 nm m price (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-1)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c))))))
  (label 1209)
  (parent 875)
  (seen 1627)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 nb-1 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
    (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
    (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-2))
    (enc nb-1 nb-0 (pubk c)) (enc nb-1 nb-1 (pubk c))
    (enc nb-1 goods-0 (pubk c)) (enc nb-1 nb-0 m nb-1 (pubk c))
    (enc nb-1 nm m price (pubk c)) (enc c nb-1 goods-0 (pubk m))
    (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 goods-0 (pubk c)))))
    ((recv (enc c nb-1 goods-0 (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 1210)
  (parent 876)
  (seen 1203)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1211)
  (parent 877)
  (seen 1194)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test
    (contracted (b-0 b) (acctnum-0 acctnum) (hash-2 hash-1)
      (nb-1 goods-0)) acctnum (1 0)
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m acctnum (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1212)
  (parent 878)
  (seen 1206)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (3 3))
    ((3 0) (0 0)) ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0))
    ((3 4) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand bank 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 goods-0 (pubk c))
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c))
    (enc acctnum-0 nm m price (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-1)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c))))))
  (label 1213)
  (parent 878)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c))))))
  (label 1214)
  (parent 882)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1215)
  (parent 882)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1216)
  (parent 883)
  (seen 1215)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (5 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c))))))
  (label 1217)
  (parent 886)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1218)
  (parent 886)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1219)
  (parent 887)
  (seen 1218)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 1220)
  (parent 889)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1221)
  (parent 889)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 2 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1222)
  (parent 890)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1223)
  (parent 891)
  (seen 1221)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1224)
  (parent 891)
  (seen 1639)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 1225)
  (parent 892)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1226)
  (parent 892)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1227)
  (parent 892)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 1228)
  (parent 893)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1229)
  (parent 893)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1230)
  (parent 893)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-1)
    (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-2 (7 0)
    (enc nc nb-2 m nb-2 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-1 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-2)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 acctnum-0 acctnum-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1231)
  (parent 894)
  (seen 951)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 1232)
  (parent 895)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1233)
  (parent 895)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1234)
  (parent 895)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 1235)
  (parent 896)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1236)
  (parent 896)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (displaced 6 2 customer 3) acctnum-0 (5 0)
    (enc nc acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1237)
  (parent 897)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 6 0 merchant 2) acctnum-0 (5 0)
    (enc price acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c))))))
  (label 1238)
  (parent 898)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1239)
  (parent 899)
  (seen 1235 1236)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1240)
  (parent 899)
  (seen 1662)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 1241)
  (parent 899)
  (seen 1665)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test
    (contracted (hash-3 hash-2) (nb-2 nb-1) (acctnum-0 nb-0)) nb-0 (5 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1242)
  (parent 901)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (displaced 8 2 bank 2) nb-1 (5 0)
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-1 (pubk c)) (enc nc nb-1 m nb-1 (pubk c))
    (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1243)
  (parent 901)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 8 3 bank 2) nb-2 (5 0)
    (enc "hash" c nc nb-2 nb-2 nb-2 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-2 nb-2 (pubk hash-3))
    (enc nc nb-2 (pubk c)) (enc nc nb-2 m nb-2 (pubk c))
    (enc c nc nb-2 nb-2 nb-2 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1244)
  (parent 901)
  (seen 1242)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 8 2 bank 2) nb-0 (5 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1245)
  (parent 901)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 1246)
  (parent 902)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1247)
  (parent 902)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 1248)
  (parent 902)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1249)
  (parent 902)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 1250)
  (parent 904)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1251)
  (parent 904)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 1252)
  (parent 904)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1253)
  (parent 904)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc acctnum acctnum price (pubk hash-2))
    (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-2))
            (privk b)) (enc nc acctnum (pubk c))))))
  (label 1254)
  (parent 906)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1255)
  (parent 906)
  (seen 1650)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 1256)
  (parent 906)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1257)
  (parent 906)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-0 price) (hash-2 hash-0))
    price (5 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1258)
  (parent 907)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1259)
  (parent 907)
  (seen 447)
  (unrealized (0 0) (0 2) (1 0) (5 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 1260)
  (parent 910)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1261)
  (parent 910)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 0 merchant 2) nb-0 (6 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 1262)
  (parent 911)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1263)
  (parent 912)
  (seen 1260 1261)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 nb-2
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 1264)
  (parent 912)
  (seen 1700)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 1265)
  (parent 913)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1266)
  (parent 913)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (5 0)
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1267)
  (parent 914)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1268)
  (parent 915)
  (seen 1266)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 1269)
  (parent 915)
  (seen 1267 1700)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c))))))
  (label 1270)
  (parent 916)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (4 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1271)
  (parent 916)
  (unrealized (0 0) (0 2) (1 0) (4 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 2 bank 2) nb-0 (4 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c))))))
  (label 1272)
  (parent 916)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (4 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1273)
  (parent 916)
  (seen 1709)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (4 0)
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1274)
  (parent 917)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 2 bank 2) price (4 0)
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1275)
  (parent 917)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 7 5 bank 2) price (4 0)
    (enc "hash" c acctnum-0 goods-0 price price (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
    (enc acctnum-0 price m price (pubk c))
    (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 1276)
  (parent 918)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 bank 2) nc (6 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1277)
  (parent 919)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 3 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1278)
  (parent 919)
  (seen 1541)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc acctnum-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 8 3 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c acctnum-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c)))))
  (label 1279)
  (parent 919)
  (seen 1123 1124)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1280)
  (parent 920)
  (seen 1221)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1281)
  (parent 920)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 3 customer 3) nc (7 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1282)
  (parent 921)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (7 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1283)
  (parent 921)
  (seen 966)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 3 customer 3) nc (7 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1284)
  (parent 922)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc acctnum-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 8 3 customer 3) nc (7 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c acctnum-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 1285)
  (parent 922)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nc (4 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1286)
  (parent 923)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm
      price-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (4 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1287)
  (parent 923)
  (seen 1709 1729)
  (unrealized (0 0) (0 2) (1 0) (4 0) (7 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1288)
  (parent 925)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 1289)
  (parent 926)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1290)
  (parent 926)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 1291)
  (parent 926)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1292)
  (parent 926)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))))
  (label 1293)
  (parent 929)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1294)
  (parent 929)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 3 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1295)
  (parent 930)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1296)
  (parent 931)
  (seen 1294)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1297)
  (parent 931)
  (seen 1750)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 1298)
  (parent 932)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1299)
  (parent 932)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 1300)
  (parent 932)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1301)
  (parent 932)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1302)
  (parent 936)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1303)
  (parent 937)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 1304)
  (parent 937)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1305)
  (parent 937)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 1306)
  (parent 938)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 1307)
  (parent 938)
  (seen 1761)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1308)
  (parent 938)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1309)
  (parent 947)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1310)
  (parent 948)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1311)
  (parent 948)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1312)
  (parent 949)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 1313)
  (parent 951)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1314)
  (parent 951)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1315)
  (parent 951)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 1316)
  (parent 952)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1317)
  (parent 952)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1318)
  (parent 953)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1319)
  (parent 954)
  (seen 1317)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1320)
  (parent 954)
  (seen 1779)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 1321)
  (parent 955)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1322)
  (parent 955)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 6 3 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1323)
  (parent 956)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1324)
  (parent 957)
  (seen 1322)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1325)
  (parent 957)
  (seen 1784)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nm price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (7 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1326)
  (parent 960)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1327)
  (parent 961)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 1328)
  (parent 962)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 7 4 customer 3) nb-0 (6 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1329)
  (parent 963)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 1330)
  (parent 964)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1331)
  (parent 964)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1332)
  (parent 965)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1333)
  (parent 966)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 1334)
  (parent 967)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1335)
  (parent 967)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1336)
  (parent 968)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1337)
  (parent 969)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1338)
  (parent 970)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1339)
  (parent 970)
  (seen 1336)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nm price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (7 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1340)
  (parent 973)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1341)
  (parent 974)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 1342)
  (parent 977)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1343)
  (parent 977)
  (seen 1309)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1344)
  (parent 978)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1345)
  (parent 979)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 1346)
  (parent 980)
  (seen 1317)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1347)
  (parent 980)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 1348)
  (parent 981)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1349)
  (parent 981)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 1350)
  (parent 983)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1351)
  (parent 983)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (7 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) price-0 (7 0)
    (enc nc price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1352)
  (parent 984)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1353)
  (parent 984)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1354)
  (parent 987)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1355)
  (parent 987)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1356)
  (parent 988)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 4 bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1357)
  (parent 988)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1358)
  (parent 989)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1359)
  (parent 989)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1360)
  (parent 990)
  (seen 1829)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1361)
  (parent 991)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1362)
  (parent 991)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 4 bank 2) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1363)
  (parent 991)
  (seen 1829)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 5 customer 3) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1364)
  (parent 991)
  (seen 1833)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1365)
  (parent 994)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1366)
  (parent 995)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1367)
  (parent 995)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 4 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1368)
  (parent 995)
  (seen 1845)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1369)
  (parent 995)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1370)
  (parent 996)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1371)
  (parent 996)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1372)
  (parent 997)
  (seen 1848 1850)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1373)
  (parent 1000)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1374)
  (parent 1000)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1375)
  (parent 1002)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1376)
  (parent 1003)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1377)
  (parent 1004)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1378)
  (parent 1004)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1379)
  (parent 1005)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1380)
  (parent 1006)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1381)
  (parent 1007)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1382)
  (parent 1007)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 6 bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1383)
  (parent 1007)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1384)
  (parent 1008)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1385)
  (parent 1009)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1386)
  (parent 1009)
  (seen 1865)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price (5 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1387)
  (parent 1009)
  (seen 1868)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 3 bank 2) nb-1 (5 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1388)
  (parent 1010)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (5 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1389)
  (parent 1010)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1390)
  (parent 1011)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1391)
  (parent 1011)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1392)
  (parent 1012)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1393)
  (parent 1013)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1394)
  (parent 1014)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 8 4 customer 3) nb-1 (7 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1395)
  (parent 1015)
  (seen 1871)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))))
  (label 1396)
  (parent 1017)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 1397)
  (parent 1017)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 1398)
  (parent 1019)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 1399)
  (parent 1020)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 1400)
  (parent 1022)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 1401)
  (parent 1022)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 1402)
  (parent 1024)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 1403)
  (parent 1025)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1404)
  (parent 1027)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 1405)
  (parent 1027)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1406)
  (parent 1028)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1407)
  (parent 1029)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1408)
  (parent 1029)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1409)
  (parent 1030)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1410)
  (parent 1030)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 6 5 bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1411)
  (parent 1030)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1412)
  (parent 1031)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 7 3 bank 2) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1413)
  (parent 1032)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1414)
  (parent 1032)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1415)
  (parent 1033)
  (seen 1896)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1416)
  (parent 1033)
  (seen 1895)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1417)
  (parent 1034)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 1418)
  (parent 1034)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1419)
  (parent 1035)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1420)
  (parent 1036)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1421)
  (parent 1036)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1422)
  (parent 1037)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1423)
  (parent 1038)
  (seen 1898)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1424)
  (parent 1039)
  (seen 1825)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1425)
  (parent 1039)
  (seen 1828 1909)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1426)
  (parent 1039)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1427)
  (parent 1040)
  (seen 1835 1911)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1428)
  (parent 1043)
  (seen 1374)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1429)
  (parent 1044)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1430)
  (parent 1045)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1431)
  (parent 1046)
  (seen 1378)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1432)
  (parent 1046)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1433)
  (parent 1047)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1434)
  (parent 1048)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1435)
  (parent 1048)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (6 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-2))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1436)
  (parent 1048)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c nb-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-2))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1437)
  (parent 1048)
  (seen 1922)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1438)
  (parent 1049)
  (seen 1391)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1439)
  (parent 1049)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c nb-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-2))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1440)
  (parent 1049)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1441)
  (parent 1050)
  (seen 1923)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1442)
  (parent 1051)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 1443)
  (parent 1053)
  (seen 1397)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1444)
  (parent 1054)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1445)
  (parent 1055)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 1446)
  (parent 1057)
  (seen 1401)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1447)
  (parent 1058)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1448)
  (parent 1059)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1449)
  (parent 1061)
  (seen 1405)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1450)
  (parent 1061)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1451)
  (parent 1061)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1452)
  (parent 1062)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1453)
  (parent 1062)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1454)
  (parent 1062)
  (seen 1944)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1455)
  (parent 1062)
  (seen 1946)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1456)
  (parent 1063)
  (seen 1418)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1457)
  (parent 1063)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1458)
  (parent 1063)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1459)
  (parent 1064)
  (seen 1945)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1460)
  (parent 1065)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 1461)
  (parent 1066)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1462)
  (parent 1066)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1463)
  (parent 1067)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1464)
  (parent 1068)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-3))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1465)
  (parent 1069)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 3 bank 2) price (5 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1466)
  (parent 1069)
  (seen 1318)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 4 customer 3) price (5 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1467)
  (parent 1069)
  (seen 1463)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 1468)
  (parent 1070)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1469)
  (parent 1070)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc acctnum nm m price (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1470)
  (parent 1071)
  (seen 1469)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 1471)
  (parent 1072)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 goods-0 (pubk c))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 1472)
  (parent 1072)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-1 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 1473)
  (parent 1073)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 goods-0 (pubk c))
    (enc acctnum-0 nb-1 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 1474)
  (parent 1073)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 1475)
  (parent 1074)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (5 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 1476)
  (parent 1074)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 1477)
  (parent 1075)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1478)
  (parent 1077)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1479)
  (parent 1077)
  (seen 1717)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1480)
  (parent 1077)
  (seen 1980)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1481)
  (parent 1077)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1482)
  (parent 1078)
  (seen 1983)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (2 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb nb-0 nm acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum acctnum (pubk hash-0))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m acctnum (pubk c)))
      (send (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 1483)
  (parent 1079)
  (seen 449)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (6 3)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 1484)
  (parent 1080)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1485)
  (parent 1081)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1486)
  (parent 1081)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1487)
  (parent 1081)
  (seen 1991)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (7 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1488)
  (parent 1081)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (3 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb nb-0 nm nm-0 acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum acctnum (pubk hash-0))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m acctnum (pubk c)))
      (send (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 1489)
  (parent 1082)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc price-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (0 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 1490)
  (parent 1083)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (7 3)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 1491)
  (parent 1084)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (6 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 1492)
  (parent 1084)
  (seen 985)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (8 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1493)
  (parent 1085)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 9 2 bank 2) price (8 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1494)
  (parent 1085)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 9 5 bank 2) price (8 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1495)
  (parent 1085)
  (seen 2002)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (8 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1496)
  (parent 1085)
  (unrealized (0 0) (0 2) (1 0) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb price)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (7 1)) ((1 1) (7 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation nonce-test (displaced 2 8 customer 5) nb (0 2)
    (enc "hash" c price nb price-1 price-1 (pubk hash))
    (enc price nb (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price price acctnum acctnum (pubk hash-0))
            (privk b)) (enc price price (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum acctnum (pubk hash-1))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-2))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 1497)
  (parent 1086)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc price-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (0 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 1498)
  (parent 1087)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 3 (acctnum price) (nc price-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (3 0)) ((2 2) (6 0)) ((3 1) (4 0)) ((4 1) (1 0))
    ((5 1) (4 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 1499)
  (parent 1088)
  (unrealized (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 3)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 1500)
  (parent 1089)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (8 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 7 customer 3) price (8 0)
    (enc price-0 price m price (pubk c)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 1501)
  (parent 1090)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1502)
  (parent 1093)
  (seen 1374)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1503)
  (parent 1094)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1504)
  (parent 1095)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1505)
  (parent 1096)
  (seen 1378)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1506)
  (parent 1096)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1507)
  (parent 1097)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1508)
  (parent 1098)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1509)
  (parent 1098)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (6 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-2))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1510)
  (parent 1098)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c nb-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-2))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1511)
  (parent 1098)
  (seen 2024)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1512)
  (parent 1099)
  (seen 1391)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1513)
  (parent 1099)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c nb-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-2))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1514)
  (parent 1099)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1515)
  (parent 1100)
  (seen 2025)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1516)
  (parent 1101)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 1517)
  (parent 1103)
  (seen 1397)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1518)
  (parent 1104)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1519)
  (parent 1105)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 1520)
  (parent 1107)
  (seen 1401)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1521)
  (parent 1108)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1522)
  (parent 1109)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1523)
  (parent 1111)
  (seen 1405)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1524)
  (parent 1111)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1525)
  (parent 1111)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))))
  (label 1526)
  (parent 1112)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 1527)
  (parent 1112)
  (seen 484)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1528)
  (parent 1113)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))))
  (label 1529)
  (parent 1114)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1530)
  (parent 1114)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1531)
  (parent 1115)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1532)
  (parent 1116)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1533)
  (parent 1116)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1534)
  (parent 1116)
  (seen 2052 2055)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1535)
  (parent 1117)
  (seen 1418)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1536)
  (parent 1117)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1537)
  (parent 1117)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1538)
  (parent 1118)
  (seen 2056)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1539)
  (parent 1119)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1540)
  (parent 1121)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (2 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1541)
  (parent 1121)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (2 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 4 bank 2) price (6 0)
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1542)
  (parent 1121)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1543)
  (parent 1122)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc acctnum nm m price (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 1544)
  (parent 1123)
  (seen 1469)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 1545)
  (parent 1124)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 1546)
  (parent 1125)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) nb-1 (6 0)
    (enc "hash" c acctnum-0 nb-0 nb-1 nb-1 (pubk hash-2))
    (enc acctnum-0 nb-1 m nb-1 (pubk c))
    (enc c acctnum-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 1547)
  (parent 1125)
  (seen 1721)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) nb-0 (6 0)
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 1548)
  (parent 1125)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 1549)
  (parent 1128)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1550)
  (parent 1130)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1551)
  (parent 1130)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 1552)
  (parent 1132)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 1553)
  (parent 1133)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1554)
  (parent 1134)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1555)
  (parent 1134)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1556)
  (parent 1135)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1557)
  (parent 1136)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 1558)
  (parent 1136)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 3 customer 3) nb-1 (6 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1559)
  (parent 1137)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (4 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1560)
  (parent 1138)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 1561)
  (parent 1139)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1562)
  (parent 1141)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1563)
  (parent 1142)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 2) (3 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1564)
  (parent 1145)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 2) (3 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 4 bank 2) price (6 0)
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1565)
  (parent 1145)
  (seen 1961)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 2) (3 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1566)
  (parent 1146)
  (seen 1311)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) nb-1 (6 0)
    (enc "hash" c acctnum-0 nb-0 nb-1 nb-1 (pubk hash-2))
    (enc acctnum-0 nb-1 m nb-1 (pubk c))
    (enc c acctnum-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 1567)
  (parent 1148)
  (seen 1974)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nb-0 nm price goods-0 acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) nb-0 (6 0)
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 1568)
  (parent 1148)
  (seen 1977)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 1569)
  (parent 1150)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1570)
  (parent 1152)
  (seen 1551)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1571)
  (parent 1153)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 7 3 customer 3) nb-1 (6 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1572)
  (parent 1154)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1573)
  (parent 1155)
  (seen 1555)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1574)
  (parent 1156)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1575)
  (parent 1157)
  (seen 1558)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1576)
  (parent 1157)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 4 customer 3) nb-1 (7 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1577)
  (parent 1158)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (5 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1578)
  (parent 1159)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1579)
  (parent 1160)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1580)
  (parent 1161)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (5 3)) ((4 1) (1 0)) ((5 0) (6 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1581)
  (parent 1164)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1582)
  (parent 1165)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1583)
  (parent 1166)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 4 bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1584)
  (parent 1166)
  (seen 1388)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1585)
  (parent 1166)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1586)
  (parent 1167)
  (seen 1395 2110)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1587)
  (parent 1168)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1588)
  (parent 1169)
  (seen 1423 2114)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 3)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0))))
  (label 1589)
  (parent 1171)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1590)
  (parent 1172)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1591)
  (parent 1172)
  (seen 1437)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1592)
  (parent 1172)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (7 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 1593)
  (parent 1173)
  (seen 750 1454)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1594)
  (parent 1173)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1595)
  (parent 1175)
  (seen 1222 1465)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 3)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0))))
  (label 1596)
  (parent 1177)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1597)
  (parent 1178)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 1598)
  (parent 1178)
  (seen 1511)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1599)
  (parent 1178)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (7 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 1600)
  (parent 1179)
  (seen 1534 2046)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1601)
  (parent 1179)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nm price-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price))))
  (label 1602)
  (parent 1181)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1603)
  (parent 1182)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1604)
  (parent 1183)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 8 6 bank 2) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1605)
  (parent 1184)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (5 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1606)
  (parent 1186)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1607)
  (parent 1186)
  (seen 810)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 6 bank 2) price (5 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1608)
  (parent 1186)
  (seen 811 812)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 4 bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1609)
  (parent 1188)
  (seen 1564 1713)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (6 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1610)
  (parent 1188)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1611)
  (parent 1189)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1612)
  (parent 1189)
  (seen 829)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-2))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1613)
  (parent 1189)
  (seen 828 830 831)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1614)
  (parent 1189)
  (seen 1577)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1615)
  (parent 1191)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1616)
  (parent 1192)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-1))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1617)
  (parent 1194)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-1))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1618)
  (parent 1195)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (6 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1619)
  (parent 1197)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1620)
  (parent 1198)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-1))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1621)
  (parent 1200)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-1))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1622)
  (parent 1201)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 nb-1 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-1 (0 0)
    (enc nb-1 nm m price (pubk c)) (enc c nb-1 goods (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((send (enc c nb-1 goods (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 goods (pubk c)))))
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 1623)
  (parent 1203)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 nb-1 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-1 (0 0)
    (enc c nb-1 goods (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((send (enc c nb-1 goods (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 goods (pubk c)))))
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 1624)
  (parent 1204)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (1 0))
    ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1625)
  (parent 1206)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum acctnum (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 m acctnum (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum acctnum (pubk c))))))
  (label 1626)
  (parent 1207)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0 (pubk hash-1))
    (enc acctnum-0 acctnum-0 (pubk c)) (enc acctnum-0 nb-0 (pubk c))
    (enc acctnum-0 goods-0 (pubk c))
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-1)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 1627)
  (parent 1207)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1628)
  (parent 1208)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 nb-1 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-1 (0 0)
    (enc c nb-1 goods (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((send (enc c nb-1 goods (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 goods (pubk c)))))
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 1629)
  (parent 1210)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-1))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1630)
  (parent 1211)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1631)
  (parent 1212)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 7 3 customer 3) nb-0 (6 0)
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-1)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c))))))
  (label 1632)
  (parent 1213)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1633)
  (parent 1215)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1634)
  (parent 1216)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1635)
  (parent 1218)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 4) (1 0))
    ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1636)
  (parent 1219)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1637)
  (parent 1221)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1638)
  (parent 1222)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1639)
  (parent 1222)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1640)
  (parent 1223)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 1641)
  (parent 1225)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1642)
  (parent 1225)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 3 customer 3) nb-0 (6 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1643)
  (parent 1226)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1644)
  (parent 1227)
  (seen 1642)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1645)
  (parent 1227)
  (seen 2159)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 1646)
  (parent 1228)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1647)
  (parent 1228)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 2 bank 2) nb-1 (6 0)
    (enc nc nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1648)
  (parent 1229)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 3 customer 3) nb-0 (6 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1649)
  (parent 1229)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (displaced 7 3 customer 3) nb-1 (6 0)
    (enc nc nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-2)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc goods-0 acctnum-0 acctnum-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1650)
  (parent 1229)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1651)
  (parent 1230)
  (seen 1647)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1652)
  (parent 1230)
  (seen 2163 2165)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 8 3 bank 2) acctnum-1 (5 0)
    (enc "hash" c nc acctnum-1 acctnum-1 acctnum-1 (pubk hash-2))
    (enc "hash" c nc nb-1 acctnum-1 acctnum-1 (pubk hash-3))
    (enc nc acctnum-1 (pubk c)) (enc nc acctnum-1 m acctnum-1 (pubk c))
    (enc c nc acctnum-1 acctnum-1 acctnum-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1653)
  (parent 1231)
  (seen 951)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 1654)
  (parent 1232)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1655)
  (parent 1232)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 3 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1656)
  (parent 1233)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1657)
  (parent 1234)
  (seen 1655)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1658)
  (parent 1234)
  (seen 2172)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1659)
  (parent 1236)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
    (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc acctnum m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1660)
  (parent 1237)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 1661)
  (parent 1237)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc acctnum-0 m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1662)
  (parent 1237)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c))))))
  (label 1663)
  (parent 1238)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 1664)
  (parent 1238)
  (seen 2176)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1665)
  (parent 1238)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 1666)
  (parent 1242)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1667)
  (parent 1242)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1668)
  (parent 1242)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (5 0)
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-1 (pubk c)) (enc nc nb-1 m nb-1 (pubk c))
    (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1669)
  (parent 1243)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (displaced 8 3 bank 2) nb-1 (5 0)
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-1 (pubk c)) (enc nc nb-1 m nb-1 (pubk c))
    (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1670)
  (parent 1243)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (5 0)
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-1 (pubk c)) (enc nc nb-1 m nb-1 (pubk c))
    (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nc nb-2 (pubk c))))))
  (label 1671)
  (parent 1243)
  (seen 1670)
  (unrealized (0 0) (0 2) (1 0) (5 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (4 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1672)
  (parent 1245)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 1673)
  (parent 1246)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1674)
  (parent 1246)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 3 customer 3) nb-0 (6 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1675)
  (parent 1247)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 0 merchant 2) nb-0 (6 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 1676)
  (parent 1248)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1677)
  (parent 1249)
  (seen 1673 1674)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1678)
  (parent 1249)
  (seen 2199)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 1679)
  (parent 1249)
  (seen 2202)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 1680)
  (parent 1250)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1681)
  (parent 1250)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (displaced 7 3 customer 3) acctnum-0 (6 0)
    (enc nc acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1682)
  (parent 1251)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 7 0 merchant 2) acctnum-0 (6 0)
    (enc price acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c))))))
  (label 1683)
  (parent 1252)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1684)
  (parent 1253)
  (seen 1680 1681)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1685)
  (parent 1253)
  (seen 2206)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 1686)
  (parent 1253)
  (seen 2209)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-2))
            (privk b)) (enc nc acctnum (pubk c))))))
  (label 1687)
  (parent 1254)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-2))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1688)
  (parent 1254)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 2 bank 2) acctnum-0 (6 0)
    (enc nc acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1689)
  (parent 1255)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (displaced 7 3 customer 3) acctnum-0 (6 0)
    (enc nc acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1690)
  (parent 1255)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 7 0 merchant 2) acctnum-0 (6 0)
    (enc price acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price acctnum-0 (pubk c))))))
  (label 1691)
  (parent 1256)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc acctnum acctnum price (pubk hash-2))
    (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-2))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1692)
  (parent 1257)
  (seen 1687 1688)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc acctnum-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1693)
  (parent 1257)
  (seen 2169 2213 2216)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 1694)
  (parent 1257)
  (seen 2219)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))))
  (label 1695)
  (parent 1258)
  (seen 1351)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1696)
  (parent 1258)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1697)
  (parent 1258)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1698)
  (parent 1261)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 1699)
  (parent 1262)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1700)
  (parent 1262)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1701)
  (parent 1266)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) price (5 0)
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1702)
  (parent 1267)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1703)
  (parent 1268)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c))))))
  (label 1704)
  (parent 1270)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1705)
  (parent 1270)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 3 customer 3) nb-0 (7 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1706)
  (parent 1271)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (4 0)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (7 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1707)
  (parent 1271)
  (unrealized (0 0) (0 2) (1 0) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c))))))
  (label 1708)
  (parent 1272)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1709)
  (parent 1272)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1710)
  (parent 1273)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (4 0)) ((8 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (4 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1711)
  (parent 1273)
  (seen 1707)
  (unrealized (0 0) (0 2) (1 0) (4 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1712)
  (parent 1274)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1713)
  (parent 1274)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (4 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1714)
  (parent 1275)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 1715)
  (parent 1276)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1716)
  (parent 1277)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (4 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1717)
  (parent 1277)
  (unrealized (0 0) (0 2) (1 0) (4 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) nb-0 (4 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1718)
  (parent 1277)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (4 0)
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1719)
  (parent 1278)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 2 bank 2) price (4 0)
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1720)
  (parent 1278)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 8 5 bank 2) price-0 (4 0)
    (enc "hash" c acctnum-0 goods-0 price-0 price-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 price-0 price-0 (pubk hash-1))
    (enc acctnum-0 price-0 m price-0 (pubk c))
    (enc c acctnum-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 1721)
  (parent 1279)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1722)
  (parent 1280)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1723)
  (parent 1281)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 5 merchant 2) nc (6 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1724)
  (parent 1281)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nm-0
      price-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 0) (7 0)) ((2 2) (4 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm nm-0)
  (operation nonce-test (added-strand merchant 2) nc (6 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm-0 m price-1 (pubk c)))))
  (label 1725)
  (parent 1281)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1726)
  (parent 1282)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (4 0)
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1727)
  (parent 1284)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (4 0)
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1728)
  (parent 1284)
  (unrealized (0 0) (0 2) (1 0) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 2 bank 2) price (4 0)
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1729)
  (parent 1284)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc acctnum-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test
    (contracted (acctnum-1 acctnum-0) (hash-2 hash-1) (nb-0 goods-0))
    price (4 0)
    (enc "hash" c acctnum-0 goods-0 price price (pubk hash-1))
    (enc acctnum-0 price m price (pubk c))
    (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 1730)
  (parent 1285)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 8 5 bank 2) price-0 (4 0)
    (enc "hash" c acctnum-0 goods-0 price-0 price-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 price-0 price-0 (pubk hash-1))
    (enc acctnum-0 price-0 m price-0 (pubk c))
    (enc c acctnum-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 1731)
  (parent 1285)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc acctnum-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) price (4 0)
    (enc "hash" c acctnum-0 goods-0 price price (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 price price (pubk hash-2))
    (enc acctnum-0 price m price (pubk c))
    (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c acctnum-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 1732)
  (parent 1285)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1733)
  (parent 1286)
  (seen 1655)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1734)
  (parent 1286)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc acctnum-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 8 3 customer 3) nc (7 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c acctnum-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 1735)
  (parent 1287)
  (seen 2265)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm
      price-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (7 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1736)
  (parent 1287)
  (seen 1718 1720)
  (unrealized (0 0) (0 2) (1 0) (4 0) (6 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 1737)
  (parent 1289)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1738)
  (parent 1289)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 5 bank 2) nb-1 (6 0)
    (enc nc nb-1 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1739)
  (parent 1290)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 2 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1740)
  (parent 1290)
  (seen 2164 2165)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (6 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1741)
  (parent 1290)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (displaced 7 2 customer 3) nb-1 (6 0)
    (enc nc nb-1 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1742)
  (parent 1290)
  (seen 2214 2216)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 0 merchant 2) nb-0 (6 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 1743)
  (parent 1291)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1744)
  (parent 1292)
  (seen 1737 1738)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1745)
  (parent 1292)
  (seen 2274 2277)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0 nb-2
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 1746)
  (parent 1292)
  (seen 2281)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1747)
  (parent 1294)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 1748)
  (parent 1295)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1749)
  (parent 1295)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1750)
  (parent 1295)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1751)
  (parent 1296)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 1752)
  (parent 1298)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1753)
  (parent 1298)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (6 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1754)
  (parent 1299)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 0 merchant 2) nb-0 (6 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 1755)
  (parent 1300)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1756)
  (parent 1301)
  (seen 1752 1753)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1757)
  (parent 1301)
  (seen 2294)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0 nb-2
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 1758)
  (parent 1301)
  (seen 2297)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 1759)
  (parent 1303)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1760)
  (parent 1303)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 0 merchant 2) nb-0 (7 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 1761)
  (parent 1304)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1762)
  (parent 1305)
  (seen 1759 1760)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 nb-2
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (6 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0))
    ((6 1) (1 0)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 1763)
  (parent 1305)
  (seen 2300)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 1764)
  (parent 1306)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1765)
  (parent 1306)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (6 0)
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1766)
  (parent 1307)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 1767)
  (parent 1308)
  (seen 1765)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 1768)
  (parent 1308)
  (seen 1766 2300)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 1769)
  (parent 1310)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1770)
  (parent 1310)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1771)
  (parent 1311)
  (seen 1770)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 1772)
  (parent 1313)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1773)
  (parent 1313)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 3 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1774)
  (parent 1314)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1775)
  (parent 1315)
  (seen 1773)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1776)
  (parent 1315)
  (seen 2308)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1777)
  (parent 1317)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1778)
  (parent 1318)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1779)
  (parent 1318)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1780)
  (parent 1319)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1781)
  (parent 1322)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 1782)
  (parent 1323)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1783)
  (parent 1323)
  (seen 1963)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1784)
  (parent 1323)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1785)
  (parent 1324)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (7 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 1786)
  (parent 1326)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1787)
  (parent 1326)
  (seen 942)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 1788)
  (parent 1327)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1789)
  (parent 1327)
  (seen 944)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 1790)
  (parent 1329)
  (seen 2314)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1791)
  (parent 1329)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 1792)
  (parent 1330)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1793)
  (parent 1330)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1794)
  (parent 1331)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1795)
  (parent 1332)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 1796)
  (parent 1333)
  (seen 1773)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (2 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1797)
  (parent 1333)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 1798)
  (parent 1334)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1799)
  (parent 1334)
  (seen 1309)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1800)
  (parent 1335)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1801)
  (parent 1336)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 1802)
  (parent 1337)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1803)
  (parent 1337)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 1804)
  (parent 1338)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 1805)
  (parent 1338)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (7 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 1806)
  (parent 1340)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1807)
  (parent 1340)
  (seen 942)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (5 3)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 1808)
  (parent 1341)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (4 3)) ((3 1) (1 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (5 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 1809)
  (parent 1341)
  (seen 944)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1810)
  (parent 1344)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 1811)
  (parent 1345)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1812)
  (parent 1345)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 1813)
  (parent 1346)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 1814)
  (parent 1347)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 1815)
  (parent 1348)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1816)
  (parent 1348)
  (seen 1781)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1817)
  (parent 1349)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1818)
  (parent 1351)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (7 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1819)
  (parent 1352)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 1820)
  (parent 1353)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1821)
  (parent 1354)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1822)
  (parent 1354)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1823)
  (parent 1355)
  (seen 1822)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1824)
  (parent 1356)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1825)
  (parent 1356)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1826)
  (parent 1357)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1827)
  (parent 1358)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (7 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1828)
  (parent 1358)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1829)
  (parent 1359)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1830)
  (parent 1360)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1831)
  (parent 1360)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 3 bank 2) price (6 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1832)
  (parent 1360)
  (seen 2360)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 5 customer 3) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1833)
  (parent 1360)
  (seen 1706)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1834)
  (parent 1361)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1835)
  (parent 1361)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1836)
  (parent 1362)
  (seen 2363)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1837)
  (parent 1363)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1838)
  (parent 1364)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1839)
  (parent 1364)
  (seen 1836)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1840)
  (parent 1365)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (7 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1841)
  (parent 1365)
  (seen 985)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1842)
  (parent 1366)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1843)
  (parent 1367)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1844)
  (parent 1367)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 4 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1845)
  (parent 1367)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1846)
  (parent 1368)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (7 0)) ((5 1) (4 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1847)
  (parent 1368)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1848)
  (parent 1369)
  (seen 2383)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1849)
  (parent 1370)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1850)
  (parent 1371)
  (seen 2383)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1851)
  (parent 1372)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1852)
  (parent 1372)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1853)
  (parent 1372)
  (seen 2387)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1854)
  (parent 1374)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1855)
  (parent 1376)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1856)
  (parent 1376)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1857)
  (parent 1378)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1858)
  (parent 1379)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1859)
  (parent 1380)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1860)
  (parent 1381)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1861)
  (parent 1381)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1862)
  (parent 1382)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1863)
  (parent 1383)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1864)
  (parent 1384)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 3 bank 2) price (5 0)
    (enc "hash" c nb-1 price price price (pubk hash-3))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-2))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1865)
  (parent 1384)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1866)
  (parent 1385)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1867)
  (parent 1385)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1868)
  (parent 1385)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1869)
  (parent 1387)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (5 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1870)
  (parent 1388)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 8 4 customer 3) nb-1 (7 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1871)
  (parent 1389)
  (seen 2411)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 1872)
  (parent 1391)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1873)
  (parent 1392)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1874)
  (parent 1393)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 1875)
  (parent 1393)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1876)
  (parent 1394)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (5 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1877)
  (parent 1395)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 1878)
  (parent 1397)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 1879)
  (parent 1399)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 1880)
  (parent 1399)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 1881)
  (parent 1401)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 1882)
  (parent 1403)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 1883)
  (parent 1403)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 1884)
  (parent 1405)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 1885)
  (parent 1406)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1886)
  (parent 1407)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 1887)
  (parent 1407)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1888)
  (parent 1408)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1889)
  (parent 1409)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 1890)
  (parent 1409)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 7 3 customer 3) price (6 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1891)
  (parent 1410)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1892)
  (parent 1411)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1893)
  (parent 1411)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1894)
  (parent 1412)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 3 bank 2) price (5 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1895)
  (parent 1412)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1896)
  (parent 1413)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1897)
  (parent 1413)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1898)
  (parent 1414)
  (seen 2435)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1899)
  (parent 1415)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1900)
  (parent 1415)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 1901)
  (parent 1418)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 1902)
  (parent 1419)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1903)
  (parent 1420)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 1904)
  (parent 1420)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1905)
  (parent 1421)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 1906)
  (parent 1422)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1907)
  (parent 1423)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1908)
  (parent 1424)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1909)
  (parent 1424)
  (seen 2355)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (7 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1910)
  (parent 1425)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1911)
  (parent 1426)
  (seen 2362 2448)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1912)
  (parent 1427)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 5 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1913)
  (parent 1427)
  (seen 2374 2448)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1914)
  (parent 1427)
  (seen 2376 2451)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1915)
  (parent 1428)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1916)
  (parent 1430)
  (seen 1856)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 1917)
  (parent 1431)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1918)
  (parent 1432)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1919)
  (parent 1433)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1920)
  (parent 1434)
  (seen 1861)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1921)
  (parent 1434)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c nb-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-2))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1922)
  (parent 1434)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1923)
  (parent 1435)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1924)
  (parent 1436)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1925)
  (parent 1436)
  (seen 2461)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1926)
  (parent 1436)
  (seen 2464)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 3 bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1927)
  (parent 1437)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1928)
  (parent 1437)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1929)
  (parent 1438)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1930)
  (parent 1439)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 1931)
  (parent 1440)
  (seen 1875)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1932)
  (parent 1441)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 5 customer 3) nb-1 (8 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 1933)
  (parent 1442)
  (seen 2467)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))))
  (label 1934)
  (parent 1443)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 1935)
  (parent 1445)
  (seen 1880)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 1936)
  (parent 1446)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 1937)
  (parent 1448)
  (seen 1883)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1938)
  (parent 1449)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1939)
  (parent 1450)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1940)
  (parent 1451)
  (seen 1887)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1941)
  (parent 1451)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1942)
  (parent 1452)
  (seen 1890)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1943)
  (parent 1452)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1944)
  (parent 1452)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1945)
  (parent 1453)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 3 bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1946)
  (parent 1454)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1947)
  (parent 1454)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 1948)
  (parent 1455)
  (seen 2483)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1949)
  (parent 1455)
  (seen 2482)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 1950)
  (parent 1456)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1951)
  (parent 1457)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 1952)
  (parent 1458)
  (seen 1904)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1953)
  (parent 1458)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 1954)
  (parent 1459)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 1955)
  (parent 1460)
  (seen 2485)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 1956)
  (parent 1461)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1957)
  (parent 1461)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1958)
  (parent 1462)
  (seen 1957)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1959)
  (parent 1463)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 1960)
  (parent 1464)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 1961)
  (parent 1464)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-3))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1962)
  (parent 1465)
  (seen 1774)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 1963)
  (parent 1467)
  (seen 2166 2167)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (added-strand bank 2) price (5 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-3))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 1964)
  (parent 1467)
  (seen 1962)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1965)
  (parent 1469)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1966)
  (parent 1470)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 1967)
  (parent 1471)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1968)
  (parent 1471)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc acctnum nm m price (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1969)
  (parent 1472)
  (seen 1968)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 1970)
  (parent 1473)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1971)
  (parent 1473)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-1 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 1972)
  (parent 1474)
  (seen 1971)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 1973)
  (parent 1475)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 goods-0 (pubk c)) (enc acctnum-0 nb-0 (pubk c))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 1974)
  (parent 1475)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 1975)
  (parent 1476)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-1 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 1976)
  (parent 1477)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 goods-0 (pubk c))
    (enc acctnum-0 nb-1 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 1977)
  (parent 1477)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1978)
  (parent 1478)
  (seen 1822)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1979)
  (parent 1479)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1980)
  (parent 1479)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1981)
  (parent 1480)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (7 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1982)
  (parent 1480)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1983)
  (parent 1481)
  (seen 2515)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1984)
  (parent 1482)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1985)
  (parent 1482)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 5 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1986)
  (parent 1482)
  (seen 2515)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1987)
  (parent 1482)
  (seen 2519)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1988)
  (parent 1485)
  (seen 1841)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1989)
  (parent 1486)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 3 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1990)
  (parent 1486)
  (seen 1979)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1991)
  (parent 1486)
  (seen 2528)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0))
    ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1992)
  (parent 1486)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 1993)
  (parent 1487)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1994)
  (parent 1487)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 1995)
  (parent 1488)
  (seen 2531 2533)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nb-0 nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm acctnum)
    (goods goods) (price acctnum) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 0) (4 0))
    ((5 2) (3 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nb nb-0 nm nm-0 acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((recv (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m acctnum (pubk c)))
      (send (enc c nb-0 acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 1996)
  (parent 1489)
  (seen 449)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb price)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (7 1)) ((1 1) (7 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (0 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation nonce-test (displaced 2 8 customer 5) nb (0 2)
    (enc "hash" c price nb price-1 price-1 (pubk hash))
    (enc price nb (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price price acctnum acctnum (pubk hash-0))
            (privk b)) (enc price price (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum acctnum (pubk hash-1))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-2))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 1997)
  (parent 1490)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (6 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 4 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 1998)
  (parent 1492)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (8 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 1999)
  (parent 1493)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2000)
  (parent 1494)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 9 3 bank 2) price (8 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2001)
  (parent 1494)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 9 5 bank 2) price (8 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2002)
  (parent 1494)
  (seen 2544)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (8 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2003)
  (parent 1494)
  (unrealized (0 0) (0 2) (1 0) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2004)
  (parent 1495)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2005)
  (parent 1495)
  (unrealized (0 0) (0 2) (1 0) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2006)
  (parent 1496)
  (seen 2547 2549)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (1 0)) ((3 1) (2 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (3 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0))
    ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum acctnum (pubk hash-0))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 2007)
  (parent 1497)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb price)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (7 1)) ((1 1) (7 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation nonce-test (displaced 2 8 customer 5) nb (0 2)
    (enc "hash" c price nb price-1 price-1 (pubk hash))
    (enc price nb (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price price acctnum acctnum (pubk hash-0))
            (privk b)) (enc price price (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum acctnum (pubk hash-1))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-2))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 2008)
  (parent 1498)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 hash-1 hash-2 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb price)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (7 1)) ((1 1) (7 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk hash-2) (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation nonce-test (displaced 2 8 customer 5) nb (0 2)
    (enc "hash" c price nb price-1 price-1 (pubk hash))
    (enc price nb (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price price acctnum acctnum (pubk hash-0))
            (privk b)) (enc price price (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum acctnum (pubk hash-1))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-2))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 2009)
  (parent 1499)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 3)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 2010)
  (parent 1500)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (7 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 2011)
  (parent 1500)
  (seen 1998)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (6 3)) ((6 0) (4 0))
    ((6 0) (7 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 2012)
  (parent 1500)
  (seen 1998)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (8 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 2013)
  (parent 1501)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 2 bank 2) price (8 0)
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 2014)
  (parent 1501)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 5 bank 2) price (8 0)
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 2015)
  (parent 1501)
  (seen 2561)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (8 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 2016)
  (parent 1501)
  (unrealized (0 0) (0 2) (1 0) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2017)
  (parent 1502)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2018)
  (parent 1504)
  (seen 1856)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 2019)
  (parent 1505)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2020)
  (parent 1506)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2021)
  (parent 1507)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2022)
  (parent 1508)
  (seen 1861)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2023)
  (parent 1508)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c nb-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-2))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2024)
  (parent 1508)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2025)
  (parent 1509)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2026)
  (parent 1510)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2027)
  (parent 1510)
  (seen 2573)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2028)
  (parent 1510)
  (seen 2576)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 3 bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2029)
  (parent 1511)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2030)
  (parent 1511)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2031)
  (parent 1512)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2032)
  (parent 1513)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2033)
  (parent 1514)
  (seen 1875)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2034)
  (parent 1515)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 5 customer 3) nb-1 (8 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2035)
  (parent 1516)
  (seen 2579)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))))
  (label 2036)
  (parent 1517)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 2037)
  (parent 1519)
  (seen 1880)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 2038)
  (parent 1520)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 2039)
  (parent 1522)
  (seen 1883)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2040)
  (parent 1523)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2041)
  (parent 1524)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2042)
  (parent 1525)
  (seen 1887)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2043)
  (parent 1525)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2044)
  (parent 1528)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2045)
  (parent 1528)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2046)
  (parent 1528)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))))
  (label 2047)
  (parent 1529)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 2048)
  (parent 1529)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2049)
  (parent 1530)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2050)
  (parent 1531)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2051)
  (parent 1531)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2052)
  (parent 1531)
  (seen 2600)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2053)
  (parent 1532)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2054)
  (parent 1532)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2055)
  (parent 1532)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2056)
  (parent 1533)
  (seen 2601)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2057)
  (parent 1534)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2058)
  (parent 1535)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2059)
  (parent 1536)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2060)
  (parent 1537)
  (seen 1904)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2061)
  (parent 1537)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2062)
  (parent 1538)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2063)
  (parent 1539)
  (seen 2609)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 2064)
  (parent 1540)
  (seen 1957)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2065)
  (parent 1541)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (2 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2066)
  (parent 1542)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2067)
  (parent 1543)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-3))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2068)
  (parent 1543)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 4 bank 2) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2069)
  (parent 1543)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (6 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2070)
  (parent 1543)
  (seen 2065)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 2071)
  (parent 1544)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc acctnum nm m price (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 2072)
  (parent 1545)
  (seen 1968)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-1 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 2073)
  (parent 1546)
  (seen 1971)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nm price goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 2074)
  (parent 1547)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 2075)
  (parent 1547)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 2076)
  (parent 1548)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2077)
  (parent 1551)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2078)
  (parent 1553)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2079)
  (parent 1553)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2080)
  (parent 1555)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2081)
  (parent 1556)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2082)
  (parent 1556)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2083)
  (parent 1558)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2084)
  (parent 1559)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2085)
  (parent 1560)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2086)
  (parent 1561)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2087)
  (parent 1561)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2088)
  (parent 1563)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2089)
  (parent 1563)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 2) (3 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2090)
  (parent 1564)
  (seen 2496)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 2) (3 0)) ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-3))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2091)
  (parent 1566)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 2) (3 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 4 bank 2) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2092)
  (parent 1566)
  (seen 1779)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 2) (3 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (6 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2093)
  (parent 1566)
  (seen 2090 2167)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nm nb-0)
  (operation nonce-test (displaced 8 4 bank 2) nb-0 (6 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))))
  (label 2094)
  (parent 1567)
  (seen 2245)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 2095)
  (parent 1567)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2096)
  (parent 1570)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2097)
  (parent 1572)
  (seen 2079)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2098)
  (parent 1573)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2099)
  (parent 1574)
  (seen 2082)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2100)
  (parent 1575)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 4 customer 3) nb-1 (7 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2101)
  (parent 1576)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (5 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2102)
  (parent 1577)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2103)
  (parent 1578)
  (seen 2087)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2104)
  (parent 1580)
  (seen 2089)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 2105)
  (parent 1582)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (7 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2106)
  (parent 1582)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 2107)
  (parent 1583)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 2108)
  (parent 1584)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2109)
  (parent 1584)
  (seen 1006)
  (unrealized (0 0) (0 2) (1 0) (6 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2110)
  (parent 1585)
  (seen 1871)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2111)
  (parent 1586)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2112)
  (parent 1586)
  (seen 1014)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price (6 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2113)
  (parent 1586)
  (seen 1010 1013 1015)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2114)
  (parent 1587)
  (seen 1898)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2115)
  (parent 1588)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2116)
  (parent 1588)
  (seen 1037)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price (6 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2117)
  (parent 1588)
  (seen 1032 1036 1038)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 2118)
  (parent 1590)
  (seen 2106)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 2119)
  (parent 1591)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 4 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 2120)
  (parent 1591)
  (seen 1927)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2121)
  (parent 1591)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2122)
  (parent 1592)
  (seen 1933 2665)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2123)
  (parent 1593)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2124)
  (parent 1594)
  (seen 1955 2669)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 6 bank 2) price (5 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2125)
  (parent 1595)
  (seen 1649)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 2 bank 2) price (5 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2126)
  (parent 1595)
  (seen 1656)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 2127)
  (parent 1597)
  (seen 2106)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 2128)
  (parent 1598)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 4 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 2129)
  (parent 1598)
  (seen 2029)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2130)
  (parent 1598)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2131)
  (parent 1599)
  (seen 2035 2679)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 4 bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))))
  (label 2132)
  (parent 1600)
  (seen 2052 2597)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2133)
  (parent 1600)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2134)
  (parent 1601)
  (seen 2063 2684)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2135)
  (parent 1603)
  (seen 2068)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2136)
  (parent 1604)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2137)
  (parent 1604)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (hash-3 hash-1) (acctnum-0 nb-1))
    nb-1 (5 0) (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2138)
  (parent 1605)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 8 3 bank 2) nb-1 (5 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2139)
  (parent 1605)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (5 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2140)
  (parent 1605)
  (unrealized (0 0) (0 2) (1 0) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2141)
  (parent 1606)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 8 6 bank 2) price (7 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2142)
  (parent 1607)
  (seen 2694 2695)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      text) (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 2 bank 2) price (6 0)
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2143)
  (parent 1609)
  (seen 2243)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 5 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2144)
  (parent 1610)
  (seen 1639 2091)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2145)
  (parent 1611)
  (seen 2137)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 7 bank 2) price-0 (8 0)
    (enc nb-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2146)
  (parent 1612)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2147)
  (parent 1614)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2148)
  (parent 1614)
  (seen 1156)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c nb-0 nb-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-2))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2149)
  (parent 1614)
  (seen 1157 1158)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum acctnum (pubk c))))))
  (label 2150)
  (parent 1626)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum acctnum (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2151)
  (parent 1626)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum acctnum (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 m acctnum (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum acctnum (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2152)
  (parent 1627)
  (seen 2151)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum acctnum (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 m acctnum (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum acctnum (pubk c))))))
  (label 2153)
  (parent 1632)
  (seen 2151)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2154)
  (parent 1638)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2155)
  (parent 1638)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2156)
  (parent 1639)
  (seen 2155)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2157)
  (parent 1642)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2158)
  (parent 1643)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2159)
  (parent 1643)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2160)
  (parent 1644)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2161)
  (parent 1647)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2162)
  (parent 1648)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2163)
  (parent 1648)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2164)
  (parent 1649)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2165)
  (parent 1649)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price acctnum) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum)
  (operation nonce-test
    (contracted (b-0 b) (acctnum-0 acctnum) (hash-3 hash-2)
      (nb-0 goods-0)) nc (1 0)
    (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
    (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
    (enc "hash" c nc goods-0 acctnum acctnum (pubk hash-2))
    (enc nc acctnum (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc acctnum m acctnum (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum acctnum (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2166)
  (parent 1650)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0 (pubk hash-0))
    (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0 (pubk hash-1))
    (enc "hash" c nc goods-0 acctnum-0 acctnum-0 (pubk hash-2))
    (enc nc acctnum-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc acctnum-0 m acctnum-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-1)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc goods-0 acctnum-0 acctnum-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2167)
  (parent 1650)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2168)
  (parent 1651)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (displaced 8 3 customer 3) nb-1 (7 0)
    (enc nc nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-2)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc goods-0 acctnum-0 acctnum-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2169)
  (parent 1652)
  (seen 2723)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2170)
  (parent 1655)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2171)
  (parent 1656)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2172)
  (parent 1656)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2173)
  (parent 1657)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2174)
  (parent 1660)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2175)
  (parent 1660)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 7 0 merchant 2) acctnum-0 (6 0)
    (enc price acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2176)
  (parent 1661)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
    (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc acctnum m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2177)
  (parent 1662)
  (seen 2174 2175)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2178)
  (parent 1662)
  (seen 2729)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c))))))
  (label 2179)
  (parent 1663)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2180)
  (parent 1663)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (5 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) price (5 0)
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2181)
  (parent 1664)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2182)
  (parent 1665)
  (seen 2180)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2183)
  (parent 1665)
  (seen 2181 2729)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2184)
  (parent 1666)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2185)
  (parent 1666)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (6 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2186)
  (parent 1667)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2187)
  (parent 1668)
  (seen 2185)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2188)
  (parent 1668)
  (seen 2735)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2189)
  (parent 1669)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2190)
  (parent 1669)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2191)
  (parent 1669)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (5 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-1 (pubk c)) (enc nc nb-1 m nb-1 (pubk c))
    (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 2192)
  (parent 1670)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2193)
  (parent 1672)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2194)
  (parent 1672)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2195)
  (parent 1672)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2196)
  (parent 1674)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2197)
  (parent 1675)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 2198)
  (parent 1675)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2199)
  (parent 1675)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 2200)
  (parent 1676)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2201)
  (parent 1676)
  (seen 2753)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2202)
  (parent 1676)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2203)
  (parent 1681)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
    (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc acctnum m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2204)
  (parent 1682)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2205)
  (parent 1682)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
    (enc nc acctnum-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc acctnum-0 m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2206)
  (parent 1682)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c))))))
  (label 2207)
  (parent 1683)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2208)
  (parent 1683)
  (seen 2763)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2209)
  (parent 1683)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-2))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2210)
  (parent 1688)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2211)
  (parent 1689)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2212)
  (parent 1689)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2213)
  (parent 1689)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc acctnum acctnum price (pubk hash-1))
    (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
    (enc nc acctnum (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc acctnum m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2214)
  (parent 1690)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-0 (pubk c))))))
  (label 2215)
  (parent 1690)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-1))
    (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
    (enc nc acctnum-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc acctnum-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2216)
  (parent 1690)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price acctnum acctnum price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-2))
            (privk b)) (enc price acctnum (pubk c))))))
  (label 2217)
  (parent 1691)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2218)
  (parent 1691)
  (seen 2778)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2219)
  (parent 1691)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))))
  (label 2220)
  (parent 1695)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 3 bank 2) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2221)
  (parent 1696)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 4 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2222)
  (parent 1696)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2223)
  (parent 1697)
  (seen 1351)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2224)
  (parent 1697)
  (seen 2788 2790)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c))))))
  (label 2225)
  (parent 1699)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2226)
  (parent 1699)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2227)
  (parent 1700)
  (seen 2226)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2228)
  (parent 1702)
  (seen 2226)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 2229)
  (parent 1704)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2230)
  (parent 1704)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2231)
  (parent 1705)
  (seen 2230)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2232)
  (parent 1706)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (4 0)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 3 customer 3) nb-0 (7 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2233)
  (parent 1707)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c))))))
  (label 2234)
  (parent 1708)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2235)
  (parent 1708)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2236)
  (parent 1709)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2237)
  (parent 1710)
  (seen 2230)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (4 0)) ((8 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 3 customer 3) nb-0 (8 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2238)
  (parent 1711)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2239)
  (parent 1712)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2240)
  (parent 1712)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2241)
  (parent 1713)
  (seen 2240)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2242)
  (parent 1714)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2243)
  (parent 1714)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c))))))
  (label 2244)
  (parent 1715)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 goods-0 (pubk c)) (enc acctnum-0 nb-0 (pubk c))
    (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 2245)
  (parent 1715)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2246)
  (parent 1716)
  (seen 2230)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 3 customer 3) nb-0 (8 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2247)
  (parent 1717)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 7 merchant 2) nb-0 (8 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2248)
  (parent 1717)
  (unrealized (0 0) (0 2) (1 0) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 nm-0
      price-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0))
    ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((8 1) (4 0)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (8 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 2249)
  (parent 1717)
  (unrealized (0 0) (0 2) (1 0) (4 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2250)
  (parent 1718)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2251)
  (parent 1719)
  (seen 2240)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (4 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2252)
  (parent 1720)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 2253)
  (parent 1721)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2254)
  (parent 1723)
  (seen 2155)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2255)
  (parent 1724)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nm-0
      price-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm nm-0)
  (operation nonce-test (displaced 8 2 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc nc nm-0 m price-1 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm-0 m price-1 (pubk c)))))
  (label 2256)
  (parent 1725)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nm-0
      price-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 0) (7 0)) ((2 2) (4 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((5 1) (6 0))
    ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 5 merchant 2) nc (6 0)
    (enc nc price m price (pubk c)) (enc nc nm-0 m price-1 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 2257)
  (parent 1725)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 2258)
  (parent 1726)
  (seen 1773)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (5 0)) ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2259)
  (parent 1726)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2260)
  (parent 1727)
  (seen 2240)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 acctnum-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (6 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 3 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2261)
  (parent 1728)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (4 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2262)
  (parent 1729)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc acctnum)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum price price price (pubk hash-0))
    (enc "hash" c acctnum goods-0 price price (pubk hash-1))
    (enc acctnum price (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum price m price (pubk c))
    (enc acctnum nm m price-0 (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum price acctnum price (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) (enc acctnum price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) (enc acctnum price (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 price price (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c))))))
  (label 2263)
  (parent 1730)
  (seen 1469)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (4 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 2264)
  (parent 1731)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc acctnum-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (4 0)
    (enc "hash" c acctnum-0 price price price (pubk hash-0))
    (enc "hash" c acctnum-0 goods-0 price price (pubk hash-1))
    (enc acctnum-0 price (pubk c))
    (enc acctnum-0 price m price (pubk c))
    (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c))))))
  (label 2265)
  (parent 1732)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2266)
  (parent 1733)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 3 customer 3) nc (7 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2267)
  (parent 1734)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((6 1) (7 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 6 merchant 2) nc (7 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2268)
  (parent 1734)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nm-0
      price-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 0) (8 0)) ((3 2) (2 0))
    ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0))
    ((6 1) (4 0)) ((7 1) (1 0)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm nm-0)
  (operation nonce-test (added-strand merchant 2) nc (7 0)
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm-0 m price-1 (pubk c)))))
  (label 2269)
  (parent 1734)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc acctnum-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 8 3 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c acctnum-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b-0)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c)))))
  (label 2270)
  (parent 1736)
  (seen 1545)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2271)
  (parent 1738)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2272)
  (parent 1739)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2273)
  (parent 1739)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2274)
  (parent 1739)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2275)
  (parent 1741)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2276)
  (parent 1741)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2277)
  (parent 1741)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-3))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-2))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2278)
  (parent 1742)
  (seen 2778)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 2279)
  (parent 1743)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2280)
  (parent 1743)
  (seen 2847)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2281)
  (parent 1743)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 2 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2282)
  (parent 1745)
  (seen 2720)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (displaced 8 2 customer 3) nb-1 (7 0)
    (enc nc nb-1 m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 price (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m price (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2283)
  (parent 1745)
  (seen 2779)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 2284)
  (parent 1748)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2285)
  (parent 1748)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 5 bank 2) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2286)
  (parent 1749)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (displaced 7 3 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2287)
  (parent 1749)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0)
  (operation nonce-test (displaced 7 3 customer 3) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc goods-0 acctnum-0 acctnum-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2288)
  (parent 1749)
  (seen 2789 2790)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2289)
  (parent 1750)
  (seen 2285)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2290)
  (parent 1750)
  (seen 2861 2863)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2291)
  (parent 1753)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2292)
  (parent 1754)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 2293)
  (parent 1754)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2294)
  (parent 1754)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 2295)
  (parent 1755)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2296)
  (parent 1755)
  (seen 2868)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2297)
  (parent 1755)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2298)
  (parent 1760)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 2299)
  (parent 1761)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2300)
  (parent 1761)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2301)
  (parent 1765)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 3 customer 3) price (6 0)
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2302)
  (parent 1766)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2303)
  (parent 1767)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2304)
  (parent 1770)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2305)
  (parent 1771)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2306)
  (parent 1773)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-3))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2307)
  (parent 1774)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-3))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2308)
  (parent 1774)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2309)
  (parent 1775)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2310)
  (parent 1778)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2311)
  (parent 1778)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2312)
  (parent 1779)
  (seen 2311)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 2313)
  (parent 1782)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2314)
  (parent 1782)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 5 bank 2) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2315)
  (parent 1783)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2316)
  (parent 1784)
  (seen 2314)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2317)
  (parent 1784)
  (seen 2167 2887)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2318)
  (parent 1790)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nm price goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 8 6 bank 2) nb-1 (7 0)
    (enc nc nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2319)
  (parent 1791)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2320)
  (parent 1791)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nm price text) (m c hash b hash-0 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods) (hash-1 hash-0)) nc
    (0 0) (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2321)
  (parent 1793)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2322)
  (parent 1794)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2323)
  (parent 1794)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2324)
  (parent 1795)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2325)
  (parent 1795)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2326)
  (parent 1796)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2327)
  (parent 1797)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2328)
  (parent 1800)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 2329)
  (parent 1801)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2330)
  (parent 1801)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2331)
  (parent 1802)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2332)
  (parent 1802)
  (seen 1777)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2333)
  (parent 1803)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 2334)
  (parent 1804)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2335)
  (parent 1804)
  (seen 1781)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 2336)
  (parent 1805)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2337)
  (parent 1810)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2338)
  (parent 1810)
  (seen 2304)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 2339)
  (parent 1811)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2340)
  (parent 1811)
  (seen 2306)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2341)
  (parent 1812)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2342)
  (parent 1814)
  (seen 2311)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 2343)
  (parent 1817)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2344)
  (parent 1817)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (7 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2345)
  (parent 1819)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2346)
  (parent 1819)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2347)
  (parent 1820)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2348)
  (parent 1820)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2349)
  (parent 1822)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2350)
  (parent 1823)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2351)
  (parent 1824)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2352)
  (parent 1824)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2353)
  (parent 1825)
  (seen 2352)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2354)
  (parent 1826)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (7 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2355)
  (parent 1826)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2356)
  (parent 1827)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (7 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2357)
  (parent 1827)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (7 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2358)
  (parent 1828)
  (seen 2357)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2359)
  (parent 1829)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 3 bank 2) price (6 0)
    (enc "hash" c nb-1 price price price (pubk hash-2))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2360)
  (parent 1829)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2361)
  (parent 1830)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2362)
  (parent 1830)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2363)
  (parent 1831)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2364)
  (parent 1832)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2365)
  (parent 1832)
  (seen 2929)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 5 customer 3) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2366)
  (parent 1832)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2367)
  (parent 1833)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2368)
  (parent 1833)
  (seen 2363)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2369)
  (parent 1834)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2370)
  (parent 1834)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2371)
  (parent 1835)
  (seen 2370)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2372)
  (parent 1836)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2373)
  (parent 1837)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2374)
  (parent 1837)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2375)
  (parent 1838)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2376)
  (parent 1838)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2377)
  (parent 1842)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (7 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2378)
  (parent 1842)
  (seen 2349)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2379)
  (parent 1843)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2380)
  (parent 1844)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2381)
  (parent 1845)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2382)
  (parent 1846)
  (seen 2357)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2383)
  (parent 1847)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2384)
  (parent 1848)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2385)
  (parent 1848)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 3 bank 2) price (7 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2386)
  (parent 1848)
  (seen 2952)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2387)
  (parent 1848)
  (seen 2380)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2388)
  (parent 1849)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (7 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2389)
  (parent 1849)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2390)
  (parent 1850)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2391)
  (parent 1851)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 10 6 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2392)
  (parent 1852)
  (seen 2954)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2393)
  (parent 1853)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2394)
  (parent 1853)
  (seen 2392)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2395)
  (parent 1856)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2396)
  (parent 1858)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2397)
  (parent 1858)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2398)
  (parent 1859)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2399)
  (parent 1859)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2400)
  (parent 1861)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2401)
  (parent 1862)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2402)
  (parent 1863)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2403)
  (parent 1863)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2404)
  (parent 1864)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2405)
  (parent 1865)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2406)
  (parent 1866)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 2407)
  (parent 1866)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2408)
  (parent 1867)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2409)
  (parent 1868)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2410)
  (parent 1868)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2411)
  (parent 1869)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2412)
  (parent 1870)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (5 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2413)
  (parent 1871)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2414)
  (parent 1873)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2415)
  (parent 1873)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2416)
  (parent 1875)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2417)
  (parent 1876)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2418)
  (parent 1876)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2419)
  (parent 1877)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (4 3))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 2420)
  (parent 1880)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (4 3))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 2421)
  (parent 1883)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 2422)
  (parent 1885)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 2423)
  (parent 1885)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 2424)
  (parent 1887)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 2425)
  (parent 1888)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 2426)
  (parent 1890)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 2427)
  (parent 1891)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2428)
  (parent 1892)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 2429)
  (parent 1892)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 7 3 customer 3) nb-0 (6 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2430)
  (parent 1893)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 2431)
  (parent 1894)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (5 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2432)
  (parent 1895)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2433)
  (parent 1896)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2434)
  (parent 1896)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2435)
  (parent 1897)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2436)
  (parent 1898)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2437)
  (parent 1899)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 2438)
  (parent 1899)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2439)
  (parent 1900)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 2440)
  (parent 1902)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 2441)
  (parent 1902)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 2442)
  (parent 1904)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 2443)
  (parent 1905)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 2444)
  (parent 1906)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 2445)
  (parent 1906)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 2446)
  (parent 1907)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2447)
  (parent 1908)
  (seen 2798)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2448)
  (parent 1910)
  (seen 2923)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2449)
  (parent 1911)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 3 bank 2) price (7 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2450)
  (parent 1911)
  (seen 2931 3010)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2451)
  (parent 1911)
  (seen 2447 2935)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 10 6 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2452)
  (parent 1912)
  (seen 2939 3011)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2453)
  (parent 1914)
  (seen 2452)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2454)
  (parent 1916)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2455)
  (parent 1918)
  (seen 2397)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2456)
  (parent 1919)
  (seen 2399)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2457)
  (parent 1920)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2458)
  (parent 1921)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2459)
  (parent 1922)
  (seen 2403)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2460)
  (parent 1923)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 3 bank 2) price (6 0)
    (enc "hash" c nb-1 price price price (pubk hash-3))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-2))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2461)
  (parent 1923)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2462)
  (parent 1924)
  (seen 2407)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2463)
  (parent 1924)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2464)
  (parent 1924)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2465)
  (parent 1926)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (6 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2466)
  (parent 1927)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 5 customer 3) nb-1 (8 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2467)
  (parent 1928)
  (seen 3025)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2468)
  (parent 1930)
  (seen 2415)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2469)
  (parent 1931)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2470)
  (parent 1932)
  (seen 2418)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (6 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2471)
  (parent 1933)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 2472)
  (parent 1935)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 2473)
  (parent 1937)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 2474)
  (parent 1939)
  (seen 2423)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2475)
  (parent 1940)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2476)
  (parent 1941)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2477)
  (parent 1942)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2478)
  (parent 1943)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2479)
  (parent 1944)
  (seen 2429)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2480)
  (parent 1944)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2481)
  (parent 1945)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 3 bank 2) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2482)
  (parent 1945)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2483)
  (parent 1946)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2484)
  (parent 1946)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2485)
  (parent 1947)
  (seen 3040)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2486)
  (parent 1948)
  (seen 2438)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2487)
  (parent 1948)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 2488)
  (parent 1951)
  (seen 2441)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2489)
  (parent 1952)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2490)
  (parent 1953)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 2491)
  (parent 1954)
  (seen 2445)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2492)
  (parent 1955)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (5 0))
    ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2493)
  (parent 1957)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2494)
  (parent 1958)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 2495)
  (parent 1959)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2496)
  (parent 1959)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 2497)
  (parent 1960)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2498)
  (parent 1960)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2499)
  (parent 1961)
  (seen 2498)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2500)
  (parent 1968)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2501)
  (parent 1969)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2502)
  (parent 1971)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2503)
  (parent 1972)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 2504)
  (parent 1973)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2505)
  (parent 1973)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc acctnum nm m price (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2506)
  (parent 1974)
  (seen 2505)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 2507)
  (parent 1975)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 2508)
  (parent 1976)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2509)
  (parent 1976)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-1 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2510)
  (parent 1977)
  (seen 2509)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2511)
  (parent 1978)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2512)
  (parent 1979)
  (seen 2352)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2513)
  (parent 1980)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2514)
  (parent 1981)
  (seen 2357)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2515)
  (parent 1982)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2516)
  (parent 1983)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2517)
  (parent 1983)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 3 bank 2) price (7 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2518)
  (parent 1983)
  (seen 3063)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2519)
  (parent 1983)
  (seen 2247)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2520)
  (parent 1984)
  (seen 2370)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 10 6 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2521)
  (parent 1985)
  (seen 3065)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2522)
  (parent 1986)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2523)
  (parent 1987)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2524)
  (parent 1987)
  (seen 2521)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2525)
  (parent 1988)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2526)
  (parent 1989)
  (seen 2378)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2527)
  (parent 1990)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 5 bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2528)
  (parent 1990)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2529)
  (parent 1991)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0))
    ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2530)
  (parent 1991)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2531)
  (parent 1992)
  (seen 3079)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2532)
  (parent 1993)
  (seen 2389)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2533)
  (parent 1994)
  (seen 3079)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2534)
  (parent 1995)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2535)
  (parent 1995)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2536)
  (parent 1995)
  (seen 3083)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (1 0)) ((3 1) (0 0))
    ((4 1) (1 0)) ((5 1) (0 0)) ((6 0) (3 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum acctnum (pubk hash-0))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 2537)
  (parent 1997)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (6 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 2538)
  (parent 1998)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (8 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2539)
  (parent 1999)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (8 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 2540)
  (parent 1999)
  (seen 985)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2541)
  (parent 2000)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2542)
  (parent 2001)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (8 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2543)
  (parent 2001)
  (unrealized (0 0) (0 2) (1 0) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 9 5 bank 2) price (8 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2544)
  (parent 2001)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2545)
  (parent 2002)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (8 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2546)
  (parent 2002)
  (unrealized (0 0) (0 2) (1 0) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2547)
  (parent 2003)
  (seen 3098)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2548)
  (parent 2004)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2549)
  (parent 2005)
  (seen 3098)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2550)
  (parent 2006)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2551)
  (parent 2006)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2552)
  (parent 2006)
  (seen 3102)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 0) (4 0))
    ((5 2) (3 0)) ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 2553)
  (parent 2007)
  (seen 449)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (1 0)) ((3 1) (0 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (3 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum acctnum (pubk hash-0))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 2554)
  (parent 2008)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 hash-1 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (6 1)) ((1 1) (6 3)) ((2 1) (1 0)) ((3 1) (2 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (0 0)) ((6 0) (3 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk hash-1)
    (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum acctnum (pubk hash-0))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 2555)
  (parent 2009)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (7 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price))))
  (label 2556)
  (parent 2011)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (6 3)) ((6 0) (4 0))
    ((6 0) (7 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 2557)
  (parent 2012)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (8 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 2558)
  (parent 2013)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 2559)
  (parent 2014)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 3 bank 2) price (8 0)
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 2560)
  (parent 2014)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 5 bank 2) price (8 0)
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 2561)
  (parent 2014)
  (seen 3118)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (8 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 2562)
  (parent 2014)
  (unrealized (0 0) (0 2) (1 0) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-2))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 2563)
  (parent 2015)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-2))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 2564)
  (parent 2015)
  (unrealized (0 0) (0 2) (1 0) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (9 0)
    (enc price-0 price m price (pubk c)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 2565)
  (parent 2016)
  (seen 3121 3123)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2566)
  (parent 2018)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2567)
  (parent 2020)
  (seen 2397)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2568)
  (parent 2021)
  (seen 2399)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2569)
  (parent 2022)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2570)
  (parent 2023)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2571)
  (parent 2024)
  (seen 2403)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2572)
  (parent 2025)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 3 bank 2) price (6 0)
    (enc "hash" c nb-1 price price price (pubk hash-3))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-2))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2573)
  (parent 2025)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2574)
  (parent 2026)
  (seen 2407)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2575)
  (parent 2026)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2576)
  (parent 2026)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2577)
  (parent 2028)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (6 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2578)
  (parent 2029)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      nb-2 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 5 customer 3) nb-1 (8 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2579)
  (parent 2030)
  (seen 3137)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2580)
  (parent 2032)
  (seen 2415)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2581)
  (parent 2033)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2582)
  (parent 2034)
  (seen 2418)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (6 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2583)
  (parent 2035)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 3))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 2584)
  (parent 2037)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 3))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 2585)
  (parent 2039)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 2586)
  (parent 2041)
  (seen 2423)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2587)
  (parent 2042)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2588)
  (parent 2043)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2589)
  (parent 2044)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 2590)
  (parent 2044)
  (seen 1021)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2591)
  (parent 2045)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2592)
  (parent 2046)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2593)
  (parent 2046)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 2594)
  (parent 2048)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2595)
  (parent 2049)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2596)
  (parent 2049)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2597)
  (parent 2049)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2598)
  (parent 2050)
  (seen 2438)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2599)
  (parent 2050)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (1 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-2))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2600)
  (parent 2050)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2601)
  (parent 2051)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2602)
  (parent 2052)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2603)
  (parent 2053)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 2604)
  (parent 2053)
  (seen 2426)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2605)
  (parent 2054)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2606)
  (parent 2055)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2607)
  (parent 2055)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2608)
  (parent 2056)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2609)
  (parent 2057)
  (seen 3159)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 2610)
  (parent 2059)
  (seen 2441)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2611)
  (parent 2060)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2612)
  (parent 2061)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 2613)
  (parent 2062)
  (seen 2445)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2614)
  (parent 2063)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 2615)
  (parent 2064)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2616)
  (parent 2065)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (2 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 2617)
  (parent 2066)
  (seen 2498)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 2618)
  (parent 2067)
  (seen 1770)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-3))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2619)
  (parent 2068)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2620)
  (parent 2069)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2621)
  (parent 2070)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-3))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2622)
  (parent 2070)
  (seen 2619)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 2623)
  (parent 2072)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 2624)
  (parent 2073)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nm price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nm m price (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 2625)
  (parent 2074)
  (seen 2505)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 2626)
  (parent 2075)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
    (enc acctnum nb-0 (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-1 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 2627)
  (parent 2076)
  (seen 2509)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2628)
  (parent 2079)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2629)
  (parent 2082)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2630)
  (parent 2084)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2631)
  (parent 2084)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2632)
  (parent 2085)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2633)
  (parent 2085)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2634)
  (parent 2087)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2635)
  (parent 2089)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0)) ((5 2) (3 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-3))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2636)
  (parent 2091)
  (seen 2308)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (2 0))
    ((5 2) (3 0)) ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-3))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2637)
  (parent 2093)
  (seen 2636)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc acctnum-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 2638)
  (parent 2095)
  (seen 3056)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2639)
  (parent 2097)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2640)
  (parent 2099)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2641)
  (parent 2101)
  (seen 2631)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2642)
  (parent 2102)
  (seen 2633)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2643)
  (parent 2103)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2644)
  (parent 2104)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (7 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (6 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2645)
  (parent 2106)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 2646)
  (parent 2107)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (7 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2647)
  (parent 2107)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 2648)
  (parent 2108)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2649)
  (parent 2110)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2650)
  (parent 2110)
  (seen 1384)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 4 bank 2) price (6 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-1))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2651)
  (parent 2110)
  (seen 2411)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 acctnum-1 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 8 7 bank 2) price (6 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2652)
  (parent 2110)
  (seen 1383 1388 1389 3190)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2653)
  (parent 2111)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 9 7 bank 2) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2654)
  (parent 2112)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2655)
  (parent 2114)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2656)
  (parent 2114)
  (seen 1412)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2657)
  (parent 2114)
  (seen 1411 1413 1414)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 4 bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2658)
  (parent 2114)
  (seen 2435)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2659)
  (parent 2115)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 7 bank 2) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2660)
  (parent 2116)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 2661)
  (parent 2118)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 2662)
  (parent 2119)
  (seen 2647)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 2663)
  (parent 2120)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2664)
  (parent 2120)
  (seen 1433)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2665)
  (parent 2121)
  (seen 2467)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2666)
  (parent 2122)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2667)
  (parent 2122)
  (seen 1441)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-3))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2668)
  (parent 2122)
  (seen 1437 1440 1442)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2669)
  (parent 2123)
  (seen 2485)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2670)
  (parent 2124)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2671)
  (parent 2124)
  (seen 1459)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-3))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2672)
  (parent 2124)
  (seen 1454 1458 1460)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 7 3 bank 2) nb-0 (5 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2673)
  (parent 2125)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (5 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 2674)
  (parent 2125)
  (unrealized (0 0) (0 2) (1 0) (5 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 2675)
  (parent 2127)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 2676)
  (parent 2128)
  (seen 2647)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))))
  (label 2677)
  (parent 2129)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2678)
  (parent 2129)
  (seen 1507)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 acctnum-1
      nb-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2679)
  (parent 2130)
  (seen 2579)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2680)
  (parent 2131)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2681)
  (parent 2131)
  (seen 1515)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-3))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2682)
  (parent 2131)
  (seen 1511 1514 1516)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2683)
  (parent 2132)
  (unrealized (0 0) (0 2) (1 0) (7 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2684)
  (parent 2133)
  (seen 2609 3230)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "5 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2685)
  (parent 2134)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2686)
  (parent 2134)
  (seen 1538)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-3))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2687)
  (parent 2134)
  (seen 1534 1537 1539)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2688)
  (parent 2135)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c nc nb-0 price-0 price-0 (pubk hash-2))
    (enc nc price-0 m price-0 (pubk c))
    (enc c nc price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2689)
  (parent 2135)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 2 bank 2) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2690)
  (parent 2135)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2691)
  (parent 2137)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2692)
  (parent 2138)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2693)
  (parent 2138)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (4 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2694)
  (parent 2139)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (4 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2695)
  (parent 2139)
  (unrealized (0 0) (0 2) (1 0) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 8 3 customer 3) nb-1 (7 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2696)
  (parent 2140)
  (seen 3246)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2697)
  (parent 2141)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2698)
  (parent 2141)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price-0 (6 0)
    (enc "hash" c nc nb-0 price-0 price-0 (pubk hash-2))
    (enc nc price-0 m price-0 (pubk c))
    (enc c nc price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2699)
  (parent 2144)
  (seen 2165)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 acctnum-1
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 2 bank 2) price (6 0)
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 2700)
  (parent 2144)
  (seen 2172)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2701)
  (parent 2145)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (hash-3 hash-1) (acctnum-0 nb-1))
    nb-1 (6 0) (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2702)
  (parent 2146)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 3 bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2703)
  (parent 2146)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 2704)
  (parent 2146)
  (unrealized (0 0) (0 2) (1 0) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2705)
  (parent 2147)
  (seen 2698)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 7 bank 2) price-0 (8 0)
    (enc nb-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2706)
  (parent 2148)
  (seen 3253 3254)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum acctnum (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2707)
  (parent 2151)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum acctnum (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2708)
  (parent 2152)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (3 3)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (4 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum acctnum (pubk c))))))
  (label 2709)
  (parent 2153)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2710)
  (parent 2155)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2711)
  (parent 2156)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2712)
  (parent 2158)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2713)
  (parent 2158)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2714)
  (parent 2159)
  (seen 2713)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2715)
  (parent 2162)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2716)
  (parent 2162)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2717)
  (parent 2163)
  (seen 2716)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2718)
  (parent 2164)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2719)
  (parent 2164)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2720)
  (parent 2165)
  (seen 2719)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum acctnum (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2721)
  (parent 2166)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum acctnum (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2722)
  (parent 2166)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
    (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
    (enc "hash" c nc goods-0 acctnum acctnum (pubk hash-2))
    (enc nc acctnum (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc acctnum m acctnum (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum acctnum (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2723)
  (parent 2167)
  (seen 2722)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2724)
  (parent 2171)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2725)
  (parent 2171)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2726)
  (parent 2172)
  (seen 2725)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2727)
  (parent 2175)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 2728)
  (parent 2176)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2729)
  (parent 2176)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2730)
  (parent 2180)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) price (5 0)
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2731)
  (parent 2181)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2732)
  (parent 2182)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2733)
  (parent 2185)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2734)
  (parent 2186)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2735)
  (parent 2186)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2736)
  (parent 2187)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2737)
  (parent 2189)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2738)
  (parent 2189)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (8 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 9 3 bank 2) nb-1 (8 0)
    (enc nc nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2739)
  (parent 2190)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 9 4 customer 3) nb-0 (8 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2740)
  (parent 2190)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2741)
  (parent 2191)
  (seen 2738)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2742)
  (parent 2191)
  (seen 3276 3278)
  (unrealized (0 0) (0 2) (1 0) (9 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2743)
  (parent 2192)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2744)
  (parent 2192)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2745)
  (parent 2192)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2746)
  (parent 2193)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2747)
  (parent 2193)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 8 3 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2748)
  (parent 2194)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2749)
  (parent 2195)
  (seen 2747)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2750)
  (parent 2195)
  (seen 3287)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2751)
  (parent 2197)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2752)
  (parent 2197)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 0 merchant 2) nb-0 (7 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2753)
  (parent 2198)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2754)
  (parent 2199)
  (seen 2751 2752)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (3 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (6 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0))
    ((6 1) (1 0)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 2755)
  (parent 2199)
  (seen 3291)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 2756)
  (parent 2200)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2757)
  (parent 2200)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (6 0)
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2758)
  (parent 2201)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2759)
  (parent 2202)
  (seen 2757)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2760)
  (parent 2202)
  (seen 2758 3291)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2761)
  (parent 2204)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2762)
  (parent 2204)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 8 0 merchant 2) acctnum-0 (7 0)
    (enc price acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2763)
  (parent 2205)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
    (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
    (enc nc acctnum (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc acctnum m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2764)
  (parent 2206)
  (seen 2761 2762)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (6 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0))
    ((6 1) (1 0)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2765)
  (parent 2206)
  (seen 3297)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c))))))
  (label 2766)
  (parent 2207)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2767)
  (parent 2207)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) price (6 0)
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2768)
  (parent 2208)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2769)
  (parent 2209)
  (seen 2767)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2770)
  (parent 2209)
  (seen 2768 3297)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2771)
  (parent 2211)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2772)
  (parent 2211)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 0 merchant 2) nb-0 (7 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2773)
  (parent 2212)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2774)
  (parent 2213)
  (seen 2771 2772)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (3 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (6 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2775)
  (parent 2213)
  (seen 3303)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2776)
  (parent 2214)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2777)
  (parent 2214)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 7 0 merchant 2) acctnum-0 (6 0)
    (enc price acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2778)
  (parent 2215)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc acctnum acctnum price (pubk hash-0))
    (enc "hash" c nc acctnum acctnum price (pubk hash-1))
    (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
    (enc nc acctnum (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc acctnum m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m))
    (enc c nc acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2779)
  (parent 2216)
  (seen 2776 2777)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-0 (pubk c))))))
  (label 2780)
  (parent 2216)
  (seen 3306)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-2))
            (privk b)) (enc price acctnum (pubk c))))))
  (label 2781)
  (parent 2217)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-2))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2782)
  (parent 2217)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nb-0)
    (nm acctnum-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb acctnum-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb acctnum-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 7 2 bank 2) price (6 0)
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum-0 nb-0 (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 acctnum-0 m nb-0 (pubk c)))
      (send (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nb-0 acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nb-0 acctnum-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nb-0 acctnum-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 acctnum-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nb-0 goods-0 (pubk c))))))
  (label 2783)
  (parent 2218)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) price (6 0)
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2784)
  (parent 2218)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price acctnum acctnum price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-2))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2785)
  (parent 2219)
  (seen 2782)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2786)
  (parent 2219)
  (seen 2784 3306 3309)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2787)
  (parent 2221)
  (seen 2346)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2788)
  (parent 2221)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2789)
  (parent 2222)
  (seen 2348)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2790)
  (parent 2222)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2791)
  (parent 2223)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2792)
  (parent 2226)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2793)
  (parent 2227)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2794)
  (parent 2228)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2795)
  (parent 2230)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2796)
  (parent 2231)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2797)
  (parent 2232)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2798)
  (parent 2232)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2799)
  (parent 2233)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (4 0)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 2 bank 2) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2800)
  (parent 2233)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 2801)
  (parent 2234)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2802)
  (parent 2234)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2803)
  (parent 2235)
  (seen 2802)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2804)
  (parent 2236)
  (seen 2802)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2805)
  (parent 2237)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2806)
  (parent 2238)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2807)
  (parent 2240)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2808)
  (parent 2241)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2809)
  (parent 2242)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2810)
  (parent 2242)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2811)
  (parent 2243)
  (seen 2810)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 2812)
  (parent 2244)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2813)
  (parent 2244)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc acctnum nm m price (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2814)
  (parent 2245)
  (seen 2813)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2815)
  (parent 2246)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2816)
  (parent 2247)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 3 customer 3) nb-0 (8 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2817)
  (parent 2248)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 nm-0
      price-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((8 1) (4 0)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 10 3 customer 3) nb-0 (8 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 2818)
  (parent 2249)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 acctnum-0 nm price-0 nm-0 price-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0))
    ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 7 merchant 2) nb-0 (8 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 2819)
  (parent 2249)
  (unrealized (0 0) (0 2) (1 0) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2820)
  (parent 2250)
  (seen 2802)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2821)
  (parent 2251)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2822)
  (parent 2252)
  (seen 2810)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc acctnum nm m price (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 2823)
  (parent 2253)
  (seen 2813)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2824)
  (parent 2254)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2825)
  (parent 2255)
  (seen 2155)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1
      text) (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc nc nm-0 m price-1 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm-0 m price-1 (pubk c)))))
  (label 2826)
  (parent 2256)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nm-0
      price-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((5 1) (6 0))
    ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 2 customer 3) price-0 (6 0)
    (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 2827)
  (parent 2257)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 2828)
  (parent 2258)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 2 customer 3) price (7 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2829)
  (parent 2259)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2830)
  (parent 2260)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 2831)
  (parent 2261)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2832)
  (parent 2262)
  (seen 2810)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) (enc acctnum price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) (enc acctnum price (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 2833)
  (parent 2263)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc acctnum nm m price (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c))))))
  (label 2834)
  (parent 2264)
  (seen 2813)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc acctnum)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum price price price (pubk hash-0))
    (enc "hash" c acctnum goods-0 price price (pubk hash-1))
    (enc acctnum price (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum price m price (pubk c))
    (enc acctnum nm m price-0 (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum price acctnum price (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) (enc acctnum price (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) (enc acctnum price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) (enc acctnum price (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 price price (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c))))))
  (label 2835)
  (parent 2265)
  (seen 1968)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2836)
  (parent 2267)
  (seen 2725)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((6 1) (7 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 8 3 customer 3) nc (7 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2837)
  (parent 2268)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nm-0
      price-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (7 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0))
    ((6 1) (4 0)) ((7 1) (1 0)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm nm-0)
  (operation nonce-test (displaced 9 3 customer 3) nc (7 0)
    (enc nc price m price (pubk c)) (enc nc nm-0 m price-1 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm-0 m price-1 (pubk c)))))
  (label 2838)
  (parent 2269)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nm-0
      price-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 0) (6 0)) ((3 0) (8 0)) ((3 2) (2 0))
    ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0))
    ((6 1) (4 0)) ((6 1) (7 0)) ((7 1) (1 0)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 6 merchant 2) nc (7 0)
    (enc nc price m price (pubk c)) (enc nc nm-0 m price-1 (pubk c))
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 2839)
  (parent 2269)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2840)
  (parent 2272)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2841)
  (parent 2272)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 0 merchant 2) nb-0 (7 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2842)
  (parent 2273)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2843)
  (parent 2274)
  (seen 2840 2841)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (2 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (7 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2844)
  (parent 2274)
  (seen 3354)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2845)
  (parent 2275)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2846)
  (parent 2275)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 7 0 merchant 2) nb-0 (6 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2847)
  (parent 2276)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2848)
  (parent 2277)
  (seen 2845 2846)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2849)
  (parent 2277)
  (seen 3357)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 5 bank 2) acctnum-0 (7 0)
    (enc price acctnum-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2850)
  (parent 2278)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 2851)
  (parent 2279)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2852)
  (parent 2279)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-1)
    (nm nb-0) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 5 bank 2) price (6 0)
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-1 nb-1 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-1 goods-0 (pubk c))))))
  (label 2853)
  (parent 2280)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 7 2 customer 3) price (6 0)
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-1 nb-0 acctnum-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-3)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c))))))
  (label 2854)
  (parent 2280)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (7 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (6 0)
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2855)
  (parent 2280)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2856)
  (parent 2281)
  (seen 2852)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2857)
  (parent 2281)
  (seen 2855 3357 3362)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-1) (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (2 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (7 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0))
    ((6 1) (1 0)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-3))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-2))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 acctnum-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-1 (pubk c))))))
  (label 2858)
  (parent 2283)
  (seen 3306 3359)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2859)
  (parent 2285)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2860)
  (parent 2286)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2861)
  (parent 2286)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2862)
  (parent 2287)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2863)
  (parent 2287)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2864)
  (parent 2289)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nc)
    (nm acctnum-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0) (nb nb-0)
    (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm acctnum-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum-0 nm)
  (operation nonce-test (displaced 8 3 customer 3) price-0 (7 0)
    (enc nc price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum-0 acctnum-0 (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc acctnum-0 m acctnum-0 (pubk c)))
      (send (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) acctnum-0)))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc acctnum-0 acctnum-0 acctnum-0
              (pubk hash-0)) (privk b-0)) (enc nc acctnum-0 (pubk c)))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc acctnum-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nc goods-0 acctnum-0 acctnum-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2865)
  (parent 2290)
  (seen 3316)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2866)
  (parent 2292)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2867)
  (parent 2292)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 8 0 merchant 2) nb-0 (7 0)
    (enc price nb-0 m price (pubk c)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2868)
  (parent 2293)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2869)
  (parent 2294)
  (seen 2866 2867)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0 nb-2
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-2)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (2 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (7 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0))
    ((6 1) (1 0)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-2 nb-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-2 (pubk c))))))
  (label 2870)
  (parent 2294)
  (seen 3379)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 2871)
  (parent 2295)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2872)
  (parent 2295)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (7 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (6 0)
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2873)
  (parent 2296)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2874)
  (parent 2297)
  (seen 2872)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price nb-1 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c))))))
  (label 2875)
  (parent 2297)
  (seen 2873 3379)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c))))))
  (label 2876)
  (parent 2299)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2877)
  (parent 2299)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2878)
  (parent 2300)
  (seen 2877)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 2879)
  (parent 2302)
  (seen 2877)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2880)
  (parent 2307)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2881)
  (parent 2307)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-3))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2882)
  (parent 2308)
  (seen 2881)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2883)
  (parent 2311)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2884)
  (parent 2312)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2885)
  (parent 2314)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2886)
  (parent 2315)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2887)
  (parent 2315)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2888)
  (parent 2316)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nm price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2889)
  (parent 2319)
  (seen 3389)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2890)
  (parent 2320)
  (seen 2722)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2891)
  (parent 2322)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2892)
  (parent 2322)
  (seen 1818)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nm price goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 9 7 bank 2) nb-1 (8 0)
    (enc nc nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2893)
  (parent 2323)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2894)
  (parent 2323)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2895)
  (parent 2324)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2896)
  (parent 2324)
  (seen 1777)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2897)
  (parent 2325)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2898)
  (parent 2327)
  (seen 2881)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2899)
  (parent 2328)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2900)
  (parent 2328)
  (seen 2304)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 2901)
  (parent 2329)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2902)
  (parent 2329)
  (seen 2306)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 9 4 customer 3) nb-0 (8 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2903)
  (parent 2330)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2904)
  (parent 2333)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 2905)
  (parent 2336)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (2 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (5 0)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2906)
  (parent 2336)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-3))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc goods-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 2907)
  (parent 2341)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 2908)
  (parent 2342)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 2909)
  (parent 2343)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2910)
  (parent 2343)
  (seen 2885)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nm price goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 9 7 bank 2) price-0 (8 0)
    (enc nc price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2911)
  (parent 2344)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 2912)
  (parent 2344)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 2913)
  (parent 2346)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 2914)
  (parent 2348)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2915)
  (parent 2352)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2916)
  (parent 2353)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2917)
  (parent 2354)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (7 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2918)
  (parent 2354)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (7 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2919)
  (parent 2355)
  (seen 2918)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (7 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2920)
  (parent 2357)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (7 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2921)
  (parent 2358)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2922)
  (parent 2359)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (8 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2923)
  (parent 2359)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2924)
  (parent 2360)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2925)
  (parent 2361)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2926)
  (parent 2361)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2927)
  (parent 2362)
  (seen 2926)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2928)
  (parent 2363)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 9 3 bank 2) price (6 0)
    (enc "hash" c nb-1 price price price (pubk hash-4))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2929)
  (parent 2363)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 2930)
  (parent 2364)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 2931)
  (parent 2364)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2932)
  (parent 2366)
  (unrealized (0 0) (0 2) (1 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2933)
  (parent 2366)
  (seen 2929)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2934)
  (parent 2367)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2935)
  (parent 2367)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2936)
  (parent 2370)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2937)
  (parent 2371)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2938)
  (parent 2372)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2939)
  (parent 2372)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2940)
  (parent 2373)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2941)
  (parent 2373)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2942)
  (parent 2374)
  (seen 2941)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2943)
  (parent 2375)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2944)
  (parent 2375)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2945)
  (parent 2376)
  (seen 2944)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2946)
  (parent 2379)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (7 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2947)
  (parent 2379)
  (seen 2915)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2948)
  (parent 2380)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2949)
  (parent 2381)
  (seen 2918)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2950)
  (parent 2382)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2951)
  (parent 2383)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 3 bank 2) price (7 0)
    (enc "hash" c nb-1 price price price (pubk hash-2))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2952)
  (parent 2383)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2953)
  (parent 2384)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 10 6 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2954)
  (parent 2385)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 2955)
  (parent 2386)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2956)
  (parent 2386)
  (seen 3443)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 2957)
  (parent 2386)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2958)
  (parent 2387)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 2959)
  (parent 2387)
  (seen 2954)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (7 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2960)
  (parent 2389)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2961)
  (parent 2390)
  (seen 2941)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2962)
  (parent 2391)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2963)
  (parent 2391)
  (seen 2936)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 2964)
  (parent 2392)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2965)
  (parent 2393)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2966)
  (parent 2397)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2967)
  (parent 2399)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2968)
  (parent 2401)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2969)
  (parent 2401)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2970)
  (parent 2403)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 2971)
  (parent 2404)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2972)
  (parent 2404)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 2973)
  (parent 2405)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 2974)
  (parent 2407)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 2975)
  (parent 2408)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2976)
  (parent 2409)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 2977)
  (parent 2409)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2978)
  (parent 2410)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2979)
  (parent 2411)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 2980)
  (parent 2412)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2981)
  (parent 2412)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2982)
  (parent 2413)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2983)
  (parent 2415)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 2984)
  (parent 2418)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 2985)
  (parent 2419)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 2986)
  (parent 2419)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 2987)
  (parent 2423)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 2988)
  (parent 2425)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 2989)
  (parent 2425)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 2990)
  (parent 2427)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 2991)
  (parent 2427)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 2992)
  (parent 2429)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 2993)
  (parent 2430)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 2994)
  (parent 2431)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 2995)
  (parent 2431)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 2996)
  (parent 2432)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 2997)
  (parent 2433)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 2998)
  (parent 2433)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 2999)
  (parent 2434)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3000)
  (parent 2435)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3001)
  (parent 2436)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3002)
  (parent 2438)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3003)
  (parent 2439)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3004)
  (parent 2441)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3005)
  (parent 2443)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3006)
  (parent 2443)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3007)
  (parent 2445)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3008)
  (parent 2446)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3009)
  (parent 2446)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 3 bank 2) price (7 0)
    (enc "hash" c nb-1 price price price (pubk hash-2))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3010)
  (parent 2448)
  (seen 3414)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 10 6 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3011)
  (parent 2449)
  (seen 3418)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3012)
  (parent 2450)
  (seen 3481)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3013)
  (parent 2450)
  (seen 3425)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3014)
  (parent 2451)
  (seen 3011)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3015)
  (parent 2455)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3016)
  (parent 2456)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3017)
  (parent 2458)
  (seen 2969)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3018)
  (parent 2459)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3019)
  (parent 2460)
  (seen 2972)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3020)
  (parent 2461)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3021)
  (parent 2462)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3022)
  (parent 2463)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3023)
  (parent 2464)
  (seen 2977)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3024)
  (parent 2464)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3025)
  (parent 2465)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3026)
  (parent 2466)
  (seen 2981)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (6 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3027)
  (parent 2467)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3028)
  (parent 2468)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3029)
  (parent 2470)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3030)
  (parent 2471)
  (seen 2986)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3031)
  (parent 2474)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3032)
  (parent 2476)
  (seen 2989)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3033)
  (parent 2478)
  (seen 2991)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3034)
  (parent 2479)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3035)
  (parent 2480)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3036)
  (parent 2481)
  (seen 2995)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3037)
  (parent 2482)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3038)
  (parent 2483)
  (seen 2998)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3039)
  (parent 2483)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3040)
  (parent 2484)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3041)
  (parent 2485)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3042)
  (parent 2486)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3043)
  (parent 2487)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3044)
  (parent 2488)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3045)
  (parent 2490)
  (seen 3006)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3046)
  (parent 2491)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3047)
  (parent 2492)
  (seen 3009)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods text) (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 3048)
  (parent 2495)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3049)
  (parent 2495)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3050)
  (parent 2496)
  (seen 3049)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3051)
  (parent 2498)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (6 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3052)
  (parent 2499)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3053)
  (parent 2505)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3054)
  (parent 2506)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-3))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-1 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 3055)
  (parent 2507)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc acctnum-0 goods-0 (pubk c)) (enc acctnum-0 nb-0 (pubk c))
    (enc acctnum-0 nb-1 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 3056)
  (parent 2507)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3057)
  (parent 2509)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3058)
  (parent 2510)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3059)
  (parent 2512)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3060)
  (parent 2513)
  (seen 2918)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3061)
  (parent 2514)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3062)
  (parent 2515)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 3 bank 2) price (7 0)
    (enc "hash" c nb-1 price price price (pubk hash-2))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3063)
  (parent 2515)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3064)
  (parent 2516)
  (seen 2926)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 10 6 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3065)
  (parent 2517)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3066)
  (parent 2518)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3067)
  (parent 2518)
  (seen 3515)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3068)
  (parent 2518)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3069)
  (parent 2519)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3070)
  (parent 2519)
  (seen 3065)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3071)
  (parent 2520)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3072)
  (parent 2521)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3073)
  (parent 2522)
  (seen 2941)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3074)
  (parent 2523)
  (seen 2944)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3075)
  (parent 2526)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3076)
  (parent 2527)
  (seen 2816)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3077)
  (parent 2528)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3078)
  (parent 2529)
  (seen 2357)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3079)
  (parent 2530)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3080)
  (parent 2531)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0))
    ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3081)
  (parent 2531)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 3 bank 2) price (7 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3082)
  (parent 2531)
  (seen 3066 3526)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3083)
  (parent 2531)
  (seen 3076)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3084)
  (parent 2532)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3085)
  (parent 2533)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3086)
  (parent 2534)
  (seen 2963)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 10 6 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3087)
  (parent 2535)
  (seen 3528)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3088)
  (parent 2536)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3089)
  (parent 2536)
  (seen 3087)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (0 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (2 0)) ((5 0) (4 0)) ((5 2) (3 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 3090)
  (parent 2537)
  (seen 436)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (8 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3091)
  (parent 2540)
  (seen 2538)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3092)
  (parent 2541)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3093)
  (parent 2541)
  (seen 2349)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3094)
  (parent 2542)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3095)
  (parent 2543)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3096)
  (parent 2544)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3097)
  (parent 2545)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3098)
  (parent 2546)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3099)
  (parent 2547)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (8 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3100)
  (parent 2547)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 10 3 bank 2) price (8 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3101)
  (parent 2547)
  (seen 3545)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3102)
  (parent 2547)
  (seen 3095)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3103)
  (parent 2548)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (8 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3104)
  (parent 2548)
  (seen 2960)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3105)
  (parent 2549)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3106)
  (parent 2550)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 11 7 customer 3) price (10 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3107)
  (parent 2551)
  (seen 3547)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3108)
  (parent 2552)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3109)
  (parent 2552)
  (seen 3107)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (0 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (2 0)) ((5 0) (4 0)) ((5 2) (3 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 3110)
  (parent 2554)
  (seen 449)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nm price nm-0 price-0 acctnum goods text)
    (c hash hash-0 b m name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc price) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc price)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (5 1)) ((1 1) (5 3)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (2 0)) ((5 0) (4 0)) ((5 2) (3 0))
    ((5 4) (0 2)))
  (non-orig (privk c) (privk hash) (privk hash-0) (privk b) (privk m))
  (uniq-orig nb nm price nm-0 acctnum)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price (pubk c))))
    ((recv (enc c price acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm-0 m price-0 (pubk c))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m acctnum (pubk c)))
      (send (enc c price acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) (enc price nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum acctnum (pubk hash))
            (privk b)) nb))))
  (label 3111)
  (parent 2555)
  (seen 436)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (8 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3112)
  (parent 2558)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (8 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3113)
  (parent 2558)
  (seen 3091)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3114)
  (parent 2558)
  (seen 1998)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3115)
  (parent 2559)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3116)
  (parent 2560)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (8 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3117)
  (parent 2560)
  (unrealized (0 0) (0 2) (1 0) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 5 bank 2) price (8 0)
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3118)
  (parent 2560)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3119)
  (parent 2561)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (8 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 3120)
  (parent 2561)
  (unrealized (0 0) (0 2) (1 0) (8 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (9 0)
    (enc price-0 price m price (pubk c)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 3121)
  (parent 2562)
  (seen 3568)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "5 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3122)
  (parent 2563)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (9 0)
    (enc price-0 price m price (pubk c)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 3123)
  (parent 2564)
  (seen 3568)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 3124)
  (parent 2565)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3125)
  (parent 2565)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3126)
  (parent 2565)
  (seen 3572)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3127)
  (parent 2567)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3128)
  (parent 2568)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3129)
  (parent 2570)
  (seen 2969)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3130)
  (parent 2571)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3131)
  (parent 2572)
  (seen 2972)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3132)
  (parent 2573)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3133)
  (parent 2574)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3134)
  (parent 2575)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3135)
  (parent 2576)
  (seen 2977)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3136)
  (parent 2576)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3137)
  (parent 2577)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3138)
  (parent 2578)
  (seen 2981)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-1 (6 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3139)
  (parent 2579)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3140)
  (parent 2580)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3141)
  (parent 2582)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3142)
  (parent 2583)
  (seen 2986)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3143)
  (parent 2586)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3144)
  (parent 2588)
  (seen 2989)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3145)
  (parent 2591)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3146)
  (parent 2592)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3147)
  (parent 2592)
  (seen 1402)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3148)
  (parent 2593)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3149)
  (parent 2595)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3150)
  (parent 2595)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3151)
  (parent 2596)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3152)
  (parent 2597)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3153)
  (parent 2597)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3154)
  (parent 2598)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3155)
  (parent 2599)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3156)
  (parent 2600)
  (seen 2998)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3157)
  (parent 2600)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3158)
  (parent 2601)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3159)
  (parent 2602)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3160)
  (parent 2605)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3161)
  (parent 2606)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3162)
  (parent 2606)
  (seen 2992)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3163)
  (parent 2607)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3164)
  (parent 2608)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3165)
  (parent 2609)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3166)
  (parent 2610)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3167)
  (parent 2612)
  (seen 3006)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3168)
  (parent 2613)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3169)
  (parent 2614)
  (seen 3009)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 3170)
  (parent 2616)
  (seen 3049)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (2 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 3171)
  (parent 2617)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 3172)
  (parent 2618)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c nc price price price (pubk hash-3))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) price)))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-3))
            (privk b-0)) (enc nc price (pubk c))))))
  (label 3173)
  (parent 2619)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 3174)
  (parent 2620)
  (seen 2311)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 3175)
  (parent 2621)
  (seen 2722)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nm price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c))))))
  (label 3176)
  (parent 2625)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nm price goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc acctnum-0 nb-0 (pubk c)) (enc acctnum-0 nb-0 m nb-0 (pubk c))
    (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c))))))
  (label 3177)
  (parent 2626)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 3178)
  (parent 2627)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3179)
  (parent 2631)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3180)
  (parent 2633)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3181)
  (parent 2641)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3182)
  (parent 2642)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (7 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3183)
  (parent 2647)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (1 0)) ((6 1) (5 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 3184)
  (parent 2648)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (7 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3185)
  (parent 2648)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3186)
  (parent 2649)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 9 7 bank 2) price (8 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3187)
  (parent 2650)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3188)
  (parent 2651)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3189)
  (parent 2651)
  (seen 1865 3622)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3190)
  (parent 2651)
  (seen 1868 1869)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3191)
  (parent 2653)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3192)
  (parent 2653)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-1))
    nb-1 (6 0) (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3193)
  (parent 2654)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 9 3 bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3194)
  (parent 2654)
  (seen 3621 3623 3624)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3195)
  (parent 2654)
  (unrealized (0 0) (0 2) (1 0) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3196)
  (parent 2655)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 7 bank 2) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3197)
  (parent 2656)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3198)
  (parent 2658)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3199)
  (parent 2658)
  (seen 1895)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 7 bank 2) price (6 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3200)
  (parent 2658)
  (seen 1896 1897)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3201)
  (parent 2659)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3202)
  (parent 2659)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-0))
    nb-0 (6 0) (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3203)
  (parent 2660)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 3 bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3204)
  (parent 2660)
  (seen 3631 3632 3633)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3205)
  (parent 2660)
  (unrealized (0 0) (0 2) (1 0) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 3206)
  (parent 2662)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 3207)
  (parent 2663)
  (seen 3185)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3208)
  (parent 2665)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3209)
  (parent 2665)
  (seen 1923)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 4 bank 2) price (7 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-1))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3210)
  (parent 2665)
  (seen 3025)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c nb-0 nb-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-3))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3211)
  (parent 2665)
  (seen 1922 1927 1928 3645)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3212)
  (parent 2666)
  (seen 3192)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 8 bank 2) price-0 (9 0)
    (enc nb-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3213)
  (parent 2667)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3214)
  (parent 2669)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3215)
  (parent 2669)
  (seen 1945)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-3))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3216)
  (parent 2669)
  (seen 1944 1946 1947)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 4 bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3217)
  (parent 2669)
  (seen 3040)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3218)
  (parent 2670)
  (seen 3202)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 8 bank 2) price-0 (9 0)
    (enc goods-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3219)
  (parent 2671)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (5 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 3220)
  (parent 2673)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 acctnum-1 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (displaced 8 4 customer 3) nb-0 (7 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 3221)
  (parent 2674)
  (seen 1643)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 3222)
  (parent 2676)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 3223)
  (parent 2677)
  (seen 3185)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3224)
  (parent 2679)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3225)
  (parent 2679)
  (seen 2025)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 4 bank 2) price (7 0)
    (enc "hash" c nb-1 nb-1 price price (pubk hash-1))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price m price (pubk c))
    (enc c nb-1 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3226)
  (parent 2679)
  (seen 3137)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nb-0 nm price acctnum-1 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-0)
    (nm nb-1) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c nb-0 nb-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price-0 price-0 (pubk hash-3))
    (enc nb-0 price-0 m price-0 (pubk c))
    (enc c nb-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3227)
  (parent 2679)
  (seen 2024 2029 2030 3666)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3228)
  (parent 2680)
  (seen 3192)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 8 bank 2) price-0 (9 0)
    (enc nb-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3229)
  (parent 2681)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nm price-0 acctnum-1 nb-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (displaced 9 6 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-1 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3230)
  (parent 2683)
  (seen 3159)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3231)
  (parent 2684)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0))
    ((6 0) (0 0)) ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3232)
  (parent 2684)
  (seen 2056)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-3))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3233)
  (parent 2684)
  (seen 2052 2055 2057)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3234)
  (parent 2685)
  (seen 3202)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 8 bank 2) price-0 (9 0)
    (enc goods-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3235)
  (parent 2686)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 3236)
  (parent 2688)
  (seen 2155)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nm price goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 3237)
  (parent 2689)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 8 3 bank 2) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 3238)
  (parent 2689)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 3239)
  (parent 2689)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum acctnum-0 nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc nc)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (6 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b-0)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 3240)
  (parent 2690)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3241)
  (parent 2692)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3242)
  (parent 2692)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 7 2 customer 3) nb-1 (6 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3243)
  (parent 2693)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3244)
  (parent 2694)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3245)
  (parent 2694)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 8 3 customer 3) nb-1 (7 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3246)
  (parent 2695)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (4 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3247)
  (parent 2696)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3248)
  (parent 2698)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 8 4 bank 2) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 3249)
  (parent 2699)
  (seen 3660)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 3250)
  (parent 2699)
  (unrealized (0 0) (0 2) (1 0) (6 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3251)
  (parent 2702)
  (seen 3242)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 0) (0 0)) ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3252)
  (parent 2702)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (5 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3253)
  (parent 2703)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (5 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3254)
  (parent 2703)
  (unrealized (0 0) (0 2) (1 0) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 4 customer 3) nb-1 (8 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3255)
  (parent 2704)
  (seen 3697)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3256)
  (parent 2705)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3257)
  (parent 2713)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3258)
  (parent 2714)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3259)
  (parent 2716)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3260)
  (parent 2717)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3261)
  (parent 2719)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3262)
  (parent 2720)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum acctnum (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3263)
  (parent 2722)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum acctnum (pubk hash))
            (privk b)) (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m acctnum (pubk c)))
      (send (enc c nc acctnum acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum acctnum (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum acctnum (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3264)
  (parent 2723)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3265)
  (parent 2725)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3266)
  (parent 2726)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c))))))
  (label 3267)
  (parent 2728)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3268)
  (parent 2728)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3269)
  (parent 2729)
  (seen 3268)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3270)
  (parent 2731)
  (seen 3268)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3271)
  (parent 2734)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3272)
  (parent 2734)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3273)
  (parent 2735)
  (seen 3272)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3274)
  (parent 2738)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (8 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3275)
  (parent 2739)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (8 0))
    ((4 0) (0 0)) ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3276)
  (parent 2739)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3277)
  (parent 2740)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3278)
  (parent 2740)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3279)
  (parent 2741)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 3280)
  (parent 2743)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3281)
  (parent 2743)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (displaced 9 4 customer 3) nb-1 (8 0)
    (enc nc nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 3282)
  (parent 2744)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3283)
  (parent 2745)
  (seen 3281)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0))
    ((4 2) (3 0)) ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (added-strand bank 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-1 (pubk c))
    (enc nc nb-1 m nb-1 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 3284)
  (parent 2745)
  (seen 3712)
  (unrealized (0 0) (0 2) (1 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3285)
  (parent 2747)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3286)
  (parent 2748)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3287)
  (parent 2748)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3288)
  (parent 2749)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3289)
  (parent 2752)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 3290)
  (parent 2753)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3291)
  (parent 2753)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3292)
  (parent 2757)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 3 customer 3) price (6 0)
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3293)
  (parent 2758)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3294)
  (parent 2759)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3295)
  (parent 2762)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 3296)
  (parent 2763)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3297)
  (parent 2763)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3298)
  (parent 2767)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 8 3 customer 3) price (6 0)
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3299)
  (parent 2768)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3300)
  (parent 2769)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3301)
  (parent 2772)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 3302)
  (parent 2773)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3303)
  (parent 2773)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nc)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb acctnum)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb acctnum price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc acctnum m price (pubk c)))
      (send (enc c nc acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-0))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc acctnum acctnum price (pubk hash-1))
            (privk b)) (enc nc acctnum (pubk c)))))
    ((recv (enc c nc acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods acctnum price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3304)
  (parent 2777)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price acctnum acctnum price (pubk hash-1))
    (enc "hash" c price goods-0 acctnum price (pubk hash-2))
    (enc price acctnum (pubk c)) (enc price goods-0 (pubk c))
    (enc price acctnum m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 3305)
  (parent 2778)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
    (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
    (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
    (enc price acctnum-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price acctnum-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3306)
  (parent 2778)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-2))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3307)
  (parent 2782)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nb-0)
    (nm acctnum) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
    (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-2))
    (enc "hash" c nb-0 nb-0 acctnum nb-0 (pubk hash-1))
    (enc "hash" c nb-0 goods-0 acctnum nb-0 (pubk hash-3))
    (enc nb-0 acctnum (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 acctnum m nb-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum nb-0 (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 acctnum m nb-0 (pubk c)))
      (send (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 acctnum nb-0 (pubk hash-3))
            (privk b)) (enc nb-0 goods-0 (pubk c))))))
  (label 3308)
  (parent 2783)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nb-0)
    (nm acctnum-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb acctnum-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb acctnum-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-0))
    (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-2))
    (enc "hash" c nb-0 nb-0 acctnum-0 nb-0 (pubk hash-1))
    (enc "hash" c nb-0 goods-0 acctnum-0 nb-0 (pubk hash-3))
    (enc nb-0 acctnum-0 (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 acctnum-0 m nb-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum-0 nb-0 (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 acctnum-0 m nb-0 (pubk c)))
      (send (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nb-0 acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nb-0 acctnum-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nb-0 acctnum-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 acctnum-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3309)
  (parent 2783)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 nb-0 goods-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb nb-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc nb-0)
    (nm acctnum-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb acctnum-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb acctnum-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm acctnum-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 8 2 bank 2) price-0 (6 0)
    (enc price-0 acctnum-0 m price-0 (pubk c))
    (enc price-0 nm m price (pubk c)) (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum-0 nb-0 (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 acctnum-0 m nb-0 (pubk c)))
      (send (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nb-0 acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nb-0 acctnum-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c nb-0 acctnum-0 acctnum-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nb-0 acctnum-0 (pubk c)))))
    ((recv (enc c nb-0 acctnum-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 acctnum-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3310)
  (parent 2784)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb acctnum-0) (nc price)
    (nm acctnum-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb acctnum-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm acctnum-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 8 3 customer 3) price (6 0)
    (enc price acctnum-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum-0 (pubk hash)) (privk m))))
    ((recv (enc c price acctnum-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum-0 price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum-0 m price (pubk c)))
      (send (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) acctnum-0)))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-0))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price acctnum-0 acctnum-0 price (pubk hash-1))
            (privk b-0)) (enc price acctnum-0 (pubk c)))))
    ((recv (enc c price acctnum-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c price goods-0 acctnum-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3311)
  (parent 2784)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-2))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3312)
  (parent 2785)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods text) (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3313)
  (parent 2787)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3314)
  (parent 2788)
  (seen 2346)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods text) (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3315)
  (parent 2789)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3316)
  (parent 2790)
  (seen 2348)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3317)
  (parent 2797)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3318)
  (parent 2797)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3319)
  (parent 2798)
  (seen 3318)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3320)
  (parent 2799)
  (seen 3318)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (4 0)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3321)
  (parent 2800)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3322)
  (parent 2802)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3323)
  (parent 2803)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3324)
  (parent 2804)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3325)
  (parent 2806)
  (seen 3318)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3326)
  (parent 2810)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3327)
  (parent 2811)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3328)
  (parent 2813)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3329)
  (parent 2814)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3330)
  (parent 2816)
  (seen 3318)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3331)
  (parent 2817)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 9 2 bank 2) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3332)
  (parent 2817)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 0) (8 0)) ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0))
    ((7 1) (3 0)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3333)
  (parent 2818)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 acctnum-0 nb-0 nm price-0 nm-0
      price-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((8 1) (4 0)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 10 2 bank 2) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3334)
  (parent 2818)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 acctnum-0 nm price-0 nm-0 price-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 3 customer 3) price-0 (8 0)
    (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3335)
  (parent 2819)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nb-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3336)
  (parent 2820)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3337)
  (parent 2822)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3338)
  (parent 2823)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3339)
  (parent 2825)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-1)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-1 (pubk c)))))
  (label 3340)
  (parent 2826)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1
      text) (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm nm-0)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm-0 m price-1 (pubk c)))))
  (label 3341)
  (parent 2826)
  (seen 2710)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((5 1) (6 0))
    ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-2))
    (enc "hash" c price-0 nb-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 goods-0 (pubk c))
    (enc price-0 nb-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3342)
  (parent 2827)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc goods-0 price price (pubk hash-3))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3343)
  (parent 2829)
  (seen 2881)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-2))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 3344)
  (parent 2831)
  (seen 3049)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3345)
  (parent 2832)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 3346)
  (parent 2834)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (5 0)) ((3 2) (2 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) (enc acctnum price (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) (enc acctnum price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum price price price (pubk hash-0))
            (privk b)) (enc acctnum price (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 3347)
  (parent 2835)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3348)
  (parent 2836)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((6 1) (7 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3349)
  (parent 2837)
  (seen 2725)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1
      text) (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (7 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0))
    ((6 1) (4 0)) ((7 1) (1 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc nc nm-0 m price-1 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm-0 m price-1 (pubk c)))))
  (label 3350)
  (parent 2838)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 nm-0
      price-1 text) (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum-0) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (7 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0))
    ((6 1) (4 0)) ((6 1) (7 0)) ((7 1) (1 0)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 3 customer 3) price-0 (7 0)
    (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3351)
  (parent 2839)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3352)
  (parent 2841)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 3353)
  (parent 2842)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3354)
  (parent 2842)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3355)
  (parent 2846)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 3356)
  (parent 2847)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3357)
  (parent 2847)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 3358)
  (parent 2850)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (8 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 1) (1 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3359)
  (parent 2850)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3360)
  (parent 2852)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-3))
    (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
    (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
    (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
    (enc nb-1 goods-0 (pubk c)) (enc nb-1 nb-0 (pubk c))
    (enc nb-1 nb-1 (pubk c)) (enc nb-1 nb-0 m nb-1 (pubk c))
    (enc c nb-1 goods-0 (pubk m))
    (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-3))
            (privk b)) (enc nb-1 goods-0 (pubk c))))))
  (label 3361)
  (parent 2853)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-1)
    (nm nb-0) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-1 (1 0)
    (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-3))
    (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
    (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
    (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
    (enc nb-1 goods-0 (pubk c)) (enc nb-1 nb-0 (pubk c))
    (enc nb-1 nb-1 (pubk c)) (enc nb-1 nb-0 m nb-1 (pubk c))
    (enc c nb-1 goods-0 (pubk m))
    (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-1 nb-1 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-1 goods-0 (pubk c)))))
    ((recv (enc c nb-1 goods-0 (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 3362)
  (parent 2853)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test
    (contracted (b-0 b) (acctnum-0 acctnum) (hash-3 hash-2)
      (nb-1 goods-0)) acctnum (1 0)
    (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-0 m acctnum (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c))))))
  (label 3363)
  (parent 2854)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (6 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (added-strand bank 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
    (enc acctnum-0 goods-0 (pubk c)) (enc acctnum-0 nb-0 (pubk c))
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-3)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c))))))
  (label 3364)
  (parent 2854)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
    (enc acctnum-0 goods-0 (pubk c)) (enc acctnum-0 nb-0 (pubk c))
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 3365)
  (parent 2854)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nb-1)
    (nm nb-0) (goods goods-0) (price nb-1) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (displaced 8 5 bank 2) price-0 (6 0)
    (enc price-0 nb-0 m price-0 (pubk c))
    (enc price-0 nm m price (pubk c)) (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-1 nb-1 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-1 goods-0 (pubk c)))))
    ((recv (enc c nb-1 goods-0 (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 3366)
  (parent 2855)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 2 customer 3) price (6 0)
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3367)
  (parent 2855)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 8 2 customer 3) price-0 (6 0)
    (enc price-0 nb-0 m price-0 (pubk c))
    (enc price-0 nm m price (pubk c)) (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-1 nb-0 acctnum-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-3)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 3368)
  (parent 2855)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3369)
  (parent 2856)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-1) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 8 2 customer 3) price-0 (7 0)
    (enc price-0 nb-0 m price-0 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-1 nb-0 acctnum-0 (pubk hash-2))
            (privk b-0)) (enc acctnum-0 nb-1 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-3)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c))))))
  (label 3370)
  (parent 2857)
  (seen 3775 3776)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3371)
  (parent 2860)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3372)
  (parent 2860)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3373)
  (parent 2861)
  (seen 3372)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3374)
  (parent 2862)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3375)
  (parent 2862)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3376)
  (parent 2863)
  (seen 3375)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3377)
  (parent 2867)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c))))))
  (label 3378)
  (parent 2868)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3379)
  (parent 2868)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (3 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3380)
  (parent 2872)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum acctnum-0 goods-0 nb-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc price)
    (nm nb-0) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price) (nm nb-0)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 8 2 customer 3) price (6 0)
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b-0)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b-0)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b-0)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3381)
  (parent 2873)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3382)
  (parent 2874)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3383)
  (parent 2877)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3384)
  (parent 2878)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3385)
  (parent 2879)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3386)
  (parent 2881)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3387)
  (parent 2882)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3388)
  (parent 2886)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3389)
  (parent 2886)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3390)
  (parent 2887)
  (seen 3389)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3391)
  (parent 2889)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3392)
  (parent 2890)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nm price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3393)
  (parent 2893)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3394)
  (parent 2894)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3395)
  (parent 2897)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3396)
  (parent 2898)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3397)
  (parent 2903)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3398)
  (parent 2904)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3399)
  (parent 2904)
  (seen 2883)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 3400)
  (parent 2905)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3401)
  (parent 2905)
  (seen 2885)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nm price goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (8 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 9 7 bank 2) nb-1 (8 0)
    (enc nc nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 3402)
  (parent 2906)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 nm price goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (displaced 9 4 customer 3) nb-0 (8 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc goods-0 (pubk c))))))
  (label 3403)
  (parent 2906)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3404)
  (parent 2907)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3405)
  (parent 2907)
  (seen 3386)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nm price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3406)
  (parent 2911)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3407)
  (parent 2912)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (7 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3408)
  (parent 2918)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (7 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3409)
  (parent 2919)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3410)
  (parent 2922)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3411)
  (parent 2922)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (8 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3412)
  (parent 2923)
  (seen 3411)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3413)
  (parent 2924)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3414)
  (parent 2924)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3415)
  (parent 2926)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3416)
  (parent 2927)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3417)
  (parent 2928)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 goods-0 nb-1 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3418)
  (parent 2928)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (6 0)
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3419)
  (parent 2929)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3420)
  (parent 2930)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3421)
  (parent 2930)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3422)
  (parent 2931)
  (seen 3421)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3423)
  (parent 2932)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (added-strand bank 2) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3424)
  (parent 2932)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3425)
  (parent 2932)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3426)
  (parent 2934)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3427)
  (parent 2934)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3428)
  (parent 2935)
  (seen 3427)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3429)
  (parent 2938)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3430)
  (parent 2938)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3431)
  (parent 2939)
  (seen 3430)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3432)
  (parent 2941)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3433)
  (parent 2942)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3434)
  (parent 2944)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3435)
  (parent 2945)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3436)
  (parent 2948)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3437)
  (parent 2949)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3438)
  (parent 2951)
  (seen 3411)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3439)
  (parent 2952)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3440)
  (parent 2953)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3441)
  (parent 2953)
  (seen 3415)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3442)
  (parent 2954)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 10 3 bank 2) price (7 0)
    (enc "hash" c nb-1 price price price (pubk hash-4))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3443)
  (parent 2954)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3444)
  (parent 2955)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3445)
  (parent 2957)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3446)
  (parent 2957)
  (seen 3443)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3447)
  (parent 2958)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3448)
  (parent 2961)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3449)
  (parent 2964)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3450)
  (parent 2965)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3451)
  (parent 2965)
  (seen 3434)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3452)
  (parent 2969)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3453)
  (parent 2972)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3454)
  (parent 2973)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3455)
  (parent 2973)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3456)
  (parent 2975)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3457)
  (parent 2975)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 3458)
  (parent 2977)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3459)
  (parent 2978)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3460)
  (parent 2979)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3461)
  (parent 2981)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3462)
  (parent 2982)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3463)
  (parent 2982)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3464)
  (parent 2986)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (5 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 3465)
  (parent 2989)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3466)
  (parent 2991)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3467)
  (parent 2993)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3468)
  (parent 2993)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3469)
  (parent 2995)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3470)
  (parent 2996)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3471)
  (parent 2996)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 3472)
  (parent 2998)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3473)
  (parent 2999)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3474)
  (parent 3000)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3475)
  (parent 3001)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3476)
  (parent 3001)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3477)
  (parent 3003)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3478)
  (parent 3003)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 3479)
  (parent 3006)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 3480)
  (parent 3009)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 10 3 bank 2) price (7 0)
    (enc "hash" c nb-1 price price price (pubk hash-4))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3481)
  (parent 3011)
  (seen 3814)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3482)
  (parent 3013)
  (seen 3481)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3483)
  (parent 3017)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3484)
  (parent 3019)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3485)
  (parent 3020)
  (seen 3455)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3486)
  (parent 3022)
  (seen 3457)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3487)
  (parent 3023)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3488)
  (parent 3024)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3489)
  (parent 3025)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3490)
  (parent 3026)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3491)
  (parent 3027)
  (seen 3463)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3492)
  (parent 3030)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3493)
  (parent 3032)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3494)
  (parent 3033)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3495)
  (parent 3035)
  (seen 3468)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3496)
  (parent 3036)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3497)
  (parent 3037)
  (seen 3471)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3498)
  (parent 3038)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3499)
  (parent 3039)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3500)
  (parent 3040)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3501)
  (parent 3041)
  (seen 3476)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3502)
  (parent 3043)
  (seen 3478)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3503)
  (parent 3045)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3504)
  (parent 3047)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3505)
  (parent 3049)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3506)
  (parent 3050)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 3507)
  (parent 3055)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3508)
  (parent 3055)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-3))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-1 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3509)
  (parent 3056)
  (seen 3508)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3510)
  (parent 3060)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3511)
  (parent 3062)
  (seen 3411)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3512)
  (parent 3063)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3513)
  (parent 3064)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3514)
  (parent 3065)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 10 3 bank 2) price (7 0)
    (enc "hash" c nb-1 price price price (pubk hash-4))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3515)
  (parent 3065)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3516)
  (parent 3066)
  (seen 3421)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3517)
  (parent 3068)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3518)
  (parent 3068)
  (seen 3515)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3519)
  (parent 3069)
  (seen 3427)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3520)
  (parent 3072)
  (seen 3430)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3521)
  (parent 3073)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3522)
  (parent 3074)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3523)
  (parent 3077)
  (seen 2918)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3524)
  (parent 3078)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3525)
  (parent 3079)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 3 bank 2) price (7 0)
    (enc "hash" c nb-1 price price price (pubk hash-2))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3526)
  (parent 3079)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3527)
  (parent 3080)
  (seen 3441)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 10 6 customer 3) price (9 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3528)
  (parent 3081)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3529)
  (parent 3082)
  (seen 3882)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 6 customer 3) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3530)
  (parent 3082)
  (seen 3517)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3531)
  (parent 3083)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 goods-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (7 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0))
    ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3532)
  (parent 3083)
  (seen 3528)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3533)
  (parent 3085)
  (seen 2941)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3534)
  (parent 3086)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3535)
  (parent 3087)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3536)
  (parent 3088)
  (seen 3451)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3537)
  (parent 3093)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3538)
  (parent 3094)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (8 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3539)
  (parent 3094)
  (seen 2915)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3540)
  (parent 3095)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3541)
  (parent 3096)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3542)
  (parent 3097)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3543)
  (parent 3097)
  (seen 2920)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3544)
  (parent 3098)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 10 3 bank 2) price (8 0)
    (enc "hash" c nb-1 price price price (pubk hash-2))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3545)
  (parent 3098)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3546)
  (parent 3099)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 11 7 customer 3) price (10 0)
    (enc nb-0 price m price (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3547)
  (parent 3100)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3548)
  (parent 3101)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (8 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3549)
  (parent 3101)
  (seen 3899)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 10 7 customer 3) price (8 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3550)
  (parent 3101)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3551)
  (parent 3102)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0
      acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (8 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3552)
  (parent 3102)
  (seen 3547)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (8 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3553)
  (parent 3104)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3554)
  (parent 3105)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3555)
  (parent 3106)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3556)
  (parent 3106)
  (seen 2936)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3557)
  (parent 3107)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3558)
  (parent 3108)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (8 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3559)
  (parent 3113)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (7 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3560)
  (parent 3114)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3561)
  (parent 3115)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3562)
  (parent 3115)
  (seen 3537)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (4 0)) ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3563)
  (parent 3115)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3564)
  (parent 3116)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (9 0)
    (enc price-0 price m price (pubk c)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3565)
  (parent 3117)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3566)
  (parent 3118)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3567)
  (parent 3119)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (9 0)
    (enc price-0 price m price (pubk c)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 3568)
  (parent 3120)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 3569)
  (parent 3121)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (8 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3570)
  (parent 3121)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm nm-0 price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 10 3 bank 2) price (8 0)
    (enc "hash" c price-1 price-1 price price (pubk hash-0))
    (enc "hash" c price-1 nb-0 price price (pubk hash-3))
    (enc price-1 price m price (pubk c))
    (enc c price-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3571)
  (parent 3121)
  (seen 3923)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "4 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 7 customer 3) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3572)
  (parent 3121)
  (seen 3565)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "3 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3573)
  (parent 3122)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0))
    ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3574)
  (parent 3122)
  (seen 3553)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3575)
  (parent 3122)
  (seen 3553)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 3576)
  (parent 3123)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 nb-0 (pubk c))
    (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 3577)
  (parent 3124)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 11 7 customer 3) price (10 0)
    (enc price-0 price m price (pubk c)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3578)
  (parent 3125)
  (seen 3925)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3579)
  (parent 3126)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 3580)
  (parent 3126)
  (seen 3578)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3581)
  (parent 3129)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3582)
  (parent 3131)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3583)
  (parent 3132)
  (seen 3455)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3584)
  (parent 3134)
  (seen 3457)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3585)
  (parent 3135)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3586)
  (parent 3136)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3587)
  (parent 3137)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3588)
  (parent 3138)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3589)
  (parent 3139)
  (seen 3463)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3590)
  (parent 3142)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (6 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3591)
  (parent 3144)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3592)
  (parent 3145)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3593)
  (parent 3145)
  (seen 1881)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3594)
  (parent 3148)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3595)
  (parent 3150)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3596)
  (parent 3151)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3597)
  (parent 3152)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3598)
  (parent 3152)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3599)
  (parent 3153)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3600)
  (parent 3155)
  (seen 3478)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3601)
  (parent 3156)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3602)
  (parent 3157)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3603)
  (parent 3158)
  (seen 3471)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3604)
  (parent 3159)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3605)
  (parent 3160)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3606)
  (parent 3160)
  (seen 3466)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3607)
  (parent 3163)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3608)
  (parent 3164)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3609)
  (parent 3164)
  (seen 3469)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3610)
  (parent 3165)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3611)
  (parent 3167)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3612)
  (parent 3169)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 3613)
  (parent 3170)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nc (1 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc price price price (pubk hash-3))
    (enc "hash" c nc goods-0 price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-0))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-3))
            (privk b)) (enc nc price (pubk c))))))
  (label 3614)
  (parent 3173)
  (seen 2881)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 3615)
  (parent 3174)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 3616)
  (parent 3175)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nm price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-3))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-1 (pubk c)) (enc acctnum nm m price (pubk c))
    (enc acctnum nb-0 m nb-0 (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 3617)
  (parent 3177)
  (seen 3508)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (7 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3618)
  (parent 3185)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3619)
  (parent 3186)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3620)
  (parent 3186)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-1))
    nb-1 (6 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3621)
  (parent 3187)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 4 bank 2) nb-0 (6 0)
    (enc "hash" c nb-1 nb-1 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nb-1 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nb-1 nb-0 nb-0 nb-0 (pubk hash-4))
    (enc nb-1 nb-0 (pubk c)) (enc nb-1 nb-0 m nb-0 (pubk c))
    (enc c nb-1 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3622)
  (parent 3187)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 9 4 bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3623)
  (parent 3187)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (6 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3624)
  (parent 3187)
  (unrealized (0 0) (0 2) (1 0) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3625)
  (parent 3188)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3626)
  (parent 3192)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3627)
  (parent 3193)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 9 4 customer 3) nb-1 (8 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3628)
  (parent 3195)
  (seen 3965)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3629)
  (parent 3196)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3630)
  (parent 3196)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-0))
    nb-0 (6 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3631)
  (parent 3197)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 4 bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3632)
  (parent 3197)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (5 0)) ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3633)
  (parent 3197)
  (unrealized (0 0) (0 2) (1 0) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3634)
  (parent 3198)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 7 bank 2) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3635)
  (parent 3199)
  (seen 3974 3975)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3636)
  (parent 3202)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3637)
  (parent 3203)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3638)
  (parent 3203)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 4 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3639)
  (parent 3205)
  (seen 3976)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 3640)
  (parent 3207)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3641)
  (parent 3208)
  (seen 3620)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 8 bank 2) price-0 (9 0)
    (enc nb-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3642)
  (parent 3209)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3643)
  (parent 3210)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3644)
  (parent 3210)
  (seen 2461 3985)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-3))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3645)
  (parent 3210)
  (seen 2464 2465)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3646)
  (parent 3212)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-1))
    nb-1 (7 0) (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3647)
  (parent 3213)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 3 bank 2) nb-1 (7 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3648)
  (parent 3213)
  (seen 3984 3986 3987)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (7 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3649)
  (parent 3213)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3650)
  (parent 3214)
  (seen 3630)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 8 bank 2) price-0 (9 0)
    (enc goods-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3651)
  (parent 3215)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3652)
  (parent 3217)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3653)
  (parent 3217)
  (seen 2482)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-3))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3654)
  (parent 3217)
  (seen 2483 2484)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3655)
  (parent 3218)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-0))
    nb-0 (7 0) (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3656)
  (parent 3219)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 3 bank 2) nb-0 (7 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3657)
  (parent 3219)
  (seen 3992 3993 3994)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (7 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3658)
  (parent 3219)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 3659)
  (parent 3220)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3660)
  (parent 3220)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 4) (1 0)) ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))))
  (label 3661)
  (parent 3223)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3662)
  (parent 3224)
  (seen 3620)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 8 bank 2) price-0 (9 0)
    (enc nb-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3663)
  (parent 3225)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3664)
  (parent 3226)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3665)
  (parent 3226)
  (seen 2573 4005)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-3))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3666)
  (parent 3226)
  (seen 2576 2577)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3667)
  (parent 3228)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-1))
    nb-1 (7 0) (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3668)
  (parent 3229)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 3 bank 2) nb-1 (7 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3669)
  (parent 3229)
  (seen 4004 4006 4007)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (7 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3670)
  (parent 3229)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum acctnum-0 goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb nb-0) (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum-0 price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3671)
  (parent 3230)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (1 0))
    ((3 1) (4 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3672)
  (parent 3230)
  (seen 2601)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum acctnum-0 goods-0 nm price acctnum-1 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-1) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 8 bank 2) price-0 (7 0)
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price-0 price-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price-0 price-0 (pubk hash-3))
    (enc goods-0 price-0 m price-0 (pubk c))
    (enc c goods-0 price-0 acctnum-0 price-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3673)
  (parent 3230)
  (seen 2600 2602)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3674)
  (parent 3231)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 8 bank 2) price-0 (9 0)
    (enc goods-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3675)
  (parent 3232)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3676)
  (parent 3234)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-0))
    nb-0 (7 0) (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3677)
  (parent 3235)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 3 bank 2) nb-0 (7 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3678)
  (parent 3235)
  (seen 4015 4016 4017)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "3 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (7 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3679)
  (parent 3235)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 3680)
  (parent 3236)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nm price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 3681)
  (parent 3237)
  (seen 2719)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nm price goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c))))))
  (label 3682)
  (parent 3238)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 3683)
  (parent 3239)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods-0 price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-2))
    (enc nc price (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 3684)
  (parent 3240)
  (seen 2725)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0))
    ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3685)
  (parent 3242)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3686)
  (parent 3243)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3687)
  (parent 3244)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3688)
  (parent 3244)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 8 3 customer 3) nb-1 (7 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3689)
  (parent 3245)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (4 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3690)
  (parent 3246)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3691)
  (parent 3247)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods acctnum acctnum-0 nm price goods-0 acctnum-1 nb-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-1) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (3 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (displaced 9 5 customer 3) nb-0 (8 0)
    (enc nc nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-1 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 3692)
  (parent 3250)
  (seen 2159)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3693)
  (parent 3251)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 8 3 customer 3) nb-1 (7 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3694)
  (parent 3252)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3695)
  (parent 3253)
  (seen 3688)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3696)
  (parent 3253)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 4 customer 3) nb-1 (8 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3697)
  (parent 3254)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (5 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3698)
  (parent 3255)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3699)
  (parent 3268)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3700)
  (parent 3269)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3701)
  (parent 3270)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3702)
  (parent 3272)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3703)
  (parent 3273)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (8 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3704)
  (parent 3275)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (8 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3705)
  (parent 3275)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (8 0))
    ((4 0) (0 0)) ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3706)
  (parent 3276)
  (seen 3705)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3707)
  (parent 3277)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3708)
  (parent 3277)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3709)
  (parent 3278)
  (seen 3708)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3710)
  (parent 3281)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-1 m nb-1 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3711)
  (parent 3282)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nb-0 goods-0 nb-1 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-1 m nb-1 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3712)
  (parent 3282)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3713)
  (parent 3283)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3714)
  (parent 3286)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3715)
  (parent 3286)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3716)
  (parent 3287)
  (seen 3715)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c))))))
  (label 3717)
  (parent 3290)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3718)
  (parent 3290)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3719)
  (parent 3291)
  (seen 3718)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3720)
  (parent 3293)
  (seen 3718)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c))))))
  (label 3721)
  (parent 3296)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3722)
  (parent 3296)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3723)
  (parent 3297)
  (seen 3722)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price goods-0 acctnum price (pubk hash-2))
    (enc "hash" c price nb-0 acctnum price (pubk hash-1))
    (enc price acctnum (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 (pubk c)) (enc price acctnum m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3724)
  (parent 3299)
  (seen 3722)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c))))))
  (label 3725)
  (parent 3302)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3726)
  (parent 3302)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc price nb-0 (pubk c)) (enc price goods-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3727)
  (parent 3303)
  (seen 3726)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c))))))
  (label 3728)
  (parent 3305)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3729)
  (parent 3305)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price acctnum acctnum price (pubk hash-1))
    (enc "hash" c price goods-0 acctnum price (pubk hash-2))
    (enc price acctnum (pubk c)) (enc price goods-0 (pubk c))
    (enc price acctnum m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3730)
  (parent 3306)
  (seen 3729)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nb-0)
    (nm acctnum) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum nb-0 (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m nb-0 (pubk c)))
      (send (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum nb-0 (pubk hash-3))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 3731)
  (parent 3308)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nb-0)
    (nm acctnum) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum nb-0 (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 acctnum m nb-0 (pubk c)))
      (send (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 acctnum nb-0 (pubk hash-3))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3732)
  (parent 3308)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nb-0)
    (nm acctnum) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
    (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-2))
    (enc "hash" c nb-0 nb-0 acctnum nb-0 (pubk hash-1))
    (enc "hash" c nb-0 goods-0 acctnum nb-0 (pubk hash-3))
    (enc nb-0 acctnum (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 acctnum m nb-0 (pubk c))
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum nb-0 (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 acctnum m nb-0 (pubk c)))
      (send (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 acctnum nb-0 (pubk hash-3))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3733)
  (parent 3309)
  (seen 3732)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nb-0)
    (nm acctnum) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
    (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-2))
    (enc "hash" c nb-0 nb-0 acctnum nb-0 (pubk hash-1))
    (enc "hash" c nb-0 goods-0 acctnum nb-0 (pubk hash-3))
    (enc nb-0 acctnum (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 acctnum m nb-0 (pubk c))
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum nb-0 (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 acctnum m nb-0 (pubk c)))
      (send (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 acctnum nb-0 (pubk hash-3))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3734)
  (parent 3310)
  (seen 3732)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price acctnum acctnum price (pubk hash-0))
    (enc "hash" c price acctnum acctnum price (pubk hash-1))
    (enc "hash" c price goods-0 acctnum price (pubk hash-2))
    (enc price acctnum (pubk c)) (enc price goods-0 (pubk c))
    (enc price acctnum m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price acctnum acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 acctnum price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3735)
  (parent 3311)
  (seen 3729)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (6 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3736)
  (parent 3314)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3737)
  (parent 3316)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3738)
  (parent 3318)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3739)
  (parent 3319)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3740)
  (parent 3320)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (4 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3741)
  (parent 3321)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3742)
  (parent 3325)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3743)
  (parent 3330)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3744)
  (parent 3331)
  (seen 3318)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3745)
  (parent 3332)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 0) (8 0)) ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0))
    ((7 1) (3 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3746)
  (parent 3333)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((8 1) (4 0)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3747)
  (parent 3334)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nm-0 price-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 0) (8 0)) ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0))
    ((6 1) (7 0)) ((7 1) (3 0)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 goods-0 price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3748)
  (parent 3335)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 acctnum-0 nm price-0 nm-0 price-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 2 bank 2) price (4 0)
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 goods-0 price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3749)
  (parent 3335)
  (unrealized (0 0) (0 2) (1 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (0 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 5 merchant 2) nc (0 0)
    (enc nc nm-0 m price-1 (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3750)
  (parent 3341)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 0) (7 0)) ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((5 1) (6 0))
    ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3751)
  (parent 3342)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (0 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 5 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3752)
  (parent 3342)
  (seen 3750)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0))
    ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3753)
  (parent 3342)
  (seen 3750)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (5 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3754)
  (parent 3343)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-2))
            (privk b)) (enc nc price (pubk c))))))
  (label 3755)
  (parent 3344)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((6 1) (7 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3756)
  (parent 3349)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-1)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (7 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0))
    ((6 1) (4 0)) ((7 1) (1 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-1 (pubk c)))))
  (label 3757)
  (parent 3350)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nc nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1
      text) (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm nm-0)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm-0 m price-1 (pubk c)))))
  (label 3758)
  (parent 3350)
  (seen 3265)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (7 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0))
    ((6 1) (4 0)) ((6 1) (7 0)) ((7 1) (1 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-2))
    (enc "hash" c price-0 nb-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 goods-0 (pubk c))
    (enc price-0 nb-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3759)
  (parent 3351)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c))))))
  (label 3760)
  (parent 3353)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3761)
  (parent 3353)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3762)
  (parent 3354)
  (seen 3761)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c))))))
  (label 3763)
  (parent 3356)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3764)
  (parent 3356)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3765)
  (parent 3357)
  (seen 3764)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b)) (enc price nb-1 (pubk c))))))
  (label 3766)
  (parent 3358)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (8 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 1) (1 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3767)
  (parent 3358)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (8 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 1) (1 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) price (1 0)
    (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 nb-0 price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3768)
  (parent 3359)
  (seen 3767)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-1 (0 0)
    (enc c nb-1 goods (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((send (enc c nb-1 goods (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods nb-0 nb-1 (pubk hash-3))
            (privk b)) (enc nb-1 goods (pubk c))))))
  (label 3769)
  (parent 3361)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-1 (0 0)
    (enc c nb-1 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-3))
            (privk b)) (enc nb-1 goods-0 (pubk c)))))
    ((recv (enc c nb-1 goods-0 (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 3770)
  (parent 3361)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-3))
    (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
    (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
    (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
    (enc nb-1 goods-0 (pubk c)) (enc nb-1 nb-0 (pubk c))
    (enc nb-1 nb-1 (pubk c)) (enc nb-1 nb-0 m nb-1 (pubk c))
    (enc nb-1 nm m price (pubk c)) (enc c nb-1 goods-0 (pubk m))
    (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-3))
            (privk b)) (enc nb-1 goods-0 (pubk c)))))
    ((recv (enc c nb-1 goods-0 (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 3771)
  (parent 3362)
  (seen 3770)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 3772)
  (parent 3363)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3773)
  (parent 3363)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (6 0)
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-3)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c))))))
  (label 3774)
  (parent 3364)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-0 m acctnum (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3775)
  (parent 3365)
  (seen 3773)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand bank 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
    (enc acctnum-0 goods-0 (pubk c)) (enc acctnum-0 nb-0 (pubk c))
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c))
    (enc acctnum-0 nm m price (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-3)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c))))))
  (label 3776)
  (parent 3365)
  (seen 4086)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-1
    (1 0) (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-3))
    (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
    (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
    (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
    (enc nb-1 goods-0 (pubk c)) (enc nb-1 nb-0 (pubk c))
    (enc nb-1 nb-1 (pubk c)) (enc nb-1 nb-0 m nb-1 (pubk c))
    (enc nb-1 nm m price (pubk c)) (enc c nb-1 goods-0 (pubk m))
    (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((send (enc c nb-1 goods-0 (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods-0 nb-0 nb-1 (pubk hash-3))
            (privk b)) (enc nb-1 goods-0 (pubk c)))))
    ((recv (enc c nb-1 goods-0 (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 3777)
  (parent 3366)
  (seen 3770)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-0 m price (pubk c)) (enc price nm m price-0 (pubk c))
    (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3778)
  (parent 3367)
  (seen 3764)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test
    (contracted (b-0 b) (acctnum-0 acctnum) (hash-3 hash-2)
      (nb-1 goods-0)) acctnum (1 0)
    (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum goods-0 (pubk c)) (enc acctnum nb-0 (pubk c))
    (enc acctnum nb-0 m acctnum (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3779)
  (parent 3368)
  (seen 3773)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 0) (0 0))
    ((2 0) (6 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand bank 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
    (enc acctnum-0 goods-0 (pubk c)) (enc acctnum-0 nb-0 (pubk c))
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c))
    (enc acctnum-0 nm m price (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-3)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c))))))
  (label 3780)
  (parent 3368)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3781)
  (parent 3372)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3782)
  (parent 3373)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3783)
  (parent 3375)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3784)
  (parent 3376)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c))))))
  (label 3785)
  (parent 3378)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) price (0 0)
    (enc c price goods-0 (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3786)
  (parent 3378)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3787)
  (parent 3379)
  (seen 3786)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
    (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
    (enc price goods-0 (pubk c)) (enc price nb-0 (pubk c))
    (enc price nb-1 (pubk c)) (enc price nb-0 m price (pubk c))
    (enc price nm m price-0 (pubk c)) (enc c price goods-0 (pubk m))
    (enc c price nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods-0 (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods-0 nb-0 price (pubk hash-3))
            (privk b)) (enc price goods-0 (pubk c)))))
    ((recv (enc c price goods-0 (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 3788)
  (parent 3381)
  (seen 3786)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3789)
  (parent 3389)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3790)
  (parent 3390)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3791)
  (parent 3393)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3792)
  (parent 3393)
  (seen 2913)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3793)
  (parent 3394)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3794)
  (parent 3394)
  (seen 2914)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3795)
  (parent 3395)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3796)
  (parent 3395)
  (seen 2883)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3797)
  (parent 3397)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3798)
  (parent 3397)
  (seen 3386)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nm price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (8 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3799)
  (parent 3402)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nc (1 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c))))))
  (label 3800)
  (parent 3403)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3801)
  (parent 3406)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 3802)
  (parent 3406)
  (seen 3789)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 3803)
  (parent 3407)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 price price (pubk hash-2))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 3804)
  (parent 3407)
  (seen 3263)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3805)
  (parent 3411)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (8 0))
    ((5 2) (2 0)) ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3806)
  (parent 3412)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3807)
  (parent 3413)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3808)
  (parent 3413)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3809)
  (parent 3414)
  (seen 3808)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3810)
  (parent 3417)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3811)
  (parent 3417)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3812)
  (parent 3418)
  (seen 3811)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3813)
  (parent 3419)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3814)
  (parent 3419)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3815)
  (parent 3421)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3816)
  (parent 3422)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3817)
  (parent 3423)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3818)
  (parent 3423)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (displaced 8 4 customer 3) price (7 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3819)
  (parent 3424)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3820)
  (parent 3425)
  (seen 3818)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand bank 2) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3821)
  (parent 3425)
  (seen 4108)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3822)
  (parent 3427)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3823)
  (parent 3428)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3824)
  (parent 3430)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3825)
  (parent 3431)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3826)
  (parent 3436)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3827)
  (parent 3436)
  (seen 3738)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3828)
  (parent 3438)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3829)
  (parent 3439)
  (seen 3808)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3830)
  (parent 3442)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3831)
  (parent 3443)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3832)
  (parent 3444)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3833)
  (parent 3444)
  (seen 3815)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3834)
  (parent 3445)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (added-strand bank 2) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3835)
  (parent 3445)
  (unrealized (0 0) (0 2) (1 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3836)
  (parent 3447)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3837)
  (parent 3447)
  (seen 3822)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3838)
  (parent 3449)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (9 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3839)
  (parent 3449)
  (seen 3824)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3840)
  (parent 3455)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3841)
  (parent 3457)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3842)
  (parent 3459)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3843)
  (parent 3459)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3844)
  (parent 3460)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3845)
  (parent 3460)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3846)
  (parent 3463)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 3847)
  (parent 3468)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3848)
  (parent 3471)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3849)
  (parent 3473)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3850)
  (parent 3473)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3851)
  (parent 3474)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3852)
  (parent 3474)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 3853)
  (parent 3476)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3854)
  (parent 3478)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3855)
  (parent 3485)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3856)
  (parent 3486)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3857)
  (parent 3488)
  (seen 3843)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3858)
  (parent 3489)
  (seen 3845)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3859)
  (parent 3491)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3860)
  (parent 3495)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3861)
  (parent 3497)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3862)
  (parent 3499)
  (seen 3850)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3863)
  (parent 3500)
  (seen 3852)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3864)
  (parent 3501)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3865)
  (parent 3502)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3866)
  (parent 3508)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc acctnum nb-1 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 3867)
  (parent 3509)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3868)
  (parent 3511)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3869)
  (parent 3512)
  (seen 3808)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3870)
  (parent 3514)
  (seen 3811)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3871)
  (parent 3515)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3872)
  (parent 3516)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 3873)
  (parent 3517)
  (seen 3818)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (added-strand bank 2) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3874)
  (parent 3517)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3875)
  (parent 3519)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3876)
  (parent 3520)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3877)
  (parent 3523)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3878)
  (parent 3525)
  (seen 3411)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3879)
  (parent 3526)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3880)
  (parent 3527)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nb-0 nm price-0 goods-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 price)) price (7 0)
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3881)
  (parent 3528)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 10 3 bank 2) price (7 0)
    (enc "hash" c nb-1 price price price (pubk hash-4))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3882)
  (parent 3528)
  (seen 3871)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (7 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (added-strand bank 2) price (7 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3883)
  (parent 3530)
  (seen 3882)
  (unrealized (0 0) (0 2) (1 0) (7 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3884)
  (parent 3531)
  (seen 3837)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3885)
  (parent 3533)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3886)
  (parent 3535)
  (seen 3839)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3887)
  (parent 3536)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (7 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3888)
  (parent 3537)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (8 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3889)
  (parent 3539)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3890)
  (parent 3540)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3891)
  (parent 3541)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (8 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3892)
  (parent 3541)
  (seen 3408)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3893)
  (parent 3543)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3894)
  (parent 3544)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3895)
  (parent 3545)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3896)
  (parent 3546)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3897)
  (parent 3546)
  (seen 3415)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nb-0 nm price-0 nm-0 price-1 goods-0 nb-1
      text) (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b-0)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b-0)) (enc nb-0 price (pubk c))))))
  (label 3898)
  (parent 3547)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 11 3 bank 2) price (8 0)
    (enc "hash" c nb-1 price price price (pubk hash-4))
    (enc "hash" c nb-1 nb-1 price price (pubk hash-0))
    (enc "hash" c nb-1 nb-0 price price (pubk hash-3))
    (enc nb-1 price (pubk c)) (enc nb-1 price m price (pubk c))
    (enc c nb-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3899)
  (parent 3547)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c))
    (enc goods-0 nm-0 m price-1 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3900)
  (parent 3548)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3901)
  (parent 3550)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (8 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3902)
  (parent 3550)
  (seen 3899)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3903)
  (parent 3551)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (8 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3904)
  (parent 3553)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3905)
  (parent 3554)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3906)
  (parent 3554)
  (seen 3432)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) nb-1 (0 0)
    (enc nb-1 nm-0 m price-1 (pubk c)) (enc c nb-1 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3907)
  (parent 3556)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3908)
  (parent 3557)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 3909)
  (parent 3558)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 3910)
  (parent 3558)
  (seen 3434)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3911)
  (parent 3562)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (4 0)) ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3912)
  (parent 3563)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (7 0))
    ((5 2) (2 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3913)
  (parent 3563)
  (seen 3888)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3914)
  (parent 3564)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0))
    ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3915)
  (parent 3564)
  (seen 3889)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3916)
  (parent 3564)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3917)
  (parent 3565)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3918)
  (parent 3566)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3919)
  (parent 3567)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3920)
  (parent 3567)
  (seen 3893)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (4 0)) ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3921)
  (parent 3567)
  (seen 3893)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 3922)
  (parent 3568)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm nm-0 price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 10 3 bank 2) price (8 0)
    (enc "hash" c price-1 price price price (pubk hash-2))
    (enc "hash" c price-1 price-1 price price (pubk hash-0))
    (enc "hash" c price-1 nb-0 price price (pubk hash-3))
    (enc price-1 price (pubk c)) (enc price-1 price m price (pubk c))
    (enc c price-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3923)
  (parent 3568)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 nb-0 (pubk c))
    (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 3924)
  (parent 3569)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 11 7 customer 3) price (10 0)
    (enc price-0 price m price (pubk c)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3925)
  (parent 3570)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm nm-0 price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3926)
  (parent 3571)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm nm-0 price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (8 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3927)
  (parent 3571)
  (seen 4172)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm nm-0 price-0 goods-0 acctnum-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 10 7 customer 3) price (8 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 3928)
  (parent 3571)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3929)
  (parent 3572)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 acctnum-0
      nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc price-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (8 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc price-0 nb-0 (pubk c))))))
  (label 3930)
  (parent 3572)
  (seen 3925)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0))
    ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3931)
  (parent 3574)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3932)
  (parent 3575)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 nb-0 (pubk c))
    (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 3933)
  (parent 3576)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 3934)
  (parent 3577)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 3935)
  (parent 3577)
  (seen 3907)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 3936)
  (parent 3577)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-4))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 3937)
  (parent 3578)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 3938)
  (parent 3579)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3939)
  (parent 3583)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3940)
  (parent 3584)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3941)
  (parent 3586)
  (seen 3843)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3942)
  (parent 3587)
  (seen 3845)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 3943)
  (parent 3589)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3944)
  (parent 3594)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3945)
  (parent 3594)
  (seen 2421)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3946)
  (parent 3596)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3947)
  (parent 3596)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 3948)
  (parent 3598)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3949)
  (parent 3599)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3950)
  (parent 3600)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3951)
  (parent 3602)
  (seen 3850)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 3952)
  (parent 3603)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nb-0
    (1 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 3953)
  (parent 3604)
  (seen 3852)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3954)
  (parent 3607)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3955)
  (parent 3607)
  (seen 3847)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 3956)
  (parent 3610)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3957)
  (parent 3610)
  (seen 3853)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) price)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-3))
            (privk b)) (enc nc price (pubk c))))))
  (label 3958)
  (parent 3614)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nm price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc acctnum nb-1 (pubk c))))))
  (label 3959)
  (parent 3617)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 3960)
  (parent 3620)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3961)
  (parent 3621)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-0))
    nb-0 (6 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3962)
  (parent 3622)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3963)
  (parent 3622)
  (unrealized (0 0) (0 2) (1 0) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (5 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3964)
  (parent 3623)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (displaced 9 4 customer 3) nb-1 (8 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3965)
  (parent 3624)
  (seen 4198)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3966)
  (parent 3625)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3967)
  (parent 3625)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3968)
  (parent 3627)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 3969)
  (parent 3627)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (5 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3970)
  (parent 3628)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 3971)
  (parent 3630)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3972)
  (parent 3631)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (3 1)) ((0 1) (7 0)) ((1 1) (0 2)) ((2 1) (4 0))
    ((3 0) (0 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3973)
  (parent 3631)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3974)
  (parent 3632)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3975)
  (parent 3632)
  (unrealized (0 0) (0 2) (1 0) (5 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 4 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3976)
  (parent 3633)
  (seen 4209)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3977)
  (parent 3634)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 3978)
  (parent 3634)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3979)
  (parent 3637)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 3980)
  (parent 3637)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 4 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3981)
  (parent 3638)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (5 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3982)
  (parent 3639)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3983)
  (parent 3641)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-1))
    nb-1 (7 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3984)
  (parent 3642)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 4 bank 2) nb-0 (7 0)
    (enc "hash" c nb-1 nb-1 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nb-1 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nb-1 nb-0 nb-0 nb-0 (pubk hash-4))
    (enc nb-1 nb-0 (pubk c)) (enc nb-1 nb-0 m nb-0 (pubk c))
    (enc c nb-1 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3985)
  (parent 3642)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 4 bank 2) nb-1 (7 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 3986)
  (parent 3642)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (7 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3987)
  (parent 3642)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3988)
  (parent 3643)
  (seen 3967)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 3989)
  (parent 3647)
  (seen 3969)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 5 customer 3) nb-1 (9 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 3990)
  (parent 3649)
  (seen 4219)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 3991)
  (parent 3650)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-0))
    nb-0 (7 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3992)
  (parent 3651)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 4 bank 2) nb-0 (7 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3993)
  (parent 3651)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (7 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3994)
  (parent 3651)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3995)
  (parent 3652)
  (seen 3978)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 8 bank 2) price-0 (9 0)
    (enc goods-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 3996)
  (parent 3653)
  (seen 4225 4226)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 3997)
  (parent 3656)
  (seen 3980)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3998)
  (parent 3656)
  (unrealized (0 0) (0 2) (1 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 3999)
  (parent 3658)
  (seen 4227)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 4000)
  (parent 3659)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4001)
  (parent 3659)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc nc nm m price (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4002)
  (parent 3660)
  (seen 4001)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4003)
  (parent 3662)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-1))
    nb-1 (7 0) (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 4004)
  (parent 3663)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 4 bank 2) nb-0 (7 0)
    (enc "hash" c nb-1 nb-1 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nb-1 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c nb-1 nb-0 nb-0 nb-0 (pubk hash-4))
    (enc nb-1 nb-0 (pubk c)) (enc nb-1 nb-0 m nb-0 (pubk c))
    (enc c nb-1 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4005)
  (parent 3663)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 4 bank 2) nb-1 (7 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 4006)
  (parent 3663)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (added-strand bank 2) nb-1 (7 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 4007)
  (parent 3663)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4008)
  (parent 3664)
  (seen 3967)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4009)
  (parent 3668)
  (seen 3969)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 5 customer 3) nb-1 (9 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 4010)
  (parent 3670)
  (seen 4238)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) price
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4011)
  (parent 3671)
  (seen 3978)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (9 0)) ((9 1) (7 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 8 bank 2) price-0 (9 0)
    (enc goods-0 price-0 m price-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4012)
  (parent 3672)
  (unrealized (0 0) (0 2) (1 0) (7 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (7 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4013)
  (parent 3674)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-0))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 4014)
  (parent 3674)
  (seen 3971)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-0))
    nb-0 (7 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4015)
  (parent 3675)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 4 bank 2) nb-0 (7 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4016)
  (parent 3675)
  (seen 4243 4244)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (7 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4017)
  (parent 3675)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4018)
  (parent 3677)
  (seen 3980)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4019)
  (parent 3677)
  (unrealized (0 0) (0 2) (1 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4020)
  (parent 3679)
  (seen 4247)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nm price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 4021)
  (parent 3681)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nm price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nm m price (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc goods-0 (pubk m)) (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 4022)
  (parent 3682)
  (seen 4001)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum acctnum-0 nm price goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (acctnum-1 acctnum-0)) nb-0 (6 0)
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m nb-0 (pubk c))
    (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc nc nb-1 (pubk c))))))
  (label 4023)
  (parent 3683)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb price acctnum nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (3 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb price nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-2))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4024)
  (parent 3684)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4025)
  (parent 3686)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4026)
  (parent 3686)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4027)
  (parent 3688)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4028)
  (parent 3689)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4029)
  (parent 3690)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4030)
  (parent 3691)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4031)
  (parent 3691)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4032)
  (parent 3694)
  (seen 4026)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4033)
  (parent 3695)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 9 4 customer 3) nb-1 (8 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 4034)
  (parent 3696)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (5 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 4035)
  (parent 3697)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4036)
  (parent 3698)
  (seen 4031)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (8 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4037)
  (parent 3705)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (8 0))
    ((4 0) (0 0)) ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4038)
  (parent 3706)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4039)
  (parent 3708)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4040)
  (parent 3709)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4041)
  (parent 3711)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4042)
  (parent 3711)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nc (1 0)
    (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nc nb-0 (pubk c)) (enc nc goods-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nb-1 m nb-1 (pubk c))
    (enc nc nm m price (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4043)
  (parent 3712)
  (seen 4042)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4044)
  (parent 3715)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4045)
  (parent 3716)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4046)
  (parent 3718)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4047)
  (parent 3719)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4048)
  (parent 3720)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4049)
  (parent 3722)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4050)
  (parent 3723)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 acctnum price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4051)
  (parent 3724)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4052)
  (parent 3726)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4053)
  (parent 3727)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4054)
  (parent 3729)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4055)
  (parent 3730)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nb-0)
    (nm acctnum) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum nb-0 (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m nb-0 (pubk c)))
      (send (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum nb-0 (pubk hash-3))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4056)
  (parent 3732)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nb-0)
    (nm acctnum) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum nb-0 (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m nb-0 (pubk c)))
      (send (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum nb-0 (pubk hash-3))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4057)
  (parent 3733)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum nb-0 goods nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm acctnum) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc nb-0)
    (nm acctnum) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum)
    (nb acctnum) (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm acctnum) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (6 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 acctnum m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb acctnum nb-0 (pubk hash))
            (privk b)) (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 acctnum nb-0 (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 acctnum m nb-0 (pubk c)))
      (send (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-0))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 acctnum acctnum nb-0 (pubk hash-2))
            (privk b)) (enc nb-0 acctnum (pubk c)))))
    ((recv (enc c nb-0 acctnum acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods acctnum nb-0 (pubk hash-3))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4058)
  (parent 3734)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm acctnum) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb acctnum) (nc price)
    (nm acctnum) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum)
    (nb acctnum) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm acctnum) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price acctnum m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb acctnum (pubk hash)) (privk m))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb acctnum price (pubk hash))
            (privk b)) (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price acctnum m price (pubk c)))
      (send (enc c price acctnum acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) acctnum)))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-0))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price acctnum acctnum price (pubk hash-1))
            (privk b)) (enc price acctnum (pubk c)))))
    ((recv (enc c price acctnum acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods acctnum price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4059)
  (parent 3735)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (4 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4060)
  (parent 3741)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4061)
  (parent 3741)
  (seen 3408)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (3 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4062)
  (parent 3744)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4063)
  (parent 3745)
  (seen 4061)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 0) (8 0)) ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0))
    ((7 1) (3 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4064)
  (parent 3746)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 0) (8 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((7 1) (3 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4065)
  (parent 3746)
  (seen 3738)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((8 1) (4 0)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 price price price (pubk hash-3))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods-0 (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4066)
  (parent 3747)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 0) (8 0)) ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0))
    ((6 1) (7 0)) ((7 1) (3 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-0))
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 goods-0 price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 goods-0 (pubk c))
    (enc price-0 price-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4067)
  (parent 3748)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nm price-0 nm-0 price-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (4 0)
    (enc "hash" c price-0 price price price (pubk hash-0))
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 goods-0 price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4068)
  (parent 3749)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (0 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4069)
  (parent 3750)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (0 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4070)
  (parent 3752)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (5 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0))
    ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4071)
  (parent 3753)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (0 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 6 merchant 2) nc (0 0)
    (enc nc nm-0 m price-1 (pubk c)) (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4072)
  (parent 3758)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (7 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0))
    ((6 1) (4 0)) ((6 1) (7 0)) ((7 1) (1 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4073)
  (parent 3759)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (0 0))
    ((7 1) (1 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 6 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4074)
  (parent 3759)
  (seen 4072)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price)
    (nb goods-0) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((6 1) (7 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4075)
  (parent 3759)
  (seen 4072)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4076)
  (parent 3761)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4077)
  (parent 3762)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4078)
  (parent 3764)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4079)
  (parent 3765)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (8 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 1) (1 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4080)
  (parent 3767)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (8 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 1) (1 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 nb-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price nb-0 nb-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-4))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4081)
  (parent 3768)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-1 (0 0)
    (enc nb-1 nm m price (pubk c)) (enc c nb-1 goods (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((send (enc c nb-1 goods (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods nb-0 nb-1 (pubk hash-3))
            (privk b)) (enc nb-1 goods (pubk c)))))
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 4082)
  (parent 3770)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-1 (0 0)
    (enc c nb-1 goods (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((send (enc c nb-1 goods (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods nb-0 nb-1 (pubk hash-3))
            (privk b)) (enc nb-1 goods (pubk c)))))
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 4083)
  (parent 3771)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (3 0))
    ((5 1) (1 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 4084)
  (parent 3773)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-3))
    (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum acctnum (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 (pubk c)) (enc acctnum nb-0 m acctnum (pubk c))
    (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-3))
            (privk b)) (enc acctnum acctnum (pubk c))))))
  (label 4085)
  (parent 3774)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum-0 (1 0)
    (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0 (pubk hash-3))
    (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0 (pubk hash-2))
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
    (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
    (enc acctnum-0 acctnum-0 (pubk c)) (enc acctnum-0 goods-0 (pubk c))
    (enc acctnum-0 nb-0 (pubk c))
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c))
    (enc c acctnum-0 goods-0 (pubk m))
    (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-3)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 4086)
  (parent 3774)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 4087)
  (parent 3775)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-1) (nm nb-0) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nb-1) (nm nb-0)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nb-1) (nm nb-0) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-1) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (6 0)) ((6 1) (1 0)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-1 (0 0)
    (enc c nb-1 goods (pubk m)))
  (traces
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nb-0 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb nb-0 nb-1 (pubk hash)) (privk b))
          (enc nb-1 nb (pubk c)))))
    ((send (enc c nb-1 goods (pubk m)))
      (recv (enc nb-1 nb-0 m nb-1 (pubk c)))
      (send (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-0))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-0 nb-0 nb-1 (pubk hash-1))
            (privk b)) (enc nb-1 nb-0 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 nb-1 nb-0 nb-1 (pubk hash-2))
            (privk b)) (enc nb-1 nb-1 (pubk c)))))
    ((recv (enc c nb-1 nb-0 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-1 goods nb-0 nb-1 (pubk hash-3))
            (privk b)) (enc nb-1 goods (pubk c)))))
    ((recv (enc c nb-1 goods (pubk m)))
      (send (enc nb-1 nm m price (pubk c)))))
  (label 4088)
  (parent 3777)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-2))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4089)
  (parent 3778)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 4090)
  (parent 3779)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum acctnum-0 goods-0 nb-0 nm price text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price acctnum-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price acctnum-0) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb-0) (nc acctnum-0)
    (nm nb-0) (goods goods-0) (price acctnum-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price acctnum-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods-0) (price acctnum-0) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb acctnum-0) (price acctnum-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 8 2 customer 3) nb-0 (7 0)
    (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
  (traces
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 nb-0 acctnum acctnum-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 acctnum-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c)))))
    ((send (enc c acctnum-0 goods-0 (pubk m)))
      (recv (enc acctnum-0 nb-0 m acctnum-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (recv
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c))))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-0))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 nb-0 nb-0 acctnum-0 (pubk hash-1))
            (privk b-0)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 goods-0 nb-0 acctnum-0
              (pubk hash-2)) (privk b-0))
          (enc acctnum-0 goods-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods-0 (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 acctnum-0 (pubk b-0)))
      (send
        (cat
          (enc
            (enc "hash" c acctnum-0 acctnum-0 nb-0 acctnum-0
              (pubk hash-3)) (privk b-0))
          (enc acctnum-0 acctnum-0 (pubk c))))))
  (label 4091)
  (parent 3780)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc price nm m price-0 (pubk c)) (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4092)
  (parent 3786)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4093)
  (parent 3787)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc price) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc price) (nm nb-0) (nb goods)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (3 0)) ((6 1) (1 0))
    ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) price (0 0)
    (enc c price goods (pubk m)))
  (traces
    ((recv (enc c price goods (pubk m)))
      (send (enc price nb-0 m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb nb-0 price (pubk hash)) (privk b))
          (enc price nb (pubk c)))))
    ((send (enc c price goods (pubk m)))
      (recv (enc price nb-0 m price (pubk c)))
      (send (enc c price nb-0 acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-0 nb-0 price (pubk hash-1))
            (privk b)) (enc price nb-0 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price nb-1 nb-0 price (pubk hash-2))
            (privk b)) (enc price nb-1 (pubk c)))))
    ((recv (enc c price nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price goods nb-0 price (pubk hash-3))
            (privk b)) (enc price goods (pubk c)))))
    ((recv (enc c price goods (pubk m)))
      (send (enc price nm m price-0 (pubk c)))))
  (label 4094)
  (parent 3788)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nm price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (8 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4095)
  (parent 3799)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (7 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4096)
  (parent 3799)
  (seen 3789)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 nm price goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4097)
  (parent 3800)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods nb-0 goods-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (6 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4098)
  (parent 3800)
  (seen 3263)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 4099)
  (parent 3808)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (6 0)) ((5 0) (0 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 4100)
  (parent 3809)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4101)
  (parent 3811)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 goods nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (5 0)) ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4102)
  (parent 3812)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-4))
            (privk b)) (enc goods price (pubk c))))))
  (label 4103)
  (parent 3813)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 4104)
  (parent 3813)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 4105)
  (parent 3814)
  (seen 4104)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 4106)
  (parent 3818)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4107)
  (parent 3819)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum goods-0 nb-0 nm price-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 4108)
  (parent 3819)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (7 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 4109)
  (parent 3820)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4110)
  (parent 3829)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0))
    ((6 0) (5 0)) ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4111)
  (parent 3830)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 goods-0 nb-1 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (9 0)) ((5 2) (2 0))
    ((5 2) (4 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4112)
  (parent 3830)
  (seen 4101)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 4113)
  (parent 3831)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 4114)
  (parent 3834)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 4115)
  (parent 3834)
  (seen 4106)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 10 6 customer 3) price (9 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4116)
  (parent 3835)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4117)
  (parent 3843)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4118)
  (parent 3845)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4119)
  (parent 3850)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4120)
  (parent 3852)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4121)
  (parent 3857)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4122)
  (parent 3858)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4123)
  (parent 3862)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4124)
  (parent 3863)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (0 0)) ((6 0) (4 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4125)
  (parent 3869)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4126)
  (parent 3870)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 4127)
  (parent 3871)
  (seen 4104)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 4128)
  (parent 3873)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (displaced 9 5 customer 3) price (8 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4129)
  (parent 3874)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4130)
  (parent 3878)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4131)
  (parent 3879)
  (seen 3808)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4132)
  (parent 3881)
  (seen 4112)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4133)
  (parent 3884)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4134)
  (parent 3886)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (7 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4135)
  (parent 3889)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4136)
  (parent 3890)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4137)
  (parent 3890)
  (seen 3738)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (8 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4138)
  (parent 3892)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4139)
  (parent 3893)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4140)
  (parent 3894)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4141)
  (parent 3894)
  (seen 3805)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c))
    (enc goods-0 nm-0 m price-1 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4142)
  (parent 3895)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) nb-1 (0 0)
    (enc nb-1 nm-0 m price-1 (pubk c)) (enc c nb-1 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4143)
  (parent 3897)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 nm-0 price-1 goods-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) nb-0 (1 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 price price price (pubk hash-4))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
    (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
    (enc nb-0 price (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 goods-0 (pubk c)) (enc nb-0 nb-1 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4144)
  (parent 3898)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 4145)
  (parent 3899)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-1 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4146)
  (parent 3900)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c)))))
  (label 4147)
  (parent 3900)
  (seen 3815)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c))
    (enc goods-0 nm-0 m price-1 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 4148)
  (parent 3901)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (8 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (added-strand bank 2) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c))
    (enc goods-0 nm-0 m price-1 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4149)
  (parent 3901)
  (unrealized (0 0) (0 2) (1 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4150)
  (parent 3903)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4151)
  (parent 3903)
  (seen 3822)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) nb-1 (0 0)
    (enc nb-1 nm-0 m price-1 (pubk c)) (enc c nb-1 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4152)
  (parent 3906)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4153)
  (parent 3907)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4154)
  (parent 3908)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (10 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4155)
  (parent 3908)
  (seen 3824)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4156)
  (parent 3910)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0))
    ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4157)
  (parent 3915)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (7 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4158)
  (parent 3916)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (7 0)) ((5 2) (2 0))
    ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4159)
  (parent 3916)
  (seen 4135)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4160)
  (parent 3917)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4161)
  (parent 3918)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0))
    ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4162)
  (parent 3918)
  (seen 4138)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4163)
  (parent 3918)
  (seen 4138)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 4) (1 0)) ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4164)
  (parent 3920)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (4 0)) ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4165)
  (parent 3921)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 nb-0 (pubk c))
    (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 4166)
  (parent 3922)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm nm-0 price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4167)
  (parent 3923)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 4168)
  (parent 3924)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 4169)
  (parent 3924)
  (seen 4143)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (4 0)) ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4170)
  (parent 3924)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c price-0 price price price (pubk hash-4))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b-0)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b-0)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b-0)) (enc price-0 price (pubk c))))))
  (label 4171)
  (parent 3925)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm nm-0 price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 11 3 bank 2) price (8 0)
    (enc "hash" c price-1 price price price (pubk hash-4))
    (enc "hash" c price-1 price-1 price price (pubk hash-0))
    (enc "hash" c price-1 nb-0 price price (pubk hash-3))
    (enc price-1 price (pubk c)) (enc price-1 price m price (pubk c))
    (enc c price-1 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 4172)
  (parent 3925)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm nm-0 price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m goods-0 (pubk c))
    (enc goods-0 nm-0 m price-0 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4173)
  (parent 3926)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm nm-0 price-0 goods-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 4174)
  (parent 3928)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm nm-0 price-0 goods-0 acctnum-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm price)
    (nb price) (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (8 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (added-strand bank 2) price (8 0)
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price acctnum-0 price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4175)
  (parent 3928)
  (seen 4172)
  (unrealized (0 0) (0 2) (1 0) (8 0) (10 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4176)
  (parent 3929)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 4177)
  (parent 3933)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 4178)
  (parent 3933)
  (seen 4152)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4179)
  (parent 3933)
  (seen 4152)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 4180)
  (parent 3935)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4181)
  (parent 3936)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4182)
  (parent 3936)
  (seen 4153)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price price price (pubk hash-4))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 nb-0 (pubk c))
    (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4183)
  (parent 3937)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4184)
  (parent 3938)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4185)
  (parent 3938)
  (seen 4156)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4186)
  (parent 3938)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4187)
  (parent 3941)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4188)
  (parent 3942)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb goods) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 4189)
  (parent 3947)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4190)
  (parent 3949)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4191)
  (parent 3949)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4192)
  (parent 3951)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4193)
  (parent 3953)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4194)
  (parent 3961)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4195)
  (parent 3961)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4196)
  (parent 3962)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4197)
  (parent 3962)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 4 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4198)
  (parent 3963)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4199)
  (parent 3964)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (5 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 4200)
  (parent 3965)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 4201)
  (parent 3967)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4202)
  (parent 3969)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4203)
  (parent 3970)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4204)
  (parent 3972)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4205)
  (parent 3972)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 8 3 customer 3) nb-0 (7 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4206)
  (parent 3973)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4207)
  (parent 3974)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (5 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4208)
  (parent 3974)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 4 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4209)
  (parent 3975)
  (unrealized (0 0) (0 2) (1 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4210)
  (parent 3976)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (6 0)) ((5 0) (8 0)) ((5 2) (2 0)) ((5 2) (3 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 4211)
  (parent 3978)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4212)
  (parent 3980)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4213)
  (parent 3981)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4214)
  (parent 3982)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4215)
  (parent 3984)
  (seen 4195)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-0))
    nb-0 (7 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4216)
  (parent 3985)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (7 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4217)
  (parent 3985)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (6 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 4218)
  (parent 3986)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 5 customer 3) nb-1 (9 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 4219)
  (parent 3987)
  (seen 4364)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4220)
  (parent 3988)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4221)
  (parent 3989)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (6 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 4222)
  (parent 3990)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4223)
  (parent 3992)
  (seen 4205)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((0 1) (8 0)) ((1 1) (0 2)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0))
    ((4 2) (6 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4224)
  (parent 3992)
  (unrealized (0 0) (0 2) (1 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4225)
  (parent 3993)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4226)
  (parent 3993)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4227)
  (parent 3994)
  (seen 4372)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4228)
  (parent 3995)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4229)
  (parent 3997)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4230)
  (parent 3998)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4231)
  (parent 3999)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4232)
  (parent 4001)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (6 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4233)
  (parent 4002)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4234)
  (parent 4004)
  (seen 4195)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-0))
    nb-0 (7 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4235)
  (parent 4005)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (7 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4236)
  (parent 4005)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (6 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c))))))
  (label 4237)
  (parent 4006)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods acctnum goods-0 nb-0 nm price acctnum-0 nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (displaced 10 5 customer 3) nb-1 (9 0)
    (enc nb-0 nb-1 m nb-1 (pubk c)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 acctnum-0 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 4238)
  (parent 4007)
  (seen 4379)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4239)
  (parent 4008)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4240)
  (parent 4009)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (6 0)
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 4241)
  (parent 4010)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (4 0))
    ((4 1) (7 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (3 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-0))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) goods)))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-2))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4242)
  (parent 4011)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (hash-4 hash-2) (acctnum-0 nb-0))
    nb-0 (7 0) (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4243)
  (parent 4012)
  (unrealized (0 0) (0 2) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (7 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4244)
  (parent 4012)
  (unrealized (0 0) (0 2) (1 0) (6 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4245)
  (parent 4015)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((2 1) (6 0)) ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0))
    ((5 0) (4 0)) ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4246)
  (parent 4015)
  (unrealized (0 0) (0 2) (1 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4247)
  (parent 4017)
  (seen 4385)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4248)
  (parent 4018)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4249)
  (parent 4019)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4250)
  (parent 4020)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nm price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 4251)
  (parent 4022)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb goods acctnum nm price goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods-0) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) nc
    (1 0) (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3))
    (enc nc goods-0 (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc nb-1 (pubk c)) (enc nc nm m price (pubk c))
    (enc nc nb-0 m nb-0 (pubk c)) (enc c nc goods-0 (pubk m))
    (enc c nc nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods-0 (pubk c)))))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 4252)
  (parent 4023)
  (seen 2713)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4253)
  (parent 4026)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4254)
  (parent 4028)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4255)
  (parent 4028)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4256)
  (parent 4029)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4257)
  (parent 4029)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3)) ((4 1) (5 0))
    ((5 1) (3 0)) ((6 1) (3 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4258)
  (parent 4031)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4259)
  (parent 4032)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4260)
  (parent 4034)
  (seen 4255)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4261)
  (parent 4035)
  (seen 4257)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 0) (0 0))
    ((3 0) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4262)
  (parent 4036)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (3 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc nc nm m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4263)
  (parent 4042)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nc nb nb-0 goods nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-1) (nc nc) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nc) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (3 0))
    ((4 2) (6 0)) ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3))
    ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (1 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nc nb nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-1 m nb-1 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-1 nb-1 (pubk hash-1)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-1 m nb-1 (pubk c)))
      (send (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          nb-1)))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-0)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-2)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-1 nb-1 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c)))))
    ((recv (enc c nc nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))))
  (label 4264)
  (parent 4043)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4265)
  (parent 4063)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 0) (8 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (0 0)) ((7 1) (3 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 6 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4266)
  (parent 4065)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((8 1) (4 0)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4267)
  (parent 4066)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((8 1) (4 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-3))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4268)
  (parent 4066)
  (seen 3408)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 0) (8 0)) ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0))
    ((6 1) (7 0)) ((7 1) (3 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4269)
  (parent 4067)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 0) (8 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (0 0)) ((7 1) (3 0))
    ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 9 6 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4270)
  (parent 4067)
  (seen 4266)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 0) (8 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (3 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4271)
  (parent 4067)
  (seen 4266)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-0))
    (enc "hash" c price-0 price price price (pubk hash-3))
    (enc "hash" c price-0 goods-0 price price (pubk hash-2))
    (enc "hash" c price-0 price-0 price price (pubk hash-1))
    (enc price-0 price (pubk c)) (enc price-0 goods-0 (pubk c))
    (enc price-0 price-0 (pubk c)) (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4272)
  (parent 4068)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (0 0))
    ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4273)
  (parent 4072)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (0 0))
    ((7 1) (1 0)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4274)
  (parent 4074)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum acctnum) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (6 0))
    ((3 0) (8 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (4 0)) ((6 1) (4 0))
    ((6 1) (7 0)) ((7 1) (1 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4275)
  (parent 4075)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-3))
            (privk b)) (enc acctnum acctnum (pubk c))))))
  (label 4276)
  (parent 4085)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (added-strand merchant 2) acctnum (0 0)
    (enc c acctnum goods-0 (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-3))
            (privk b)) (enc acctnum acctnum (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 4277)
  (parent 4085)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-3))
    (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum acctnum (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 (pubk c)) (enc acctnum nb-0 m acctnum (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-3))
            (privk b)) (enc acctnum acctnum (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 4278)
  (parent 4086)
  (seen 4277)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods-0) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb goods-0) (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum-0 acctnum)) acctnum
    (1 0) (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-3))
    (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
    (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
    (enc acctnum acctnum (pubk c)) (enc acctnum goods-0 (pubk c))
    (enc acctnum nb-0 (pubk c)) (enc acctnum nb-0 m acctnum (pubk c))
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods-0 (pubk m))
    (enc c acctnum nb-0 acctnum acctnum (pubk b)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods-0 (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods-0 nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods-0 (pubk c)))))
    ((recv (enc c acctnum goods-0 (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-3))
            (privk b)) (enc acctnum acctnum (pubk c))))))
  (label 4279)
  (parent 4091)
  (seen 4277)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-4))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 4280)
  (parent 4104)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-4))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 4281)
  (parent 4105)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-4))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4282)
  (parent 4107)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 4283)
  (parent 4107)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 4284)
  (parent 4108)
  (seen 4283)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-4))
            (privk b)) (enc goods price (pubk c))))))
  (label 4285)
  (parent 4113)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (9 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 4286)
  (parent 4113)
  (seen 4280)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4287)
  (parent 4116)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-4))
            (privk b)) (enc goods price (pubk c))))))
  (label 4288)
  (parent 4127)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c)) (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4289)
  (parent 4129)
  (seen 4283)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (0 0))
    ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4290)
  (parent 4131)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (0 0)) ((6 0) (4 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (7 0)) ((9 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4291)
  (parent 4132)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4292)
  (parent 4137)
  (seen 4397)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (8 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4293)
  (parent 4138)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) nb-1 (0 0)
    (enc nb-1 nm-0 m price-1 (pubk c)) (enc c nb-1 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4294)
  (parent 4141)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-1 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4295)
  (parent 4142)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c)))))
  (label 4296)
  (parent 4142)
  (seen 4099)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4297)
  (parent 4143)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nb-0 nm price-0 nm-0 price-1 goods nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4298)
  (parent 4144)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nb-0 nm price-0 goods-0 nb-1 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc nb-0) (nm price)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (10 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 price price (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-4))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4299)
  (parent 4144)
  (seen 4101)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c))
    (enc goods-0 nm-0 m price-1 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 4300)
  (parent 4145)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods price-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods price-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods price-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) goods-0 (0 0)
    (enc goods-0 nm-0 m price-1 (pubk c))
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 price-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4301)
  (parent 4147)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-1 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 4302)
  (parent 4148)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c)))))
  (label 4303)
  (parent 4148)
  (seen 4106)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nb goods price acctnum nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 11 7 customer 3) price (10 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4304)
  (parent 4149)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4305)
  (parent 4151)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4306)
  (parent 4152)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (10 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 11 4 merchant 2) nb-1 (0 0)
    (enc nb-1 nm-0 m price-1 (pubk c)) (enc c nb-1 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4307)
  (parent 4155)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4308)
  (parent 4156)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4309)
  (parent 4160)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4310)
  (parent 4160)
  (seen 4292)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4311)
  (parent 4160)
  (seen 4266)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 4) (1 0))
    ((8 1) (7 3)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4312)
  (parent 4162)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (8 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4313)
  (parent 4163)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 4314)
  (parent 4166)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 4315)
  (parent 4166)
  (seen 4294)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (4 0)) ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4316)
  (parent 4166)
  (seen 4294)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm nm-0 price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m goods-0 (pubk c))
    (enc goods-0 nm-0 m price-0 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4317)
  (parent 4167)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 4318)
  (parent 4169)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (4 0)) ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4319)
  (parent 4170)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4320)
  (parent 4170)
  (seen 4297)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) price-0
    (1 0) (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 price price price (pubk hash-4))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc "hash" c price-0 goods-0 price price (pubk hash-1))
    (enc "hash" c price-0 nb-0 price price (pubk hash-3))
    (enc price-0 price (pubk c)) (enc price-0 price-0 (pubk c))
    (enc price-0 goods-0 (pubk c)) (enc price-0 nb-0 (pubk c))
    (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4321)
  (parent 4171)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm nm-0 price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (acctnum-0 price)) price (8 0)
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 price m price (pubk c))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b-0)) (enc goods-0 price (pubk c))))))
  (label 4322)
  (parent 4172)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm nm-0 price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4323)
  (parent 4173)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm nm-0 price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 10 4 merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4324)
  (parent 4173)
  (seen 4301)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm goods-0 nb-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c)))))
  (label 4325)
  (parent 4173)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm nm-0 price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m goods-0 (pubk c))
    (enc goods-0 nm-0 m price-0 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 4326)
  (parent 4174)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm nm-0 price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((0 1) (10 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (8 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (added-strand bank 2) goods-0 (1 0)
    (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m goods-0 (pubk c))
    (enc goods-0 nm-0 m price-0 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4327)
  (parent 4174)
  (unrealized (0 0) (0 2) (1 0) (10 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4328)
  (parent 4176)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4329)
  (parent 4176)
  (seen 4305)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (4 0)) ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4330)
  (parent 4176)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 4331)
  (parent 4178)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4332)
  (parent 4179)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4333)
  (parent 4183)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0))
    ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 11 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4334)
  (parent 4183)
  (seen 4307)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (10 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4335)
  (parent 4183)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4336)
  (parent 4185)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4337)
  (parent 4186)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4338)
  (parent 4186)
  (seen 4308)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum acctnum) (nb goods) (nc goods)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (1 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4339)
  (parent 4191)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4340)
  (parent 4195)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4341)
  (parent 4196)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4342)
  (parent 4196)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 4 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4343)
  (parent 4197)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4344)
  (parent 4198)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4345)
  (parent 4199)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4346)
  (parent 4199)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4347)
  (parent 4200)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4348)
  (parent 4203)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4349)
  (parent 4203)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 4) (1 0)) ((4 1) (3 3))
    ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4350)
  (parent 4205)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4351)
  (parent 4206)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4352)
  (parent 4207)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4353)
  (parent 4207)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (displaced 9 4 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4354)
  (parent 4208)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (5 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4355)
  (parent 4209)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4356)
  (parent 4210)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4357)
  (parent 4213)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4358)
  (parent 4213)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4359)
  (parent 4214)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4360)
  (parent 4214)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4361)
  (parent 4215)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4362)
  (parent 4216)
  (seen 4342)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4363)
  (parent 4216)
  (unrealized (0 0) (0 2) (1 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4364)
  (parent 4217)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4365)
  (parent 4218)
  (seen 4346)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (6 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 4366)
  (parent 4219)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4367)
  (parent 4222)
  (seen 4349)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4368)
  (parent 4223)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 9 4 customer 3) nb-0 (8 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4369)
  (parent 4224)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4370)
  (parent 4225)
  (seen 4353)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4371)
  (parent 4225)
  (unrealized (0 0) (0 2) (1 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4372)
  (parent 4226)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4373)
  (parent 4227)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4374)
  (parent 4230)
  (seen 4358)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4375)
  (parent 4231)
  (seen 4360)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4376)
  (parent 4234)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4377)
  (parent 4235)
  (seen 4342)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4378)
  (parent 4235)
  (unrealized (0 0) (0 2) (1 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4379)
  (parent 4236)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4380)
  (parent 4237)
  (seen 4346)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nb-0 nm price nb-1 nb-2 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b-0) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (acctnum-0 nb-1)) nb-1 (6 0)
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 acctnum nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b-0)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b-0)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b-0)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b-0)) (enc nb-0 nb-2 (pubk c))))))
  (label 4381)
  (parent 4238)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4382)
  (parent 4241)
  (seen 4349)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4383)
  (parent 4243)
  (seen 4353)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((0 1) (9 0)) ((1 1) (0 2)) ((2 1) (3 0))
    ((3 1) (6 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4384)
  (parent 4243)
  (unrealized (0 0) (0 2) (1 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price acctnum-0 nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc goods-0) (nm nb-0)
    (nb goods-0) (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 acctnum-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4385)
  (parent 4244)
  (unrealized (0 0) (0 2) (1 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4386)
  (parent 4245)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4387)
  (parent 4245)
  (seen 4350)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4388)
  (parent 4246)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4389)
  (parent 4247)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4390)
  (parent 4249)
  (seen 4358)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4391)
  (parent 4250)
  (seen 4360)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nc nb acctnum nm price goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc nc) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (1 0)) ((3 1) (2 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nc nb nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (0 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m nb-0 (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m nb-0 (pubk c)))
      (send (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          nb-0)))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 nb-0 (pubk hash-2)) (privk b))
          (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 nb-0 (pubk hash-3)) (privk b))
          (enc nc nb-1 (pubk c))))))
  (label 4392)
  (parent 4252)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4393)
  (parent 4255)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4394)
  (parent 4257)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4395)
  (parent 4260)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4396)
  (parent 4261)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 0) (8 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (0 0)) ((7 1) (3 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4397)
  (parent 4266)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((8 1) (4 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 7 merchant 2) nb-0 (0 0)
    (enc nb-0 nm-0 m price-1 (pubk c)) (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4398)
  (parent 4268)
  (seen 4293)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 0) (8 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (0 0)) ((7 1) (3 0))
    ((8 1) (7 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4399)
  (parent 4270)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (6 0)) ((2 0) (8 0))
    ((2 2) (4 0)) ((2 2) (7 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (5 0)) ((5 1) (3 0)) ((6 1) (5 0)) ((6 1) (7 0))
    ((7 1) (3 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4400)
  (parent 4271)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 0) (7 0)) ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0))
    ((3 2) (8 0)) ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0))
    ((6 1) (4 0)) ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4401)
  (parent 4272)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((8 1) (4 0)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 7 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4402)
  (parent 4272)
  (seen 4398)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-2))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4403)
  (parent 4272)
  (seen 4138)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (7 0)) ((2 2) (4 0))
    ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0)) ((3 1) (2 3))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0)) ((7 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc acctnum nm m price (pubk c)) (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-3))
            (privk b)) (enc acctnum acctnum (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 4404)
  (parent 4277)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (7 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (1 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-3))
            (privk b)) (enc acctnum acctnum (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 4405)
  (parent 4278)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price acctnum) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price acctnum) (b b) (c c) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb-0) (nc acctnum)
    (nm nb-0) (goods goods) (price acctnum) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price acctnum) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price acctnum) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0)
    (nb acctnum) (price acctnum) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (2 1)) ((1 1) (0 2)) ((2 0) (0 0)) ((2 0) (6 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (7 0)) ((2 4) (1 0))
    ((3 1) (2 3)) ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) acctnum (0 0)
    (enc c acctnum goods (pubk m)))
  (traces
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m acctnum (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 acctnum (pubk hash))
            (privk b)) (enc acctnum nb (pubk c)))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m acctnum (pubk c)))
      (send (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 acctnum (pubk hash-1))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 acctnum (pubk hash-2))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((recv (enc c acctnum nb-0 acctnum acctnum (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum acctnum nb-0 acctnum (pubk hash-3))
            (privk b)) (enc acctnum acctnum (pubk c))))))
  (label 4406)
  (parent 4279)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (7 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-4))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 4407)
  (parent 4283)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (7 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (5 0))
    ((7 1) (1 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-4))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c)))))
  (label 4408)
  (parent 4284)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (1 0)) ((5 1) (4 0)) ((6 0) (0 0)) ((6 0) (5 0))
    ((6 2) (2 0)) ((6 2) (4 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-4))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4409)
  (parent 4287)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price goods-0 nb-0 nm price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (9 0)) ((5 2) (2 0)) ((5 2) (4 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price goods-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c)))))
  (label 4410)
  (parent 4287)
  (seen 4407)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-4))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4411)
  (parent 4289)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4412)
  (parent 4294)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods price-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods price-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods price-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) goods-0 (0 0)
    (enc goods-0 nm-0 m price-1 (pubk c))
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 price-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4413)
  (parent 4296)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (10 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 11 4 merchant 2) nb-1 (0 0)
    (enc nb-1 nm-0 m price-1 (pubk c)) (enc c nb-1 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4414)
  (parent 4299)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-1 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-4))
            (privk b)) (enc goods price (pubk c))))))
  (label 4415)
  (parent 4300)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (10 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c)))))
  (label 4416)
  (parent 4300)
  (seen 4280)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods price-0)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods price-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods price-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods price-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods price-0)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 price-0 (pubk m)))
  (traces
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 price-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4417)
  (parent 4301)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods price-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods price-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods price-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) goods-0 (0 0)
    (enc goods-0 nm-0 m price-1 (pubk c))
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 price-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4418)
  (parent 4303)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m price-0 (pubk c))
    (enc goods-0 nm-0 m price-1 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4419)
  (parent 4304)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4420)
  (parent 4305)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (9 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4421)
  (parent 4307)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4422)
  (parent 4310)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4423)
  (parent 4311)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c))))))
  (label 4424)
  (parent 4315)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0))
    ((6 0) (4 0)) ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4425)
  (parent 4316)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm nm-0 price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (8 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4426)
  (parent 4317)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm nm-0 price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 10 4 merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4427)
  (parent 4317)
  (seen 4413)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm goods-0 nb-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c)))))
  (label 4428)
  (parent 4317)
  (seen 4413)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 1) (5 0)) ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0))
    ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4429)
  (parent 4321)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nm-0 price-1 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 11 4 merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4430)
  (parent 4321)
  (seen 4414)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (4 0)) ((6 0) (10 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (7 0)) ((9 1) (7 0)) ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (added-strand merchant 2) price-0 (0 0)
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4431)
  (parent 4321)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm nm-0 price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-4))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m goods-0 (pubk c))
    (enc goods-0 nm-0 m price-0 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 4432)
  (parent 4322)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm nm-0 price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m goods (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4433)
  (parent 4324)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm goods nb-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods nm-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm-0 m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c)))))
  (label 4434)
  (parent 4325)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm goods-0 nb-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) goods-0 (0 0)
    (enc goods-0 nm-0 m price-0 (pubk c))
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c)))))
  (label 4435)
  (parent 4325)
  (seen 4417)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm nm-0 price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 4436)
  (parent 4326)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm nm-0 price-0 goods-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 10 4 merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 4437)
  (parent 4326)
  (seen 4418)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm goods-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c)))))
  (label 4438)
  (parent 4326)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price acctnum nm nm-0 price-0 goods-0 nb-0 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b-0) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 11 7 customer 3) price (10 0)
    (enc goods-0 price m price (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) price)))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b-0)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-0 (pubk c))))))
  (label 4439)
  (parent 4327)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4440)
  (parent 4329)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (4 0)) ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4441)
  (parent 4330)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4442)
  (parent 4330)
  (seen 4420)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0))
    ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4443)
  (parent 4334)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (10 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4444)
  (parent 4335)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (9 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 11 4 merchant 2) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4445)
  (parent 4335)
  (seen 4421)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4446)
  (parent 4342)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4447)
  (parent 4343)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4448)
  (parent 4344)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4449)
  (parent 4346)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4450)
  (parent 4347)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc c nb-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4451)
  (parent 4347)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4452)
  (parent 4349)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4453)
  (parent 4351)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4454)
  (parent 4351)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (8 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 4) (1 0))
    ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0)) ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4455)
  (parent 4353)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4456)
  (parent 4354)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4457)
  (parent 4355)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (0 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4458)
  (parent 4356)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4459)
  (parent 4356)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4460)
  (parent 4358)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4461)
  (parent 4360)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4462)
  (parent 4362)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4463)
  (parent 4363)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4464)
  (parent 4364)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4465)
  (parent 4365)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4466)
  (parent 4366)
  (seen 4451)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4467)
  (parent 4367)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4468)
  (parent 4369)
  (seen 4454)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4469)
  (parent 4370)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4470)
  (parent 4371)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4471)
  (parent 4372)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4472)
  (parent 4373)
  (seen 4459)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4473)
  (parent 4374)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4474)
  (parent 4375)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4475)
  (parent 4377)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4476)
  (parent 4378)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4477)
  (parent 4379)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c))))))
  (label 4478)
  (parent 4380)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods-0)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods-0) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods-0) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-1)) nb-1 (1 0)
    (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
    (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
    (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
    (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
    (enc nb-0 nb-1 (pubk c)) (enc nb-0 nb-1 m nb-1 (pubk c))
    (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods-0 nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods-0 (pubk c)))))
    ((recv (enc c nb-0 goods-0 (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods-0 (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4479)
  (parent 4381)
  (seen 4451)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4480)
  (parent 4382)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (8 0))
    ((8 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4481)
  (parent 4383)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (displaced 10 5 customer 3) nb-0 (9 0)
    (enc goods-0 nb-0 m nb-0 (pubk c)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4482)
  (parent 4384)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods acctnum goods-0 nm price nb-0 nb-1 text)
    (b m c hash b-0 hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b-0) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b-0) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b-0) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b-0) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0)
    (privk hash-0) (privk hash-1) (privk hash-2) (privk hash-3)
    (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (acctnum-0 nb-0)) nb-0 (6 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b-0)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b-0)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b-0)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b-0)) (enc goods-0 nb-1 (pubk c))))))
  (label 4483)
  (parent 4385)
  (unrealized (0 0) (0 2) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4484)
  (parent 4388)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4485)
  (parent 4389)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4486)
  (parent 4390)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (1 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4487)
  (parent 4391)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (0 0)) ((8 1) (4 0)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4488)
  (parent 4402)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (7 0))
    ((3 0) (9 0)) ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (8 0))
    ((3 4) (1 0)) ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0))
    ((7 1) (6 0)) ((7 1) (8 0)) ((8 1) (4 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-2))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4489)
  (parent 4403)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods price-0)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods price-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods price-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods price-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 1) (7 0)) ((6 0) (4 0)) ((6 0) (9 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 4) (1 0))
    ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods price-0)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 price-0 (pubk m)))
  (traces
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 price-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4490)
  (parent 4413)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (9 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4491)
  (parent 4414)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods price-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods price-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods price-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (10 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (7 0))
    ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 11 4 merchant 2) goods-0 (0 0)
    (enc goods-0 nm-0 m price-1 (pubk c))
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 price-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4492)
  (parent 4416)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods price-0)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods price-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods price-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods price-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods price-0)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 price-0 (pubk m)))
  (traces
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 price-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4493)
  (parent 4418)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (0 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price-0 (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-1 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-4))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4494)
  (parent 4419)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (10 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (1 0))
    ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-1 (pubk c)))))
  (label 4495)
  (parent 4419)
  (seen 4407)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm nm-0 price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (8 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m goods (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4496)
  (parent 4427)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm goods nb-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (7 0)) ((6 0) (4 0))
    ((6 0) (9 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods nm-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm-0 m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c)))))
  (label 4497)
  (parent 4428)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nm-0 price-1 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (8 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0))
    ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3))
    ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c)) (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c))))))
  (label 4498)
  (parent 4430)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 goods nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (7 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0))
    ((6 0) (4 0)) ((6 0) (10 0)) ((6 2) (2 0)) ((6 2) (5 0))
    ((6 2) (8 0)) ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3))
    ((8 1) (7 0)) ((9 1) (7 0)) ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods-0 goods)) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4499)
  (parent 4431)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 goods-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (9 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 11 4 merchant 2) price-0 (0 0)
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc price-0 goods-0 (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 goods-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4500)
  (parent 4431)
  (seen 4491)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm nm-0 price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-4))
            (privk b)) (enc goods price (pubk c))))))
  (label 4501)
  (parent 4432)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm nm-0 price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0))
    ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 11 4 merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c))))))
  (label 4502)
  (parent 4432)
  (seen 4492)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm goods-0 nb-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (10 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (7 0)) ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c)))))
  (label 4503)
  (parent 4432)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm nm-0 price-0 goods text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m goods (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c))))))
  (label 4504)
  (parent 4437)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm goods nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods nm-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm-0 m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c)))))
  (label 4505)
  (parent 4438)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm goods-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (8 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 4) (1 0)) ((6 1) (5 3)) ((7 1) (6 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (displaced 10 4 merchant 2) goods-0 (0 0)
    (enc goods-0 nm-0 m price-0 (pubk c))
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c)))))
  (label 4506)
  (parent 4438)
  (seen 4493)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm nm-0 price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (contracted (b-0 b) (acctnum price)) goods-0
    (1 0) (enc "hash" c goods-0 price price price (pubk hash-2))
    (enc "hash" c goods-0 price price price (pubk hash-3))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
    (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
    (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
    (enc goods-0 price (pubk c)) (enc goods-0 goods-0 (pubk c))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 price m price (pubk c))
    (enc goods-0 nm m goods-0 (pubk c))
    (enc goods-0 nm-0 m price-0 (pubk c))
    (enc c goods-0 goods-0 (pubk m))
    (enc c goods-0 price price price (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4507)
  (parent 4439)
  (unrealized (0 0) (0 2))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4508)
  (parent 4447)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4509)
  (parent 4447)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4510)
  (parent 4448)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4511)
  (parent 4448)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nb-2 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nb-1 nm)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc nb-0 nm m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c)))))
  (label 4512)
  (parent 4451)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (1 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4513)
  (parent 4454)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4514)
  (parent 4456)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4515)
  (parent 4456)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (0 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4516)
  (parent 4457)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4517)
  (parent 4457)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (3 1)) ((1 1) (0 2)) ((2 1) (4 0)) ((3 0) (8 0))
    ((3 2) (2 0)) ((3 2) (5 0)) ((3 2) (7 0)) ((3 4) (1 0))
    ((4 1) (3 3)) ((5 1) (6 0)) ((6 1) (4 0)) ((7 1) (4 0))
    ((8 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4518)
  (parent 4459)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4519)
  (parent 4463)
  (seen 4509)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4520)
  (parent 4464)
  (seen 4511)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4521)
  (parent 4466)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4522)
  (parent 4468)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4523)
  (parent 4470)
  (seen 4515)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4524)
  (parent 4471)
  (seen 4517)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (5 0)) ((3 1) (1 0))
    ((4 0) (0 0)) ((4 0) (3 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4525)
  (parent 4472)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4526)
  (parent 4476)
  (seen 4509)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4527)
  (parent 4477)
  (seen 4511)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nm price nb-1 nb-2 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm nb-1) (goods goods)
    (price nb-1) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb)
    (price nb-1) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb goods)
    (price nb-1) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-1) (nb nb-0) (nc nb-0) (nm nb-1)
    (goods goods) (price nb-1) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-0)
    (price nb-1) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-1)
    (price nb-1) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-1) (nc nb-0) (nm nb-1) (nb nb-2)
    (price nb-1) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb nb-0 nm nb-1)
  (operation nonce-test (contracted (goods-0 goods)) nb-0 (0 0)
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nb-1 m nb-1 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-1 (pubk hash)) (privk m))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb nb-1 nb-1 (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods nb-1 nb-1 (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price (pubk c))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 nb-1 m nb-1 (pubk c)))
      (send (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) nb-0)))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 nb-1 nb-1 (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-2))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-1 nb-1 nb-1 (pubk hash-3))
            (privk b)) (enc nb-0 nb-1 (pubk c)))))
    ((recv (enc c nb-0 nb-1 nb-1 nb-1 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-2 nb-1 nb-1 (pubk hash-4))
            (privk b)) (enc nb-0 nb-2 (pubk c))))))
  (label 4528)
  (parent 4479)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4529)
  (parent 4482)
  (seen 4515)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nm nb-0)
  (operation nonce-test (contracted (b-0 b) (acctnum nb-0)) nb-0 (1 0)
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
    (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
    (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
    (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
    (enc goods-0 nb-0 (pubk c)) (enc goods-0 nb-0 m nb-0 (pubk c))
    (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c))))))
  (label 4530)
  (parent 4483)
  (seen 4517)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4531)
  (parent 4484)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4532)
  (parent 4484)
  (seen 4513)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (6 0))
    ((3 1) (1 0)) ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0))
    ((5 2) (2 0)) ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0))
    ((6 1) (5 3)) ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4533)
  (parent 4485)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods goods-0 nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods-0) (nc goods-0)
    (nm nb-0) (goods goods-0) (price nb-0) (b b) (c c) (m m)
    (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb goods-0)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods-0) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price price) (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((2 1) (5 0))
    ((3 1) (1 0)) ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0))
    ((4 2) (8 0)) ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0))
    ((7 1) (5 0)) ((8 1) (5 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods-0 nb-0 nm)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 nb-0 m nb-0 (pubk c)))
      (send (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) goods-0)))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods-0 nb-1 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m price (pubk c)))))
  (label 4534)
  (parent 4485)
  (seen 4518)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods price-0)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods price-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods price-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods price-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (9 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods price-0)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 price-0 (pubk m)))
  (traces
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 price-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-4))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4535)
  (parent 4492)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm price-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods price-0)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods price-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods price-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 0) (4 0)) ((6 0) (10 0))
    ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0)) ((6 2) (9 0))
    ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0)) ((9 1) (1 0))
    ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 11 4 merchant 2) goods-0 (0 0)
    (enc goods-0 nm-0 m price-1 (pubk c))
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((send (enc c price-0 price-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4536)
  (parent 4495)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm nm-0 price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0))
    ((10 1) (8 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m goods (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-4))
            (privk b)) (enc goods price (pubk c))))))
  (label 4537)
  (parent 4502)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm goods nb-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (9 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods nm-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm-0 m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-4))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c)))))
  (label 4538)
  (parent 4503)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm goods-0 nb-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (9 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (6 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (displaced 11 4 merchant 2) goods-0 (0 0)
    (enc goods-0 nm-0 m price-0 (pubk c))
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-3))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-4))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c)))))
  (label 4539)
  (parent 4503)
  (seen 4535)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price nm nm-0 price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 1) (5 0))
    ((7 0) (0 0)) ((7 0) (4 0)) ((7 0) (6 0)) ((7 2) (2 0))
    ((7 2) (5 0)) ((7 2) (9 0)) ((7 2) (10 0)) ((7 4) (1 0))
    ((8 1) (7 3)) ((9 1) (8 0)) ((10 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-4))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4540)
  (parent 4507)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm nm-0 price-0 goods-0 nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0))
    ((10 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods-0)
  (operation nonce-test (displaced 11 4 merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c))))))
  (label 4541)
  (parent 4507)
  (seen 4536)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm goods-0 nb-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (6 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (7 0))
    ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (1 0)) ((6 0) (4 0))
    ((6 0) (10 0)) ((6 2) (2 0)) ((6 2) (5 0)) ((6 2) (8 0))
    ((6 2) (9 0)) ((6 4) (1 0)) ((7 1) (6 3)) ((8 1) (7 0))
    ((9 1) (1 0)) ((10 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (added-strand merchant 2) goods-0 (0 0)
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c)))))
  (label 4542)
  (parent 4507)
  (unrealized (0 0) (0 2))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4543)
  (parent 4509)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4544)
  (parent 4511)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4545)
  (parent 4515)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nb-0 nb-1 nm price text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (4 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (5 0))
    ((4 0) (9 0)) ((4 2) (2 0)) ((4 2) (6 0)) ((4 2) (8 0))
    ((4 4) (1 0)) ((5 1) (4 3)) ((6 1) (7 0)) ((7 1) (5 0))
    ((8 1) (5 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nb-0 nm)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m price (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c)))))
  (label 4546)
  (parent 4517)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4547)
  (parent 4519)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4548)
  (parent 4520)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4549)
  (parent 4523)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4550)
  (parent 4524)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4551)
  (parent 4526)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4552)
  (parent 4527)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4553)
  (parent 4529)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods nm price nb-0 nb-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand customer 5 (acctnum nb-0) (nb goods) (nc goods) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum nb-0) (nc goods) (nm nb-0) (nb nb-1)
    (price nb-0) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (0 0)) ((5 0) (4 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (9 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (8 0)) ((8 1) (6 0)) ((9 1) (6 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb goods nm nb-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb nb-0 nb-0 (pubk hash)) (privk b))
          (enc goods nb (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m price (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods nb-0 m nb-0 (pubk c)))
      (send (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) goods)))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-2))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 nb-0 nb-0 (pubk hash-3))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods nb-0 nb-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-1 nb-0 nb-0 (pubk hash-4))
            (privk b)) (enc goods nb-1 (pubk c))))))
  (label 4554)
  (parent 4530)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm price-0 nb-0 nm-0 price-1 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods price-0)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods price-0)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc price-0)
    (nm price) (goods price-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods price-0)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (9 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (goods price-0)) price-0 (0 0)
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 price-0 (pubk m)))
  (traces
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-1))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((send (enc c price-0 price-0 (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-3))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc price-0 nb-0 (pubk c)))))
    ((recv (enc c price-0 price-0 (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4555)
  (parent 4536)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm nm-0 price-0 goods nb-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (precedes ((0 1) (7 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (8 0))
    ((4 1) (0 0)) ((5 1) (1 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 0) (6 0)) ((7 2) (2 0)) ((7 2) (5 0)) ((7 2) (9 0))
    ((7 2) (10 0)) ((7 4) (1 0)) ((8 1) (7 3)) ((9 1) (8 0))
    ((10 1) (1 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm nm-0 goods)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm m goods (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-4))
            (privk b)) (enc goods nb-0 (pubk c))))))
  (label 4556)
  (parent 4541)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb price nm goods nb-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods) (nm nm) (goods goods) (price goods)
    (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods) (nm nm-0) (goods goods)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (3 0)) ((5 0) (4 0)) ((5 0) (9 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods nm-0)
  (operation nonce-test (contracted (goods-0 goods)) goods (0 0)
    (enc goods nm-0 m price-0 (pubk c)) (enc c goods goods (pubk m)))
  (traces
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb price price (pubk hash))
            (privk b)) (enc goods nb (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-0))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods goods price price (pubk hash-1))
            (privk b)) (enc goods goods (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm m goods (pubk c))))
    ((send (enc c goods goods (pubk m)))
      (recv (enc goods price m price (pubk c)))
      (send (enc c goods price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-2))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods price price price (pubk hash-3))
            (privk b)) (enc goods price (pubk c)))))
    ((recv (enc c goods price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods nb-0 price price (pubk hash-4))
            (privk b)) (enc goods nb-0 (pubk c)))))
    ((recv (enc c goods goods (pubk m)))
      (send (enc goods nm-0 m price-0 (pubk c)))))
  (label 4557)
  (parent 4542)
  (unrealized (0 2))
  (comment "empty cohort"))

(defskeleton epmo_acctnum
  (vars (nb goods price nm goods-0 nb-0 nm-0 price-0 text)
    (m c hash b hash-0 hash-1 hash-2 hash-3 hash-4 name))
  (defstrand merchant 4 (nb nb) (nc goods-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb)
    (price price) (b b) (c c) (hash hash))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb goods-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc goods-0) (nm nm) (goods goods-0)
    (price goods-0) (c c) (m m))
  (defstrand customer 5 (acctnum price) (nb price) (nc goods-0)
    (nm price) (goods goods-0) (price price) (b b) (c c) (m m)
    (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-3))
  (defstrand bank 2 (acctnum price) (nc goods-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-4))
  (defstrand merchant 2 (nc goods-0) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (5 1)) ((1 1) (0 2)) ((2 1) (3 0)) ((3 1) (6 0))
    ((4 1) (0 0)) ((5 0) (4 0)) ((5 0) (9 0)) ((5 2) (2 0))
    ((5 2) (7 0)) ((5 2) (8 0)) ((5 4) (1 0)) ((6 1) (5 3))
    ((7 1) (6 0)) ((8 1) (1 0)) ((9 1) (0 0)))
  (non-orig (privk m) (privk c) (privk hash) (privk b) (privk hash-0)
    (privk hash-1) (privk hash-2) (privk hash-3) (privk hash-4))
  (uniq-orig nb price nm goods-0 nm-0)
  (operation nonce-test (displaced 11 4 merchant 2) goods-0 (0 0)
    (enc goods-0 nm-0 m price-0 (pubk c))
    (enc c goods-0 goods-0 (pubk m)))
  (traces
    ((recv (enc c goods-0 goods (pubk m)))
      (send (enc goods-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb price price (pubk hash))
            (privk b)) (enc goods-0 nb (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-0))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 goods-0 price price (pubk hash-1))
            (privk b)) (enc goods-0 goods-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm m goods-0 (pubk c))))
    ((send (enc c goods-0 goods-0 (pubk m)))
      (recv (enc goods-0 price m price (pubk c)))
      (send (enc c goods-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c))))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) price)))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-2))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 price price price (pubk hash-3))
            (privk b)) (enc goods-0 price (pubk c)))))
    ((recv (enc c goods-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c goods-0 nb-0 price price (pubk hash-4))
            (privk b)) (enc goods-0 nb-0 (pubk c)))))
    ((recv (enc c goods-0 goods-0 (pubk m)))
      (send (enc goods-0 nm-0 m price-0 (pubk c)))))
  (label 4558)
  (parent 4542)
  (seen 4555)
  (unrealized (0 0) (0 2))
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")

(defprotocol epmo_acctnum basic
  (defrole bank
    (vars (b c m name) (acctnum text) (hash name) (nc nm nb price text))
    (trace (recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    (non-orig (privk hash))
    (annotations b
      (1
        (forall ((pm name))
          (implies
            (and (authtransfer c acctnum b price pm nm)
              (reqtransfer pm b price pm nm))
            (dotransfer acctnum b price pm nm))))
      (2
        (and (says c (authtransfer c acctnum b price m nm))
          (says m (reqtransfer m b price m nm))))))
  (defrole customer
    (vars (b c m hash name) (acctnum nb nc nm goods price text))
    (trace (send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    (non-orig (privk b) (privk hash))
    (uniq-orig nc)
    (annotations c
      (1
        (says m
          (implies
            (exists ((acctnum2 text))
              (dotransfer acctnum2 b price m nm)) (doship m goods c))))
      (3
        (says b
          (forall ((pm name))
            (implies
              (and (authtransfer c acctnum b price m nm)
                (reqtransfer pm b price pm nm))
              (dotransfer acctnum b price pm nm)))))
      (4 (authtransfer c acctnum b price m nm))))
  (defrole merchant
    (vars (b c m hash name) (nb nc nm goods price text))
    (trace (recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    (non-orig (privk hash))
    (uniq-orig nm)
    (annotations m
      (1
        (implies
          (exists ((acctnum2 text)) (dotransfer acctnum2 b price m nm))
          (doship m goods c)))
      (2
        (and
          (says b
            (forall ((pm name))
              (exists ((acctnum2 text))
                (implies
                  (and (authtransfer c acctnum2 b price m nm)
                    (reqtransfer pm b price pm nm))
                  (dotransfer acctnum2 b price pm nm)))))
          (says c
            (exists ((acctnum2 text))
              (authtransfer c acctnum2 b price m nm)))))
      (3 (and (reqtransfer m b price m nm) (doship m goods c))))))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price text) (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nb nc)
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m)))))
  (label 4559)
  (unrealized (0 2))
  (origs (nb (0 1)))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price nc-0 goods price-0 text)
    (b m c hash c-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc-0) (nm nm) (goods goods)
    (price price-0) (b b) (c c-0) (m m) (hash hash))
  (precedes ((0 1) (1 2)) ((1 1) (0 0)) ((1 3) (0 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nb nc)
  (operation encryption-test (added-strand merchant 4)
    (enc (enc "hash" b m nb nm (pubk hash)) (privk m)) (0 2))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c-0 nc-0 goods (pubk m)))
      (send (enc nc-0 nm m price-0 (pubk c-0)))
      (recv
        (cat
          (enc (enc "hash" c-0 nc-0 nb nm price-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m)))))
  (label 4560)
  (parent 4559)
  (unrealized (1 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price goods text) (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (1 2)) ((1 1) (0 0)) ((1 3) (0 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nb nc)
  (operation encryption-test (displaced 2 0 bank 2)
    (enc (enc "hash" c-0 nc-0 nb nm price-0 (pubk hash)) (privk b))
    (1 2))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m)))))
  (label 4561)
  (parent 4560)
  (unrealized (0 0) (1 2))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price goods acctnum-0 goods-0 text)
    (b m c hash b-0 m-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods-0) (price price) (b b-0) (c c) (m m-0) (hash hash))
  (precedes ((0 1) (2 3)) ((1 1) (0 0)) ((1 1) (2 1)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 4) (1 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0))
  (uniq-orig nm nb nc)
  (operation nonce-test (added-strand customer 5) nb (1 2)
    (enc "hash" c nc nb nm price (pubk hash)) (enc nc nb (pubk c)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods-0 (pubk m-0)))
      (recv (enc nc nm m-0 price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b-0)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b-0))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b-0))
          nb))))
  (label 4562)
  (parent 4561)
  (unrealized (0 0) (2 1) (2 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price goods acctnum-0 goods-0 text)
    (b m c hash b-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods-0) (price price) (b b-0) (c c) (m m) (hash hash))
  (precedes ((0 1) (2 3)) ((1 1) (0 0)) ((1 1) (2 1)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 4) (1 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk b-0))
  (uniq-orig nm nb nc)
  (operation nonce-test (contracted (m-0 m)) nm (2 1)
    (enc nc nm m price (pubk c)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b-0)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b-0))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b-0))
          nb))))
  (label 4563)
  (parent 4562)
  (unrealized (0 0) (1 0) (2 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price goods acctnum-0 goods-0 text)
    (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (2 3)) ((1 1) (0 0)) ((1 1) (2 1)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 4) (1 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nb nc)
  (operation encryption-test (displaced 3 0 bank 2)
    (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b-0)) (2 3))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))))
  (label 4564)
  (parent 4563)
  (unrealized (0 0) (1 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price acctnum-0 goods text) (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (2 3)) ((1 1) (0 0)) ((1 1) (2 1)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 4) (1 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nb nc)
  (operation nonce-test (contracted (goods-0 goods)) nc (1 0)
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))))
  (label 4565)
  (parent 4564)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars
    (nm nb nc acctnum price goods acctnum-0 goods-0 nm-0 price-0 text)
    (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods-0) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods-0)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (0 0)) ((1 1) (2 1)) ((1 3) (0 2))
    ((2 0) (3 0)) ((2 4) (1 2)) ((3 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nb nc nm-0)
  (operation nonce-test (added-strand merchant 2) nc (1 0)
    (enc c nc goods-0 (pubk m)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods-0 (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods-0 (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c)))))
  (label 4566)
  (parent 4564)
  (unrealized (0 0) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price acctnum-0 goods text) (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (0 0)) ((2 4) (1 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nb nc)
  (operation nonce-test (displaced 3 2 customer 3) nm (0 0)
    (enc nc nm m price (pubk c)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))))
  (label 4567)
  (parent 4565)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price acctnum-0 goods nm-0 price-0 text)
    (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (0 0)) ((1 1) (2 1)) ((1 3) (0 2))
    ((2 0) (3 0)) ((2 4) (1 2)) ((3 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nb nc nm-0)
  (operation nonce-test (contracted (goods-0 goods)) nc (1 0)
    (enc nc nm-0 m price-0 (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c)))))
  (label 4568)
  (parent 4566)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc price acctnum goods text) (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (0 0)) ((2 4) (1 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nb nc)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nm (0 0)
    (enc nc nm m price (pubk c)) (enc c nc nm acctnum price (pubk b)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))))
  (label 4569)
  (parent 4567)
  (realized)
  (shape)
  (maps
    ((0)
      ((b b) (m m) (c c) (nm nm) (nb nb) (nc nc) (hash hash)
        (acctnum acctnum) (price price))))
  (origs (nc (2 0)) (nm (1 1)) (nb (0 1))))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price acctnum-0 goods nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (3 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 2) (0 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nm nb nc)
  (operation nonce-test (added-strand bank 2) nm (0 0)
    (enc nc nm m price (pubk c)) (enc c nc nm acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc nm acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 4570)
  (parent 4567)
  (unrealized (0 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price acctnum-0 goods nm-0 price-0 text)
    (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (0 0)) ((2 4) (1 2)) ((3 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nb nc nm-0)
  (operation nonce-test (displaced 4 2 customer 3) nm (0 0)
    (enc nc nm m price (pubk c)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c)))))
  (label 4571)
  (parent 4568)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price acctnum-0 goods nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nm nb nc)
  (operation nonce-test (displaced 4 2 customer 3) nm (3 0)
    (enc nc nm m price (pubk c)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc nm acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 4572)
  (parent 4570)
  (unrealized (0 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc price acctnum goods nm-0 price-0 text)
    (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (0 0)) ((2 4) (1 2)) ((3 1) (1 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nm nb nc nm-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nm (0 0)
    (enc nc nm m price (pubk c)) (enc c nc nm acctnum price (pubk b)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c)))))
  (label 4573)
  (parent 4571)
  (seen 4569)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price acctnum-0 goods nm-0 price-0 nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (4 0)) ((1 3) (0 2))
    ((2 0) (3 0)) ((2 2) (0 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nm nb nc nm-0)
  (operation nonce-test (added-strand bank 2) nm (0 0)
    (enc nc nm m price (pubk c)) (enc c nc nm acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c))))
    ((recv (enc c nc nm acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 4574)
  (parent 4571)
  (unrealized (0 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc price acctnum goods nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nm nb nc)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nm (0 0)
    (enc "hash" c nc nb-0 nm price (pubk hash-0))
    (enc nc nm m price (pubk c)) (enc c nc nm acctnum price (pubk b)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 4575)
  (parent 4572)
  (seen 4569)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 nb-1 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (4 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (added-strand bank 2) price (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 4576)
  (parent 4572)
  (unrealized (0 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc nb-0)
  (operation nonce-test (displaced 4 3 bank 2) nm (0 0)
    (enc "hash" c nc nb-0 nm price (pubk hash-0))
    (enc nc nm m price (pubk c)) (enc c nc nm acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4577)
  (parent 4572)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc acctnum price acctnum-0 goods nm-0 price-0 nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nm nb nc nm-0)
  (operation nonce-test (displaced 5 2 customer 3) nm (4 0)
    (enc nc nm m price (pubk c)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c))))
    ((recv (enc c nc nm acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 4578)
  (parent 4574)
  (unrealized (0 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price)
  (operation nonce-test (displaced 5 1 merchant 2) price (4 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4579)
  (parent 4576)
  (unrealized (0 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nb-0 text) (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc nb-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4580)
  (parent 4577)
  (seen 4569)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 nb-1 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (4 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (0 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 4581)
  (parent 4577)
  (unrealized (0 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nm nb nc price acctnum goods nm-0 price-0 nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nm) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nm) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nm)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm-0) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nm) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nm nb nc nm-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nm (0 0)
    (enc "hash" c nc nb-0 nm price (pubk hash-0))
    (enc nc nm m price (pubk c)) (enc c nc nm acctnum price (pubk b)))
  (traces
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nm (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nm m price (pubk c)))
      (send (enc c nc nm acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nm price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm-0 m price-0 (pubk c))))
    ((recv (enc c nc nm acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nm price (pubk hash-0)) (privk b))
          (enc nc nb-0 (pubk c))))))
  (label 4582)
  (parent 4578)
  (seen 4575)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 nb-0 nb-1 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand bank 2) price (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 4583)
  (parent 4578)
  (unrealized (0 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc nm nb-0)
  (operation nonce-test (displaced 5 4 bank 2) nm-0 (0 0)
    (enc "hash" c nc nb-0 nm-0 price (pubk hash-0))
    (enc nc nm-0 m price (pubk c))
    (enc c nc nm-0 acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4584)
  (parent 4578)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nb-0 text) (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nc (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4585)
  (parent 4579)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (4 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (added-strand bank 2) nc (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods (pubk m)) (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4586)
  (parent 4579)
  (unrealized (0 0) (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (4 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (added-strand bank 2) nc (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods (pubk m)) (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4587)
  (parent 4579)
  (unrealized (0 0) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc c nc goods (pubk m)) (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4588)
  (parent 4579)
  (unrealized (0 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 nb-1 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc nb-0)
  (operation nonce-test (displaced 5 2 customer 3) nb-0 (4 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 4589)
  (parent 4581)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 6 1 merchant 2) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4590)
  (parent 4583)
  (unrealized (0 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nm price-0 nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc nm nb-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4591)
  (parent 4584)
  (seen 4580)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 nb-0 nb-1 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (0 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 4592)
  (parent 4584)
  (unrealized (0 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods text) (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (0 0)) ((2 4) (1 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nb nc price)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 4593)
  (parent 4585)
  (seen 4569)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (displaced 5 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4594)
  (parent 4586)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (4 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (5 0)) ((2 2) (3 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand merchant 2) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4595)
  (parent 4586)
  (unrealized (0 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (4 0)) ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 5 3 bank 2) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 4596)
  (parent 4587)
  (unrealized (0 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price)
  (operation nonce-test (displaced 5 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4597)
  (parent 4587)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods) (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 5 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods price price (pubk hash-1))
            (privk b)) (enc acctnum-0 goods (pubk c))))))
  (label 4598)
  (parent 4587)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (4 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (5 0)) ((2 2) (3 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand merchant 2) nc (4 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4599)
  (parent 4587)
  (unrealized (0 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nb-0 nm price-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nc (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4600)
  (parent 4588)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (4 0)) ((2 2) (3 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand bank 2) nc (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4601)
  (parent 4588)
  (seen 4595)
  (unrealized (0 0) (5 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (4 0)) ((2 2) (3 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand bank 2) nc (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m))
    (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4602)
  (parent 4588)
  (seen 4599 4613)
  (unrealized (0 0) (5 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nb-0 nb-1 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc nb-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 4603)
  (parent 4589)
  (seen 4575)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 nb-1 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nc price)
  (operation nonce-test (added-strand bank 2) price (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 4604)
  (parent 4589)
  (unrealized (0 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nm price-0 nb-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nc (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4605)
  (parent 4590)
  (seen 4585)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand bank 2) nc (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4606)
  (parent 4590)
  (unrealized (0 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand bank 2) nc (0 0)
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc nb-0 (pubk c)) (enc nc price m price (pubk c))
    (enc nc nm m price-0 (pubk c)) (enc c nc goods (pubk m))
    (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4607)
  (parent 4590)
  (unrealized (0 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 nb-0 nb-1 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc nm nb-0)
  (operation nonce-test (displaced 6 2 customer 3) nb-0 (5 0)
    (enc nc nb-0 m price (pubk c)))
  (traces
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum-0 price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 4608)
  (parent 4592)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods nb-0 text) (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4609)
  (parent 4594)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4610)
  (parent 4595)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (4 0)) ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 4611)
  (parent 4596)
  (seen 4614)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (0 0)
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods (pubk m))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4612)
  (parent 4596)
  (unrealized (0 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4613)
  (parent 4596)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods text) (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price)
  (operation nonce-test (contracted (acctnum-0 acctnum)) price (0 0)
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4614)
  (parent 4597)
  (seen 4593)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (4 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (added-strand bank 2) price (0 0)
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4615)
  (parent 4597)
  (unrealized (0 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods text) (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb price acctnum)
  (operation nonce-test
    (contracted (acctnum-0 acctnum) (hash-1 hash-0) (nb-0 goods)) price
    (0 0) (enc "hash" c acctnum goods price price (pubk hash-0))
    (enc acctnum price m price (pubk c))
    (enc c acctnum price acctnum price (pubk b)))
  (traces
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 4616)
  (parent 4598)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum acctnum-0 goods nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods) (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0)
  (operation nonce-test (displaced 5 3 bank 2) price (0 0)
    (enc "hash" c acctnum-0 goods price price (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 price price (pubk hash-0))
    (enc acctnum-0 price m price (pubk c))
    (enc c acctnum-0 price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum-0 goods (pubk c))))))
  (label 4617)
  (parent 4598)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 6 3 bank 2) nc (4 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4618)
  (parent 4599)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4619)
  (parent 4599)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (4 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods price price (pubk hash-1))
            (privk b)) (enc acctnum-0 goods (pubk c)))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c)))))
  (label 4620)
  (parent 4599)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nm price-0 text) (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (3 0)) ((2 2) (0 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nb nc price nm)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4621)
  (parent 4600)
  (seen 4593)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4622)
  (parent 4601)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4623)
  (parent 4602)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods) (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods price price (pubk hash-1))
            (privk b)) (enc acctnum-0 goods (pubk c))))))
  (label 4624)
  (parent 4602)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (displaced 6 1 merchant 2) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4625)
  (parent 4604)
  (unrealized (0 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4626)
  (parent 4606)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0)) ((4 1) (5 0))
    ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nm nb-0)
  (operation nonce-test (displaced 6 4 bank 2) nc (5 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 4627)
  (parent 4607)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4628)
  (parent 4607)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods) (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods price price (pubk hash-1))
            (privk b)) (enc acctnum-0 goods (pubk c))))))
  (label 4629)
  (parent 4607)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nm price-0 nb-0 nb-1 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm nb-0) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm nb-0)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm nb-0) (nb nb-1)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc nm nb-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
    (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
    (enc nc nb-0 (pubk c)) (enc nc nb-0 m price (pubk c))
    (enc c nc nb-0 acctnum price (pubk b)))
  (traces
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nb-0 m price (pubk c)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc nb-0 m price (pubk c)))
      (send (enc c nc nb-0 acctnum price (pubk b)))
      (recv
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat (enc (enc "hash" c nc nb nb-0 price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 nb-0 price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc nb-0 acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 nb-0 price (pubk hash-1))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 4630)
  (parent 4608)
  (seen 4603)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 nb-0 nb-1 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-1)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (6 0)) ((1 3) (0 2))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand bank 2) price (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-1 price price (pubk hash-2))
            (privk b)) (enc nc nb-1 (pubk c))))))
  (label 4631)
  (parent 4608)
  (unrealized (0 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods text) (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))))
  (label 4632)
  (parent 4609)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4633)
  (parent 4610)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 2 customer 3) nb-0 (5 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4634)
  (parent 4612)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (6 0)) ((2 2) (3 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (0 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (5 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4635)
  (parent 4612)
  (unrealized (0 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4636)
  (parent 4613)
  (seen 4648)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (6 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (5 0)) ((2 2) (3 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (0 0)
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods (pubk m)) (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4637)
  (parent 4613)
  (seen 4635)
  (unrealized (0 0) (6 0))
  (comment "2 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (displaced 5 2 customer 3) price (4 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4638)
  (parent 4615)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods text) (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (0 0)) ((2 4) (1 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nb price acctnum)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))))
  (label 4639)
  (parent 4616)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 text) (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc acctnum nb-0 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 4640)
  (parent 4617)
  (seen 4616)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4641)
  (parent 4618)
  (seen 4643)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (6 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (5 0)) ((2 2) (3 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand bank 2) nb-0 (0 0)
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods (pubk m)) (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4642)
  (parent 4618)
  (unrealized (0 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nm price-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) price (0 0)
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4643)
  (parent 4619)
  (seen 4621)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (4 0)) ((2 2) (3 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand bank 2) price (0 0)
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4644)
  (parent 4619)
  (unrealized (0 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test
    (contracted (acctnum-0 acctnum) (hash-1 hash-0) (nb-0 goods)) price
    (0 0) (enc "hash" c acctnum goods price price (pubk hash-0))
    (enc acctnum price m price (pubk c))
    (enc c acctnum price acctnum price (pubk b)))
  (traces
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c)))))
  (label 4645)
  (parent 4620)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum acctnum-0 goods nb-0 nm price text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods) (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods)
    (price price) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 6 3 bank 2) price-0 (0 0)
    (enc "hash" c acctnum-0 goods price-0 price-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc acctnum-0 price-0 m price-0 (pubk c))
    (enc c acctnum-0 price-0 acctnum-0 price-0 (pubk b)))
  (traces
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum-0 goods (pubk c)))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c)))))
  (label 4646)
  (parent 4620)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4647)
  (parent 4622)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nm price-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) price (0 0)
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4648)
  (parent 4623)
  (seen 4614)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand bank 2) price (0 0)
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4649)
  (parent 4623)
  (unrealized (0 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test
    (contracted (acctnum-0 acctnum) (hash-1 hash-0) (nb-0 goods)) price
    (0 0) (enc "hash" c acctnum goods price price (pubk hash-0))
    (enc acctnum price m price (pubk c))
    (enc c acctnum price acctnum price (pubk b)))
  (traces
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 4650)
  (parent 4624)
  (seen 4616)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum acctnum-0 goods nb-0 nm price text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods) (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum-0 nb-0 nm)
  (operation nonce-test (displaced 6 3 bank 2) price-0 (0 0)
    (enc "hash" c acctnum-0 goods price-0 price-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc acctnum-0 price-0 m price-0 (pubk c))
    (enc c acctnum-0 price-0 acctnum-0 price-0 (pubk b)))
  (traces
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum-0 goods (pubk c))))))
  (label 4651)
  (parent 4624)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nc (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4652)
  (parent 4625)
  (seen 4585)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nc price)
  (operation nonce-test (added-strand bank 2) nc (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m))
    (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4653)
  (parent 4625)
  (unrealized (0 0) (5 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand merchant 2) nc (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m))
    (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4654)
  (parent 4625)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc nb-0 price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-0))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4655)
  (parent 4626)
  (seen 4609)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0)) ((4 1) (5 0))
    ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nm nb-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 4656)
  (parent 4627)
  (seen 4611)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (6 0)) ((1 3) (0 2))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (5 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm nb-0)
  (operation nonce-test (added-strand bank 2) nb-0 (0 0)
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 goods (pubk c)) (enc nb-0 nb-0 (pubk c))
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods (pubk m)) (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4657)
  (parent 4627)
  (unrealized (0 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nm price-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) price (0 0)
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4658)
  (parent 4628)
  (seen 4614)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand bank 2) price (0 0)
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4659)
  (parent 4628)
  (unrealized (0 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (1 0)) ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb price acctnum nm)
  (operation nonce-test
    (contracted (acctnum-0 acctnum) (hash-1 hash-0) (nb-0 goods)) price
    (0 0) (enc "hash" c acctnum goods price price (pubk hash-0))
    (enc acctnum price m price (pubk c))
    (enc c acctnum price acctnum price (pubk b)))
  (traces
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c))))
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods price price (pubk hash-0))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 4660)
  (parent 4629)
  (seen 4616)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum acctnum-0 goods nm price nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm nb-0) (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods)
    (price price) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb nb-0) (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm nb-0)
    (nb goods) (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum-0 nm nb-0)
  (operation nonce-test (displaced 6 4 bank 2) price-0 (0 0)
    (enc "hash" c acctnum-0 goods price-0 price-0 (pubk hash-1))
    (enc "hash" c acctnum-0 nb-0 price-0 price-0 (pubk hash-0))
    (enc acctnum-0 price-0 m price-0 (pubk c))
    (enc c acctnum-0 price-0 acctnum-0 price-0 (pubk b)))
  (traces
    ((recv (enc c acctnum-0 nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb nb-0 nb-0 (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nm m price (pubk c))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 nb-0 acctnum-0 nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum-0 goods (pubk c))))))
  (label 4661)
  (parent 4629)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 7 1 merchant 2) price (6 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4662)
  (parent 4631)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods text) (b m c hash name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (0 0)) ((2 4) (1 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nb nc price)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))))
  (label 4663)
  (parent 4632)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods nm price-0 text) (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (3 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4664)
  (parent 4633)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (4 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4665)
  (parent 4634)
  (seen 4668)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (6 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (0 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (5 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4666)
  (parent 4635)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4667)
  (parent 4637)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods text) (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4668)
  (parent 4638)
  (seen 4632)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods price-0 text) (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price-0) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price-0) (b b) (c c) (m m)
    (hash hash))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (0 0)) ((2 4) (1 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nb price acctnum)
  (operation generalization separated price-0)
  (traces
    ((recv (enc c acctnum price acctnum price-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price-0 (pubk c)))
      (send (enc c acctnum price acctnum price-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price-0 (pubk hash))
            (privk b)) nb))))
  (label 4669)
  (parent 4639)
  (seen 4569)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4670)
  (parent 4642)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (6 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (5 0)) ((2 2) (3 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0)) ((5 1) (6 0))
    ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 5 merchant 2) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4671)
  (parent 4642)
  (unrealized (0 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 nm price-0 nm-0 price-1 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (6 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (5 0)) ((2 0) (7 0)) ((2 2) (3 0))
    ((2 4) (1 2)) ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0))
    ((6 1) (0 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (added-strand merchant 2) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4672)
  (parent 4642)
  (unrealized (0 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 6 2 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4673)
  (parent 4644)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 text) (b m c hash name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (3 0)) ((2 2) (0 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nb price acctnum nm)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c acctnum price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum price m price (pubk c)))
      (send (enc c acctnum price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price-0 (pubk c)))))
  (label 4674)
  (parent 4645)
  (seen 4639)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc acctnum nb-0 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c)))))
  (label 4675)
  (parent 4646)
  (seen 4645)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods nm price-0 text) (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb nc price nm)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c))))))
  (label 4676)
  (parent 4647)
  (seen 4632)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 6 2 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4677)
  (parent 4649)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nb-0 nm price text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc acctnum nb-0 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 4678)
  (parent 4651)
  (seen 4650)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (5 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (displaced 6 4 bank 2) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 4679)
  (parent 4653)
  (unrealized (0 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4680)
  (parent 4653)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods) (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0)
  (operation nonce-test (displaced 6 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods price price (pubk hash-2))
            (privk b)) (enc acctnum-0 goods (pubk c))))))
  (label 4681)
  (parent 4653)
  (seen 4640)
  (unrealized (0 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (5 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (6 0)) ((2 2) (3 0)) ((2 2) (4 0))
    ((2 4) (1 2)) ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0))
    ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand merchant 2) nc (5 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4682)
  (parent 4653)
  (unrealized (0 0) (5 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nc (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4683)
  (parent 4654)
  (seen 4600)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (6 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0))
    ((2 4) (1 2)) ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0))
    ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand bank 2) nc (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)) (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4684)
  (parent 4654)
  (seen 4682 4701)
  (unrealized (0 0) (6 0))
  (comment "4 in cohort - 2 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (5 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm nb-0)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4685)
  (parent 4657)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 6 2 customer 3) price (5 0)
    (enc nc price m price (pubk c)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4686)
  (parent 4659)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum goods nm price nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum) (nm nb-0) (goods goods)
    (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc acctnum) (nm nb-0)
    (goods goods) (price nb-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc acctnum) (nm nm) (goods goods) (price price)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb nb-0)
    (price nb-0) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc acctnum) (nm nb-0) (nb goods)
    (price nb-0) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb acctnum nm nb-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
    (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
    (enc acctnum nb-0 (pubk c)) (enc acctnum nb-0 m nb-0 (pubk c))
    (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
  (traces
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (recv (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nb-0 m nb-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb nb-0 (pubk hash)) (privk m))))
    ((send (enc c acctnum goods (pubk m)))
      (recv (enc acctnum nb-0 m nb-0 (pubk c)))
      (send (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) (enc acctnum nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum nb nb-0 nb-0 (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum goods (pubk m)))
      (send (enc acctnum nm m price (pubk c))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum nb-0 nb-0 nb-0 (pubk hash-0))
            (privk b)) (enc acctnum nb-0 (pubk c)))))
    ((recv (enc c acctnum nb-0 acctnum nb-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum goods nb-0 nb-0 (pubk hash-1))
            (privk b)) (enc acctnum goods (pubk c))))))
  (label 4687)
  (parent 4661)
  (seen 4640)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nc (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)) (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c))))))
  (label 4688)
  (parent 4662)
  (seen 4652)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (6 0)) ((1 3) (0 2))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb nc price nm)
  (operation nonce-test (added-strand bank 2) nc (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc nb-0 price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc nb-0 (pubk c))
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)) (enc c nc price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb-0 price price (pubk hash-1))
            (privk b)) (enc nc nb-0 (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-2))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4689)
  (parent 4662)
  (unrealized (0 0) (6 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods price-0 text) (b m c hash name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price-0) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price-0) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price-0) (b b) (c c) (m m) (hash hash))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (0 0)) ((2 4) (1 2)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nb nc price)
  (operation generalization separated price-0)
  (traces
    ((recv (enc c nc price price price-0 (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price-0 (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price-0 (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price-0 (pubk c)))
      (send (enc c nc price price price-0 (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price-0 (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price-0 (pubk hash)) (privk b))
          nb))))
  (label 4690)
  (parent 4663)
  (seen 4569)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods nm price-0 text) (b m c hash name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (3 0)) ((2 2) (0 0)) ((2 4) (1 2)) ((3 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash))
  (uniq-orig nb nc price nm)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4691)
  (parent 4664)
  (seen 4663)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (6 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (0 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4692)
  (parent 4666)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4693)
  (parent 4667)
  (seen 4699)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4694)
  (parent 4670)
  (seen 4698)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0)) ((5 1) (6 0))
    ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm m price-0 (pubk c))
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4695)
  (parent 4671)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nb-0 nm price-0 nm-0 price-1 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 0) (7 0)) ((2 2) (3 0)) ((2 2) (6 0))
    ((2 4) (1 2)) ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0))
    ((6 1) (0 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (displaced 8 2 customer 3) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4696)
  (parent 4672)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nm price-0 nm-0 price-1 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc price-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 1) (6 0)) ((1 3) (0 2))
    ((2 0) (1 0)) ((2 0) (5 0)) ((2 0) (7 0)) ((2 2) (3 0))
    ((2 4) (1 2)) ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 5 merchant 2) nb-0 (6 0)
    (enc nb-0 price m price (pubk c)) (enc nb-0 nm-0 m price-1 (pubk c))
    (enc c nb-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4697)
  (parent 4672)
  (unrealized (0 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (3 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4698)
  (parent 4673)
  (seen 4676)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (3 0)) ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4699)
  (parent 4677)
  (seen 4668)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (5 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 4700)
  (parent 4679)
  (seen 4611)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (6 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (5 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (added-strand merchant 2) nb-0 (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 goods (pubk m))
    (enc c nb-0 price acctnum-0 price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4701)
  (parent 4679)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2)) ((3 1) (0 0))
    ((4 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price)
  (operation nonce-test (contracted (acctnum-0 acctnum)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4702)
  (parent 4680)
  (seen 4614)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (6 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (5 0)) ((5 1) (0 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (displaced 7 4 bank 2) nc (5 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4703)
  (parent 4682)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4704)
  (parent 4682)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods) (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (6 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 2) (5 0))
    ((2 4) (1 2)) ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0))
    ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (5 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods price price (pubk hash-2))
            (privk b)) (enc acctnum-0 goods (pubk c)))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c)))))
  (label 4705)
  (parent 4682)
  (seen 4675)
  (unrealized (0 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4706)
  (parent 4684)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods) (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 2) (6 0))
    ((2 4) (1 2)) ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0))
    ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods price price (pubk hash-2))
            (privk b)) (enc acctnum-0 goods (pubk c))))))
  (label 4707)
  (parent 4684)
  (seen 4678)
  (unrealized (0 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (6 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (5 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm nb-0)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4708)
  (parent 4685)
  (seen 4665)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-1))
    (enc "hash" c nc goods price price (pubk hash-0))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price price price (pubk b)))
  (traces
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-0))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-1))
            (privk b)) (enc nc price (pubk c))))))
  (label 4709)
  (parent 4686)
  (seen 4668)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm nb-0)
  (operation nonce-test (displaced 7 5 bank 2) nc (6 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 4710)
  (parent 4689)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc acctnum price acctnum-0 goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4711)
  (parent 4689)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price acctnum-0 goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc acctnum-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc acctnum-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum-0) (nb nb) (nc acctnum-0)
    (nm price) (goods goods) (price price) (b b) (c c) (m m)
    (hash hash))
  (defstrand merchant 2 (nc acctnum-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb price) (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb nb-0) (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum-0) (nc acctnum-0) (nm price)
    (nb goods) (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 2) (6 0)) ((2 4) (1 2))
    ((3 1) (1 0)) ((4 1) (0 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price acctnum-0 nm)
  (operation nonce-test (displaced 7 2 customer 3) nc (6 0)
    (enc nc price m price (pubk c)) (enc nc nm m price-0 (pubk c))
    (enc c nc goods (pubk m)))
  (traces
    ((recv (enc c acctnum-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c acctnum-0 goods (pubk m)))
      (recv (enc acctnum-0 price m price (pubk c)))
      (send (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) (enc acctnum-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c acctnum-0 goods (pubk m)))
      (send (enc acctnum-0 nm m price-0 (pubk c))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 price price price (pubk hash-0))
            (privk b)) (enc acctnum-0 price (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc acctnum-0 nb-0 (pubk c)))))
    ((recv (enc c acctnum-0 price acctnum-0 price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c acctnum-0 goods price price (pubk hash-2))
            (privk b)) (enc acctnum-0 goods (pubk c))))))
  (label 4712)
  (parent 4689)
  (seen 4687)
  (unrealized (0 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-0))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-1))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4713)
  (parent 4692)
  (seen 4664)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (6 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0)) ((5 1) (6 0))
    ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4714)
  (parent 4695)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nm-0 price-1 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 0) (7 0)) ((2 2) (3 0)) ((2 2) (6 0))
    ((2 4) (1 2)) ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0))
    ((6 1) (0 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-2))
    (enc "hash" c nb-0 goods price price (pubk hash-1))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
    (enc nb-0 price (pubk c)) (enc nb-0 price m price (pubk c))
    (enc c nb-0 price price price (pubk b)))
  (traces
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-0))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-1))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-2))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4715)
  (parent 4696)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb acctnum price goods nm price-0 nm-0 price-1 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc price-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 0) (7 0)) ((2 2) (3 0)) ((2 2) (6 0))
    ((2 4) (1 2)) ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (displaced 8 2 customer 3) price-0 (6 0)
    (enc price-0 price m price (pubk c))
    (enc price-0 nm m price-0 (pubk c))
    (enc price-0 nm-0 m price-1 (pubk c))
    (enc c price-0 goods (pubk m)))
  (traces
    ((recv (enc c price-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4716)
  (parent 4697)
  (unrealized (0 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (6 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (5 0)) ((5 1) (0 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4717)
  (parent 4701)
  (seen 4636)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (6 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (5 0)) ((5 1) (0 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nb-0 nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c)))))
  (label 4718)
  (parent 4703)
  (seen 4641)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 2) (3 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c)))))
  (label 4719)
  (parent 4704)
  (seen 4643)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4720)
  (parent 4706)
  (seen 4648)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price acctnum goods nm price-0 nb-0 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum acctnum) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb nb-0)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand bank 2 (acctnum acctnum) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-2))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (6 0)) ((6 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm nb-0)
  (operation nonce-test (contracted (acctnum-0 acctnum)) nb-0 (0 0)
    (enc "hash" c nb-0 price price price (pubk hash-0))
    (enc "hash" c nb-0 goods price price (pubk hash-2))
    (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
    (enc nb-0 price (pubk c)) (enc nb-0 goods (pubk c))
    (enc nb-0 nb-0 (pubk c)) (enc nb-0 price m price (pubk c))
    (enc nb-0 nm m price-0 (pubk c)) (enc c nb-0 goods (pubk m))
    (enc c nb-0 price acctnum price (pubk b)))
  (traces
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb-0 price price (pubk hash-1))
            (privk b)) (enc nb-0 nb-0 (pubk c)))))
    ((recv (enc c nb-0 price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-2))
            (privk b)) (enc nb-0 goods (pubk c))))))
  (label 4721)
  (parent 4710)
  (seen 4700)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb nc price acctnum goods nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum acctnum) (nc nc) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nc) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum acctnum) (nb nb) (nc nc) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nc) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum acctnum) (nc nc) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (3 0))
    ((2 2) (4 0)) ((2 2) (5 0)) ((2 4) (1 2)) ((3 1) (1 0))
    ((4 1) (0 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb nc price nm)
  (operation nonce-test (contracted (acctnum-0 acctnum)) price (0 0)
    (enc "hash" c nc price price price (pubk hash-0))
    (enc "hash" c nc goods price price (pubk hash-1))
    (enc nc price (pubk c)) (enc nc price m price (pubk c))
    (enc c nc price acctnum price (pubk b)))
  (traces
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nc goods (pubk m)))
      (recv (enc nc price m price (pubk c)))
      (send (enc c nc price acctnum price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          (enc nc nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nc nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nc goods (pubk m)))
      (send (enc nc nm m price-0 (pubk c))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc price price price (pubk hash-0))
            (privk b)) (enc nc price (pubk c)))))
    ((recv (enc c nc price acctnum price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nc goods price price (pubk hash-1))
            (privk b)) (enc nc goods (pubk c))))))
  (label 4722)
  (parent 4711)
  (seen 4702)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 2) (3 0)) ((2 2) (5 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (3 0)) ((4 1) (5 0)) ((5 1) (0 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-0))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-1))
            (privk b)) (enc nb-0 price (pubk c))))))
  (label 4723)
  (parent 4714)
  (seen 4664)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nm-0 price-1 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 0) (6 0)) ((2 2) (3 0)) ((2 2) (5 0))
    ((2 4) (1 2)) ((3 1) (0 0)) ((4 1) (3 0)) ((5 1) (0 0))
    ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 goods price price (pubk hash-0))
            (privk b)) (enc nb-0 goods (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-1))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4724)
  (parent 4715)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (b m c hash hash-0 hash-1 hash-2 name))
  (defstrand bank 3 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc price-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price-0)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-2))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (5 0)) ((2 0) (7 0)) ((2 2) (3 0)) ((2 2) (6 0))
    ((2 4) (1 2)) ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0))
    ((5 1) (6 0)) ((6 1) (0 0)) ((7 1) (6 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1) (privk hash-2))
  (uniq-orig nb price nm price-0 nm-0)
  (operation nonce-test (contracted (acctnum price)) price (0 0)
    (enc "hash" c price-0 price price price (pubk hash-2))
    (enc "hash" c price-0 goods price price (pubk hash-1))
    (enc "hash" c price-0 price-0 price price (pubk hash-0))
    (enc price-0 price (pubk c)) (enc price-0 price m price (pubk c))
    (enc c price-0 price price price (pubk b)))
  (traces
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price-0 price price (pubk hash-0))
            (privk b)) (enc price-0 price-0 (pubk c)))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-1))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-2))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4725)
  (parent 4716)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nb-0 nm price-0 nm-0 price-1 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum price) (nc nb-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc nb-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc nb-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc nb-0) (nm nm) (goods goods) (price price-0)
    (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc nb-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc nb-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (3 0)) ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (0 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb price nb-0 nm nm-0)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c nb-0 goods (pubk m)))
      (recv (enc nb-0 price m price (pubk c)))
      (send (enc c nb-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          (enc nb-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c nb-0 nb price price (pubk hash)) (privk b))
          nb)))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm m price-0 (pubk c))))
    ((recv (enc c nb-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c nb-0 price price price (pubk hash-0))
            (privk b)) (enc nb-0 price (pubk c)))))
    ((recv (enc c nb-0 goods (pubk m)))
      (send (enc nb-0 nm-0 m price-1 (pubk c)))))
  (label 4726)
  (parent 4724)
  (seen 4664)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (b m c hash hash-0 hash-1 name))
  (defstrand bank 3 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc price-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb goods)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-1))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (4 0)) ((2 0) (6 0)) ((2 2) (3 0)) ((2 2) (5 0))
    ((2 4) (1 2)) ((3 1) (0 0)) ((4 1) (3 0)) ((4 1) (5 0))
    ((5 1) (0 0)) ((6 1) (5 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0)
    (privk hash-1))
  (uniq-orig nb price nm price-0 nm-0)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 goods price price (pubk hash-0))
            (privk b)) (enc price-0 goods (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-1))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4727)
  (parent 4725)
  (realized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton epmo_acctnum
  (vars (nb price goods nm price-0 nm-0 price-1 text)
    (b m c hash hash-0 name))
  (defstrand bank 3 (acctnum price) (nc price-0) (nm price) (nb nb)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 4 (nb nb) (nc price-0) (nm price) (goods goods)
    (price price) (b b) (c c) (m m) (hash hash))
  (defstrand customer 5 (acctnum price) (nb nb) (nc price-0) (nm price)
    (goods goods) (price price) (b b) (c c) (m m) (hash hash))
  (defstrand merchant 2 (nc price-0) (nm nm) (goods goods)
    (price price-0) (c c) (m m))
  (defstrand bank 2 (acctnum price) (nc price-0) (nm price) (nb price)
    (price price) (b b) (c c) (hash hash-0))
  (defstrand merchant 2 (nc price-0) (nm nm-0) (goods goods)
    (price price-1) (c c) (m m))
  (precedes ((0 1) (2 3)) ((1 1) (2 1)) ((1 3) (0 2)) ((2 0) (1 0))
    ((2 0) (3 0)) ((2 0) (5 0)) ((2 2) (4 0)) ((2 4) (1 2))
    ((3 1) (4 0)) ((4 1) (0 0)) ((5 1) (4 0)))
  (non-orig (privk b) (privk m) (privk c) (privk hash) (privk hash-0))
  (uniq-orig nb price nm price-0 nm-0)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c))))
      (recv (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 price m price (pubk c)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb))
      (send (enc (enc "hash" b m nb price (pubk hash)) (privk m))))
    ((send (enc c price-0 goods (pubk m)))
      (recv (enc price-0 price m price (pubk c)))
      (send (enc c price-0 price price price (pubk b)))
      (recv
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) (enc price-0 nb (pubk c))))
      (send
        (cat
          (enc (enc "hash" c price-0 nb price price (pubk hash))
            (privk b)) nb)))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm m price-0 (pubk c))))
    ((recv (enc c price-0 price price price (pubk b)))
      (send
        (cat
          (enc (enc "hash" c price-0 price price price (pubk hash-0))
            (privk b)) (enc price-0 price (pubk c)))))
    ((recv (enc c price-0 goods (pubk m)))
      (send (enc price-0 nm-0 m price-1 (pubk c)))))
  (label 4728)
  (parent 4727)
  (seen 4664)
  (realized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")
